package imcacat.imcadr;

import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertNotNull;

import java.io.File;

import org.junit.Test;

/**
 * I test <code>ImcadrServerSession</code>.
 */
public class ImcadrServerSessionTest {
  /* Newly constructed session is invalid. */
  @Test
  public void testValidate01() {
    ImcadrServerSession session = createSession();
    assertNotNull(session.validate());
  }

  /* Valid. */
  @Test
  public void testValidate02() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setZxSize(new Rectangle(4.0, 2.0, LengthUnit.CELL));
    assertNull(session.validate());
  }

  /* At lest one raster size must be specified. */
  @Test
  public void testValidate03() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    assertNotNull(session.validate());
  }

  /* One raster may be omitted. */
  @Test
  public void testValidate04() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZxSize(new Rectangle(4.0, 2.0, LengthUnit.CELL));
    assertNull(session.validate());
  }

  /* Valid with chasing disabled. */
  @Test
  public void testValidate05() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setChasingEnabled(Boolean.FALSE);
    assertNull(session.validate());
  }

  /* Valid with weak diffracter. */
  @Test
  public void testValidate06() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setWeakDiffracter(Boolean.TRUE);
    assertNull(session.validate());
  }

  /* Valid with high resolution cutoff. */
  @Test
  public void testValidate07() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setWeakDiffracter(Boolean.TRUE);
    session.setHighResolutionCutoff(new Double(3.5));
    assertNull(session.validate());
  }

  /* Invalid with high resolution cutoff out of range. */
  @Test
  public void testValidate08() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(new Double(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setWeakDiffracter(Boolean.TRUE);
    session.setHighResolutionCutoff(Double.valueOf(Resources.getInstance()
        .getHighResolutionCutoffMin() - 0.1));
    assertNotNull(session.validate());
    session.setHighResolutionCutoff(Double.valueOf(Resources.getInstance()
        .getHighResolutionCutoffMax() + 0.1));
    assertNotNull(session.validate());
  }

  /* Valid with low resolution cutoff. */
  @Test
  public void testValidate09() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setLowResolutionCutoff(new Double(3.5));
    assertNull(session.validate());
  }

  /* Invalid with low resolution cutoff out of range. */
  @Test
  public void testValidate10() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setLowResolutionCutoff(Double.valueOf(Resources.getInstance()
        .getLowResolutionCutoffMin() - 0.1));
    assertNotNull(session.validate());
    session.setLowResolutionCutoff(Double.valueOf(Resources.getInstance()
        .getLowResolutionCutoffMax() + 0.1));
    assertNotNull(session.validate());
  }

  /* Valid with high resolution cutoff < low resolution cutoff. */
  @Test
  public void testValidate11() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setHighResolutionCutoff(Double.valueOf(4.9));
    session.setLowResolutionCutoff(Double.valueOf(5.0));
    assertNull(session.validate());
  }

  /* Invalid with high resolution cutoff >= low resolution cutoff. */
  @Test
  public void testValidate12() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setHighResolutionCutoff(Double.valueOf(5.0));
    session.setLowResolutionCutoff(Double.valueOf(5.0));
    assertNotNull(session.validate());
    session.setHighResolutionCutoff(Double.valueOf(5.1));
    assertNotNull(session.validate());
  }

  /* Valid with minimum spot area. */
  @Test
  public void testValidate13() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setMinimumSpotArea(Integer.valueOf(5));
    assertNull(session.validate());
  }

  /* Invalid with minimum spot area out of range. */
  @Test
  public void testValidate14() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setMinimumSpotArea(Integer
        .valueOf(Resources.getInstance().getMinimumSpotAreaMin() - 1));
    assertNotNull(session.validate());
    session.setMinimumSpotArea(Integer
        .valueOf(Resources.getInstance().getMinimumSpotAreaMax() + 1));
    assertNotNull(session.validate());
  }

  /* Valid with minimum signal height. */
  @Test
  public void testValidate15() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setMinimumSignalHeight(Double.valueOf(5.1));
    assertNull(session.validate());
  }

  /* Invalid with minimum signal height out of range. */
  @Test
  public void testValidate16() {
    ImcadrServerSession session = createSession();
    session.setImagesDirectory(getTmpDir());
    session.setExposureTime(Double.valueOf(0.5));
    session.setCellSize(Rectangle.um(35.0, 70.0));
    session.setZySize(new Rectangle(5.0, 3.0, LengthUnit.CELL));
    session.setMinimumSignalHeight(Double.valueOf(Resources.getInstance()
        .getMinimumSignalHeightMin() - 0.6));
    assertNotNull(session.validate());
    session.setMinimumSignalHeight(Double.valueOf(Resources.getInstance()
        .getMinimumSignalHeightMax() + 0.6));
    assertNotNull(session.validate());
  }

  protected static ImcadrServerSession createSession() {
    return new ImcadrServerSession(new MockBeamlineDriver());
  }

  protected static File getTmpDir() {
    return new File(System.getProperty("java.io.tmpdir"));
  }
}
