package imcacat.imcadr;

import java.io.InterruptedIOException;
import java.nio.channels.ClosedByInterruptException;

import imcacat.imcadr.util.LoggingUtils;

/**
 * I am a simulator for running this application in server mode. I delay for
 * the specified exposure time for each exposure, and I move and score
 * practically instantaneously.
 */
public class ImcadrServerSimulator {
  public static void main(String[] args) {
    try {
      disableConsoleLogging();
      runServer();
    } catch (RuntimeException e) {
      System.err.println("error: fatal exception");
      e.printStackTrace();
      basicExit(ExitStatus.FAILURE.getStatus());
    }
  }

  protected static void disableConsoleLogging() {
    LoggingUtils.disableConsoleLogging();
  }

  protected static void runServer() {
    MockImcadrServer server = createServer();
    final Thread serverThread = new Thread(server, "MockImcadrServer");

    try {
      Runtime.getRuntime().addShutdownHook(new Thread() {
        @Override
        public void run() {
          try {
            serverThread.interrupt();
            serverThread.join(MockImcadrServer.SHUTDOWN_TIMEOUT);
          } catch (InterruptedException e) {
            /* IGNORED */
          }
        }
      });
    } catch (IllegalStateException e) {
      basicExit(ExitStatus.FAILURE.getStatus());
    }

    serverThread.start();
    try {
      serverThread.join();
    } catch (InterruptedException e) {
      serverThread.interrupt();
      try {
        serverThread.join(MockImcadrServer.SHUTDOWN_TIMEOUT);
      } catch (InterruptedException ex) {
        /* IGNORED */
      }
      basicExit(ExitStatus.FAILURE.getStatus());
    }

    Exception exception = server.exception();
    if (exception == null) {
      basicExit(ExitStatus.SUCCESS.getStatus());
    } else {
      try {
        throw exception;
      } catch (InterruptedException | InterruptedIOException | ClosedByInterruptException e) {
        Thread.interrupted();
        basicExit(ExitStatus.FAILURE.getStatus());
      } catch (Exception e) {
        System.err.println("error: fatal exception");
        e.printStackTrace();
        basicExit(ExitStatus.FAILURE.getStatus());
      }
    }
  }

  protected static MockImcadrServer createServer() {
    MockBeamlineDriver driver = new MockBeamlineDriver();
    driver.setRealisticExposureTimeEnabled(true);
    return new MockImcadrServer(new Imcadr(driver), System.in, System.out);
  }

  protected static void basicExit(int exitStatus) {
    System.exit(exitStatus);
  }
}
