package imcacat.imcadr;

/**
 * I model the status of a diffraction rastering as obtained via the standard
 * input and standard output server API.
 * <p>
 * Units: progress [0-100], score [0-100], estimated resolution 1 [angstroms],
 * estimated resolution 2 [angstroms], mosaicity [degrees], unit cell [lengths:
 * angstroms; angles: degrees], position [mm], time [s].
 */
public class ImcadrServerStatus {
  private int progress = 0;
  private ImcadrServerState state = ImcadrServerState.IDLE;
  private String message = null;
  private Position2d originZy = null;
  private Position2d originZx = null;
  private Integer[][] scoresZy = null;
  private Integer[][] scoresZx = null;
  private Integer[][] spotTotalsZy = null;
  private Integer[][] spotTotalsZx = null;
  private Integer[][] inResTotalsZy = null;
  private Integer[][] inResTotalsZx = null;
  private Integer[][] goodBraggTotalsZy = null;
  private Integer[][] goodBraggTotalsZx = null;
  private Integer[][] iceRingTotalsZy = null;
  private Integer[][] iceRingTotalsZx = null;
  private Double[][] estimatedRes1Zy = null;
  private Double[][] estimatedRes1Zx = null;
  private Double[][] estimatedRes2Zy = null;
  private Double[][] estimatedRes2Zx = null;
  private Integer[][] totalIntSignalsZy = null;
  private Integer[][] totalIntSignalsZx = null;
  private Double[][] mosaicitiesZy = null;
  private Double[][] mosaicitiesZx = null;
  private SpaceGroup[][] spaceGroupsZy = null;
  private SpaceGroup[][] spaceGroupsZx = null;
  private UnitCell[][] unitCellsZy = null;
  private UnitCell[][] unitCellsZx = null;
  private Position2d centerZy = null;
  private Position2d centerZx = null;
  private Position3d center = null;
  private Integer time = null;

  public int getProgress() {
    return progress;
  }

  public void setProgress(int progress) {
    this.progress = progress;
  }

  public ImcadrServerState getState() {
    return state;
  }

  public void setState(ImcadrServerState state) {
    this.state = state;
  }

  public boolean isIdle() {
    return ImcadrServerState.IDLE.equals(state);
  }

  public boolean isRunning() {
    return ImcadrServerState.RUNNING.equals(state);
  }

  public boolean isPaused() {
    return ImcadrServerState.PAUSED.equals(state);
  }

  public boolean isRunningOrPaused() {
    return isRunning() || isPaused();
  }

  public boolean isError() {
    return ImcadrServerState.ERROR.equals(state);
  }

  public String getMessage() {
    return message;
  }

  public void setMessage(String message) {
    this.message = message;
  }

  public Position2d getOriginZy() {
    return originZy;
  }

  public void setOriginZy(Position2d originZy) {
    this.originZy = originZy;
  }

  public Position2d getOriginZx() {
    return originZx;
  }

  public void setOriginZx(Position2d originZx) {
    this.originZx = originZx;
  }

  public Integer[][] getScoresZy() {
    return scoresZy;
  }

  public void setScoresZy(Integer[][] scoresZy) {
    this.scoresZy = scoresZy;
  }

  public Integer[][] getScoresZx() {
    return scoresZx;
  }

  public void setScoresZx(Integer[][] scoresZx) {
    this.scoresZx = scoresZx;
  }

  public Integer[][] getSpotTotalsZy() {
    return spotTotalsZy;
  }

  public void setSpotTotalsZy(Integer[][] spotTotalsZy) {
    this.spotTotalsZy = spotTotalsZy;
  }

  public Integer[][] getSpotTotalsZx() {
    return spotTotalsZx;
  }

  public void setSpotTotalsZx(Integer[][] spotTotalsZx) {
    this.spotTotalsZx = spotTotalsZx;
  }

  public Integer[][] getInResolutionTotalsZy() {
    return inResTotalsZy;
  }

  public void setInResolutionTotalsZy(Integer[][] inResTotalsZy) {
    this.inResTotalsZy = inResTotalsZy;
  }

  public Integer[][] getInResolutionTotalsZx() {
    return inResTotalsZx;
  }

  public void setInResolutionTotalsZx(Integer[][] inResTotalsZx) {
    this.inResTotalsZx = inResTotalsZx;
  }

  public Integer[][] getGoodBraggTotalsZy() {
    return goodBraggTotalsZy;
  }

  public void setGoodBraggTotalsZy(Integer[][] goodBraggTotalsZy) {
    this.goodBraggTotalsZy = goodBraggTotalsZy;
  }

  public Integer[][] getGoodBraggTotalsZx() {
    return goodBraggTotalsZx;
  }

  public void setGoodBraggTotalsZx(Integer[][] goodBraggTotalsZx) {
    this.goodBraggTotalsZx = goodBraggTotalsZx;
  }

  public Integer[][] getIceRingTotalsZy() {
    return iceRingTotalsZy;
  }

  public void setIceRingTotalsZy(Integer[][] iceRingTotalsZy) {
    this.iceRingTotalsZy = iceRingTotalsZy;
  }

  public Integer[][] getIceRingTotalsZx() {
    return iceRingTotalsZx;
  }

  public void setIceRingTotalsZx(Integer[][] iceRingTotalsZx) {
    this.iceRingTotalsZx = iceRingTotalsZx;
  }

  public Double[][] getEstimatedResolutions1Zy() {
    return estimatedRes1Zy;
  }

  public void setEstimatedResolutions1Zy(Double[][] estimatedRes1Zy) {
    this.estimatedRes1Zy = estimatedRes1Zy;
  }

  public Double[][] getEstimatedResolutions1Zx() {
    return estimatedRes1Zx;
  }

  public void setEstimatedResolutions1Zx(Double[][] estimatedRes1Zx) {
    this.estimatedRes1Zx = estimatedRes1Zx;
  }

  public Double[][] getEstimatedResolutions2Zy() {
    return estimatedRes2Zy;
  }

  public void setEstimatedResolutions2Zy(Double[][] estimatedRes2Zy) {
    this.estimatedRes2Zy = estimatedRes2Zy;
  }

  public Double[][] getEstimatedResolutions2Zx() {
    return estimatedRes2Zx;
  }

  public void setEstimatedResolutions2Zx(Double[][] estimatedRes2Zx) {
    this.estimatedRes2Zx = estimatedRes2Zx;
  }

  public Integer[][] getTotalIntegratedSignalsZy() {
    return totalIntSignalsZy;
  }

  public void setTotalIntegratedSignalsZy(Integer[][] totalIntSignalsZy) {
    this.totalIntSignalsZy = totalIntSignalsZy;
  }

  public Integer[][] getTotalIntegratedSignalsZx() {
    return totalIntSignalsZx;
  }

  public void setTotalIntegratedSignalsZx(Integer[][] totalIntSignalsZx) {
    this.totalIntSignalsZx = totalIntSignalsZx;
  }

  public Double[][] getMosaicitiesZy() {
    return mosaicitiesZy;
  }

  public void setMosaicitiesZy(Double[][] mosaicitiesZy) {
    this.mosaicitiesZy = mosaicitiesZy;
  }

  public Double[][] getMosaicitiesZx() {
    return mosaicitiesZx;
  }

  public void setMosaicitiesZx(Double[][] mosaicitiesZx) {
    this.mosaicitiesZx = mosaicitiesZx;
  }

  public SpaceGroup[][] getSpaceGroupsZy() {
    return spaceGroupsZy;
  }

  public void setSpaceGroupsZy(SpaceGroup[][] spaceGroupsZy) {
    this.spaceGroupsZy = spaceGroupsZy;
  }

  public SpaceGroup[][] getSpaceGroupsZx() {
    return spaceGroupsZx;
  }

  public void setSpaceGroupsZx(SpaceGroup[][] spaceGroupsZx) {
    this.spaceGroupsZx = spaceGroupsZx;
  }

  public UnitCell[][] getUnitCellsZy() {
    return unitCellsZy;
  }

  public void setUnitCellsZy(UnitCell[][] unitCellsZy) {
    this.unitCellsZy = unitCellsZy;
  }

  public UnitCell[][] getUnitCellsZx() {
    return unitCellsZx;
  }

  public void setUnitCellsZx(UnitCell[][] unitCellsZx) {
    this.unitCellsZx = unitCellsZx;
  }

  public Position2d getCenterZy() {
    return centerZy;
  }

  public void setCenterZy(Position2d centerZy) {
    this.centerZy = centerZy;
  }

  public Position2d getCenterZx() {
    return centerZx;
  }

  public void setCenterZx(Position2d centerZx) {
    this.centerZx = centerZx;
  }

  public Position3d getCenter() {
    return center;
  }

  public void setCenter(Position3d center) {
    this.center = center;
  }

  public Integer getTime() {
    return time;
  }

  public void setTime(Integer time) {
    this.time = time;
  }
}
