package imcacat.imcadr;

import static org.junit.Assert.assertArrayEquals;

import java.io.IOException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * I test <code>MockImcadrServerSession</code>.
 */
public class MockImcadrServerSessionTest {
  private static final Range<Integer> SCORE_RANGE = new Range<Integer>(0, 100);

  @Before
  public void setUp() throws IOException {
    MockImcadrServerSession.setScoreCreationRange(SCORE_RANGE);
  }

  @After
  public void tearDown() {
    MockImcadrServerSession.setScoreCreationRange(null);
  }

  /* 1x1. */
  @Test
  public void testCreateScore01() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {100}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 2x1. */
  @Test
  public void testCreateScore02() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {100, 100}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 1x2. */
  @Test
  public void testCreateScore03() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {100},
      {100}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 2x2. */
  @Test
  public void testCreateScore04() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {100, 100},
      {100, 100}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 3x2. */
  @Test
  public void testCreateScore05() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {0, 100, 0},
      {0, 100, 0}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 2x3. */
  @Test
  public void testCreateScore06() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {  0,   0},
      {100, 100},
      {  0,   0},
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 3x3. */
  @Test
  public void testCreateScore07() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {0,   0, 0},
      {0, 100, 0},
      {0,   0, 0},
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 3x1. */
  @Test
  public void testCreateScore08() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {0, 100, 0}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 1x3. */
  @Test
  public void testCreateScore09() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {  0},
      {100},
      {  0}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 4x1. */
  @Test
  public void testCreateScore10() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {0, 50, 100, 0}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }

  /* 1x4. */
  @Test
  public void testCreateScore11() {
    /* @formatter:off */
    int[][] expected = new int[][] {
      {  0},
      { 50},
      {100},
      {  0}
    };
    /* @formatter:on */
    int maxRow = expected.length - 1;
    int maxCol = expected[0].length - 1;
    int[][] actual = new int[expected.length][expected[0].length];
    for (int r = 0; r < expected.length; r++) {
      for (int c = 0; c < expected[0].length; c++) {
        actual[r][c] = MockImcadrServerSession.createScore(c, r, maxCol, maxRow, SCORE_RANGE);
      }
    }
    assertArrayEquals(expected, actual);
  }
}
