package imcacat.imcadr;

import imcacat.imcadr.util.MathUtils;

/**
 * I am a Mock object of <tt>RasterCellAnalyzer</tt>.
 */
public class MockRasterCellAnalyzer extends RasterCellAnalyzer {
  private Integer score;

  public MockRasterCellAnalyzer(Raster2dIndex index, Raster2d raster, Integer score) {
    super(index, raster);
    this.score = score;
  }

  @Override
  protected void waitForImageToExist() throws InterruptedException {
  }

  @Override
  protected ImageAnalysisResult analyzeImage() {
    final AnalysisPackage pkg = getAnalysisSpec().getAnalysisPackage();
    ImageAnalysisResult result = new ImageAnalysisResult();
    result.setScore(getScore());
    result.setEstimatedResolution1(createEstimatedResolution());
    switch (pkg) {
      case LABELIT:
        result.setSpotTotal(createSpotTotal(1));
        result.setInResolutionTotal(createInResolutionTotal(2));
        result.setGoodBraggTotal(createGoodBraggTotal(3));
        result.setIceRingTotal(createIceRingTotal(4));
        result.setTotalIntegratedSignal(createTotalIntegratedSignal(5));
        break;
      case DTREK:
        result.setMosaicity(createMosaicity());
        result.setSpaceGroup(createSpaceGroup());
        result.setUnitCell(createUnitCell());
        break;
    }
    return result;
  }

  private Integer getScore() {
    return score;
  }

  private int createSpotTotal(int salt) {
    return column + row + salt;
  }

  private int createInResolutionTotal(int salt) {
    return column + row + salt;
  }

  private int createGoodBraggTotal(int salt) {
    return column + row + salt;
  }

  private int createIceRingTotal(int salt) {
    return column + row + salt;
  }

  private double createEstimatedResolution() {
    final double columnRowSumMax = (double)(raster.columnCount() + raster.rowCount() - 2);
    return MathUtils.scale((double)(column + row), 0.0, columnRowSumMax, Resources.getInstance()
        .getHighResolutionCutoffMin(), Resources.getInstance().getHighResolutionCutoffMax());
  }

  private int createTotalIntegratedSignal(int salt) {
    return column + row + salt;
  }

  private double createMosaicity() {
    final double columnRowSumMax = (double)(raster.columnCount() + raster.rowCount() - 2);
    return MathUtils.scale((double)(column + row), 0.0, columnRowSumMax, 0.0, 10.0);
  }

  private SpaceGroup createSpaceGroup() {
    final int columnRowSumMax = raster.columnCount() + raster.rowCount() - 2;
    final SpaceGroup[] groups = SpaceGroup.values();
    return groups[MathUtils.scale(column + row, 0, columnRowSumMax, 0, groups.length - 1)];
  }

  private UnitCell createUnitCell() {
    final double columnRowSumMax = (double)(raster.columnCount() + raster.rowCount() - 2);
    final double length =
        MathUtils.scale((double)(column + row), 0.0, columnRowSumMax, 0.1, 0.96);
    final double angle =
        MathUtils.scale((double)(column + row), 0.0, columnRowSumMax, 0.1, 179.96);
    /* @formatter:off */
    return new UnitCell(new double[] {
      length + 0.01,
      length + 0.02,
      length + 0.03
    },
      new double[] {
        angle + 0.01,
        angle + 0.02,
        angle + 0.03
      });
    /* @formatter:on */
  }
}
