package imcacat.imcadr;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.io.File;
import java.util.NoSuchElementException;

import org.junit.Test;

/**
 * I test <code>Raster2dIterator</code>.
 */
public class Raster2dAllIteratorTest {
  @Test
  public void testNext_rowMajorWithNumRowsGtNumCols() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.3, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(1,0),
      new Raster2dIndex(2,0),

      new Raster2dIndex(2,1),
      new Raster2dIndex(1,1),
      new Raster2dIndex(0,1),

      new Raster2dIndex(0,2),
      new Raster2dIndex(1,2),
      new Raster2dIndex(2,2),

      new Raster2dIndex(2,3),
      new Raster2dIndex(1,3),
      new Raster2dIndex(0,3)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_rowMajorWithNumColsGtNumRows() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.5, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(1,0),
      new Raster2dIndex(2,0),
      new Raster2dIndex(3,0),

      new Raster2dIndex(3,1),
      new Raster2dIndex(2,1),
      new Raster2dIndex(1,1),
      new Raster2dIndex(0,1),

      new Raster2dIndex(0,2),
      new Raster2dIndex(1,2),
      new Raster2dIndex(2,2),
      new Raster2dIndex(3,2)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_colMajorWithNumColsGtNumRows() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.3, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(0,1),
      new Raster2dIndex(0,2),

      new Raster2dIndex(1,2),
      new Raster2dIndex(1,1),
      new Raster2dIndex(1,0),

      new Raster2dIndex(2,0),
      new Raster2dIndex(2,1),
      new Raster2dIndex(2,2),

      new Raster2dIndex(3,2),
      new Raster2dIndex(3,1),
      new Raster2dIndex(3,0)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_colMajorWithNumRowsGtNumCols() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.5, 0.3, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(0,1),
      new Raster2dIndex(0,2),
      new Raster2dIndex(0,3),

      new Raster2dIndex(1,3),
      new Raster2dIndex(1,2),
      new Raster2dIndex(1,1),
      new Raster2dIndex(1,0),

      new Raster2dIndex(2,0),
      new Raster2dIndex(2,1),
      new Raster2dIndex(2,2),
      new Raster2dIndex(2,3)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_verticalStripe() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(0,1),
      new Raster2dIndex(0,2)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_horizontalStripe() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(1,0),
      new Raster2dIndex(2,0)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_oneCell() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0)
    };
    /* @formatter:on */
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test(expected = NoSuchElementException.class)
  public void testNext_whenDoesNotHaveNext() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    Raster2dIterator harness = new Raster2dAllIterator(raster);
    harness.next();
    harness.next();
  }
}
