package imcacat.imcadr;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.io.File;

import org.junit.Test;

/**
 * I test <code>Raster2dEdgeFlyScanIterator</code>.
 */
public class Raster2dEdgeFlyScanIteratorTest {
  @Test
  public void testTop() {
    Raster2d raster = createRaster(4, 3);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTop_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(2, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTop_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 3);
    raster.getCell(0, 0).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTop_onlyOneUnmasked() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 1, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(2, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(2, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopRight() {
    Raster2d raster = createRaster(4, 4);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopRight_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(2, 0), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopRight_maskedOnRight() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(2, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopRight_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 4);
    raster.getCell(0, 0).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopRight_onlyOneUnmaskedOnEachComponentEdge() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 1, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(2, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(2, 0), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(3, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testRight() {
    Raster2d raster = createRaster(4, 4);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testRight_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testRight_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 4);
    raster.getCell(3, 0).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomRight() {
    Raster2d raster = createRaster(4, 4);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 3), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(2, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomRight_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 2), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(2, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(1, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomRight_maskedOnBottom() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 1, 1, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomRight_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 4);
    raster.getCell(3, 0).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_RIGHT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 1), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 3), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(2, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottom() {
    Raster2d raster = createRaster(4, 3);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottom_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(2, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(1, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottom_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 3);
    raster.getCell(3, 2).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(2, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 2), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomLeft() {
    Raster2d raster = createRaster(4, 4);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 3), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(0, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomLeft_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 1}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(2, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(1, 3), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(0, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 1), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomLeft_maskedOnLeft() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(1, 3), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testBottomLeft_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 4);
    raster.getCell(0, 3).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.BOTTOM_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(3, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(1, 3), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(0, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testLeft() {
    Raster2d raster = createRaster(4, 4);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testLeft_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 1), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testLeft_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 4);
    raster.getCell(0, 3).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopLeft() {
    Raster2d raster = createRaster(4, 4);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 0), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopLeft_maskedOnBothEnds() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 2), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 1), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(2, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopLeft_maskedOnLeft() {
    /* @formatter:off */
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(maskArray);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  @Test
  public void testTopLeft_scoredOnOneEnd() {
    Raster2d raster = createRaster(4, 4);
    raster.getCell(0, 0).setScore(23);
    Raster2dEdgeFlyScanIterator iter = newIterator(raster, Raster2dEdge.TOP_LEFT);
    Raster2dFlyScan scan = iter.next();
    assertEquals(new Raster2dIndex(0, 3), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(0, 1), scan.endCell().getIndex());
    scan = iter.next();
    assertEquals(new Raster2dIndex(1, 0), scan.startCell().getIndex());
    assertEquals(new Raster2dIndex(3, 0), scan.endCell().getIndex());
    assertFalse(iter.hasNext());
  }

  private static Raster2dEdgeFlyScanIterator newIterator(Raster2d raster, Raster2dEdge edge) {
    return newIterator(raster, edge, null);
  }

  private static Raster2dEdgeFlyScanIterator newIterator(Raster2d raster, Raster2dEdge edge,
      Raster2dIndex edgeIndex) {
    return new Raster2dEdgeFlyScanIterator(raster, edge, edgeIndex);
  }

  private static Raster2d createRaster(int numCols, int numRows) {
    return new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, numCols, numRows, 0.3, 0.2,
        LengthUnit.MILLIMETER, 500, 2, true, true, 25, new File("/nonexistent"), null), ".cbf",
        false);
  }

  private static Raster2d createRaster(int[][] maskArray) {
    return new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length, maskArray.length,
        null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER, 500L, 2, true, true, null,
        new File("/nonexistent"), null), ".cbf", false);
  }

  /**
   * @param maskArray [row][column] mask array where nonzero is {@code true}
   */
  private static Raster2dMask newMask(int[][] maskArray) {
    return TestUtils.newMask(maskArray);
  }
}
