package imcacat.imcadr;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.io.File;
import java.util.NoSuchElementException;

import org.junit.Test;

/**
 * I test <code>Raster2dEdgeIterator</code>.
 */
public class Raster2dEdgeIteratorTest {
  @Test
  public void testNext01() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(1,0),
      new Raster2dIndex(2,0),
      new Raster2dIndex(3,0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.TOP);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext02() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0),
      new Raster2dIndex(1,0),
      new Raster2dIndex(2,0),
      new Raster2dIndex(3,0),
      new Raster2dIndex(3,1),
      new Raster2dIndex(3,2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.TOP_RIGHT);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext03() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(3,0),
      new Raster2dIndex(3,1),
      new Raster2dIndex(3,2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.RIGHT);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext04() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(3,0),
      new Raster2dIndex(3,1),
      new Raster2dIndex(3,2),
      new Raster2dIndex(2,2),
      new Raster2dIndex(1,2),
      new Raster2dIndex(0,2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.BOTTOM_RIGHT);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext05() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(3,2),
      new Raster2dIndex(2,2),
      new Raster2dIndex(1,2),
      new Raster2dIndex(0,2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.BOTTOM);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext06() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(3,2),
      new Raster2dIndex(2,2),
      new Raster2dIndex(1,2),
      new Raster2dIndex(0,2),
      new Raster2dIndex(0,1),
      new Raster2dIndex(0,0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.BOTTOM_LEFT);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext07() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,2),
      new Raster2dIndex(0,1),
      new Raster2dIndex(0,0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.LEFT);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext08() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,2),
      new Raster2dIndex(0,1),
      new Raster2dIndex(0,0),
      new Raster2dIndex(1,0),
      new Raster2dIndex(2,0),
      new Raster2dIndex(3,0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.TOP_LEFT);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext09() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0,0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.TOP);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test(expected = NoSuchElementException.class)
  public void testNext10() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.TOP);
    harness.next();
    harness.next();
  }

  @Test
  public void testNext_allEdgeOn1x1() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn1x1WithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.ALL, new Raster2dIndex(0, 0));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn4x3() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0),
      new Raster2dIndex(1, 0),
      new Raster2dIndex(2, 0),
      new Raster2dIndex(3, 0),
      new Raster2dIndex(3, 1),
      new Raster2dIndex(3, 2),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(1, 2),
      new Raster2dIndex(0, 2),
      new Raster2dIndex(0, 1)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn4x3WithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(1, 1)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.ALL, new Raster2dIndex(1, 1));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn4x1() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0),
      new Raster2dIndex(1, 0),
      new Raster2dIndex(2, 0),
      new Raster2dIndex(3, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn4x1WithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(1, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.ALL, new Raster2dIndex(1, 0));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn1x4() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0),
      new Raster2dIndex(0, 1),
      new Raster2dIndex(0, 2),
      new Raster2dIndex(0, 3)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn1x4WithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 1)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.ALL, new Raster2dIndex(0, 1));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn2x1() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 2, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0),
      new Raster2dIndex(1, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn1x2() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 2, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0),
      new Raster2dIndex(0, 1)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_allEdgeOn2x2() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 2, 2, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 0),
      new Raster2dIndex(1, 0),
      new Raster2dIndex(1, 1),
      new Raster2dIndex(0, 1)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness = new Raster2dEdgeIterator(raster, Raster2dEdge.ALL);
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_topEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 1),
      new Raster2dIndex(1, 1),
      new Raster2dIndex(2, 1),
      new Raster2dIndex(3, 1)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.TOP, new Raster2dIndex(1, 1));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_topRightEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(0, 1),
      new Raster2dIndex(1, 1),
      new Raster2dIndex(2, 1),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(2, 3)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.TOP_RIGHT, new Raster2dIndex(2, 1));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_rightEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(2, 0),
      new Raster2dIndex(2, 1),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(2, 3)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.RIGHT, new Raster2dIndex(2, 1));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_bottomRightEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(2, 0),
      new Raster2dIndex(2, 1),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(1, 2),
      new Raster2dIndex(0, 2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.BOTTOM_RIGHT, new Raster2dIndex(2, 2));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_bottomEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(4, 2),
      new Raster2dIndex(3, 2),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(1, 2),
      new Raster2dIndex(0, 2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.BOTTOM, new Raster2dIndex(2, 2));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_bottomLeftEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(4, 2),
      new Raster2dIndex(3, 2),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(2, 1),
      new Raster2dIndex(2, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.BOTTOM_LEFT, new Raster2dIndex(2, 2));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_leftEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(2, 3),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(2, 1),
      new Raster2dIndex(2, 0)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.LEFT, new Raster2dIndex(2, 2));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }

  @Test
  public void testNext_topLeftEdgeWithIndex() {
    Raster2d raster =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    Raster2dIndex[] indices = new Raster2dIndex[] {
      new Raster2dIndex(2, 3),
      new Raster2dIndex(2, 2),
      new Raster2dIndex(3, 2),
      new Raster2dIndex(4, 2)
    };
    /* @formatter:on */
    Raster2dEdgeIterator harness =
        new Raster2dEdgeIterator(raster, Raster2dEdge.TOP_LEFT, new Raster2dIndex(2, 2));
    for (int i = 0; i < indices.length; i++) {
      RasterCell cell = harness.next();
      assertEquals(indices[i].getColumn(), cell.getColumn());
      assertEquals(indices[i].getRow(), cell.getRow());
    }
    assertFalse(harness.hasNext());
  }
}
