package imcacat.imcadr;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

/**
 * I test <code>Raster2dFlyScanCalculator</code>.
 */
public class Raster2dFlyScanCalculatorTest {
  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_zeroCellLength() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 0.0, 0.25, 0.0, 0.0, 0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_zeroReqExposureTime() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeMinVelo() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, -1.0, 0.0, 0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeMaxVelo() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 0.0, -1.0, 0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeAccelTime() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 0.0, 0.0, -0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeDetectorReadoutTime() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 0.0, 0.0, -0.25, -1.0, 0.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeShutterLatency() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 0.0, 0.0, 0.25, 0.0, -1.0, 0.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeDetectorLatency() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 0.0, 0.0, 0.25, 0.0, 0.0, -1.0, 0.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_negativeCvSettleTime() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 0.0, 0.0, 0.25, 0.0, 0.0, 0.0, -1.0);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor_minVeloGreaterThanMaxVelo() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 2.0, 1.0, 0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test
  public void testConstructor_minVeloGreaterThanMaxVeloWhenMaxVeloZero() {
    new Raster2dFlyScanCalculator(0.0, 0.0, 1.0, 0.25, 2.0, 0.0, 0.25, 0.0, 0.0, 0.0, 0.0);
  }

  @Test
  public void testPositiveMove() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(0.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(7.0, c.shutterClosePosition(), 1e-6);
    assertEquals(0.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-1.0, c.startPosition(), 1e-6);
    assertEquals(8.0, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testNegativeMove() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(6.5, 0.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(7.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(0.0, c.shutterClosePosition(), 1e-6);
    assertEquals(7.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(8.0, c.startPosition(), 1e-6);
    assertEquals(-1.0, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testNonzeroShutterLatency() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(-1.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(6.0, c.shutterClosePosition(), 1e-6);
    assertEquals(0.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-2.0, c.startPosition(), 1e-6);
    assertEquals(8.0, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testNonzeroDetectorLatency() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(0.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(7.0, c.shutterClosePosition(), 1e-6);
    assertEquals(-1.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-2.0, c.startPosition(), 1e-6);
    assertEquals(8.0, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testNonzeroCvSettleTime() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(0.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(7.0, c.shutterClosePosition(), 1e-6);
    assertEquals(0.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-2.0, c.startPosition(), 1e-6);
    assertEquals(9.0, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testShutterLatencyGtCvSettleTimeGtDetectorLatency() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 2.0, 1.0, 1.5);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(-2.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(5.0, c.shutterClosePosition(), 1e-6);
    assertEquals(-1.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-3.0, c.startPosition(), 1e-6);
    assertEquals(9.5, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testShutterLatencyGtDetectorLatencyGtCvSettleTime() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 2.0, 1.5, 1.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(-2.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(5.0, c.shutterClosePosition(), 1e-6);
    assertEquals(-1.5, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-3.0, c.startPosition(), 1e-6);
    assertEquals(9.0, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testDetectorLatencyGtCvSettleTimeGtShutterLatency() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 1.0, 2.0, 1.5);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(-1.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(6.0, c.shutterClosePosition(), 1e-6);
    assertEquals(-2.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-3.0, c.startPosition(), 1e-6);
    assertEquals(9.5, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testDetectorLatencyGtShutterLatencyGtCvSettleTime() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 6.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 1.0, 2.0, 0.5);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(-1.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(6.0, c.shutterClosePosition(), 1e-6);
    assertEquals(-2.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-3.0, c.startPosition(), 1e-6);
    assertEquals(8.5, c.endPosition(), 1e-6);
    assertEquals(7, c.cellCount());
  }

  @Test
  public void testOneCell() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 0.5, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0);
    assertEquals(1.0, c.exposureTime(), 1e-6);
    assertEquals(1.0, c.velocity(), 1e-6);
    assertEquals(0.0, c.shutterOpenPosition(), 1e-6);
    assertEquals(1.0, c.shutterClosePosition(), 1e-6);
    assertEquals(0.0, c.detectorTriggerPosition(), 1e-6);
    assertEquals(-1.0, c.startPosition(), 1e-6);
    assertEquals(2.0, c.endPosition(), 1e-6);
    assertEquals(1, c.cellCount());
  }

  @Test
  public void testAdjustedDueToMinVelo() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 0.5, 1.0, 1.0, 2.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0);
    assertEquals(0.5, c.exposureTime(), 1e-6);
    assertEquals(2.0, c.velocity(), 1e-6);
  }

  @Test
  public void testAdjustedDueToMaxVelo() {
    Raster2dFlyScanCalculator c =
        new Raster2dFlyScanCalculator(0.5, 0.5, 1.0, 0.25, 0.0, 2.0, 1.0, 0.0, 0.0, 0.0, 0.0);
    assertEquals(0.5, c.exposureTime(), 1e-6);
    assertEquals(2.0, c.velocity(), 1e-6);
  }
}
