package imcacat.imcadr;

import static org.junit.Assert.assertEquals;

import java.io.File;

import org.junit.Test;

/**
 * I test <code>Raster2dPositionCalculator</code>.
 */
public class Raster2dPositionCalculatorTest {
  @Test
  public void testCenter01() {
    Raster2dSpec spec =
        new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER, 500, 2,
            new File("/nonexistent"));
    Raster2dPositionCalculator calc = new Raster2dPositionCalculator(spec);
    calc.center(0.0, 0.0);

    assertEquals(0, calc.centerHorizPos());
    assertEquals(0, calc.centerVertPos());
    assertEquals(0.0, calc.unscaledCenterHorizPos(), 1e-6);
    assertEquals(0.0, calc.unscaledCenterVertPos(), 1e-6);

    assertEquals(-450000, calc.topLeftHorizPos());
    assertEquals(-400000, calc.topLeftVertPos());
    assertEquals(-0.45, calc.unscaledTopLeftHorizPos(), 1e-6);
    assertEquals(-0.4, calc.unscaledTopLeftVertPos(), 1e-6);

    assertEquals(-300000, calc.topLeftCellHorizPos());
    assertEquals(-300000, calc.topLeftCellVertPos());
    assertEquals(-0.3, calc.unscaledTopLeftCellHorizPos(), 1e-6);
    assertEquals(-0.3, calc.unscaledTopLeftCellVertPos(), 1e-6);
  }

  @Test
  public void testCenter02() {
    Raster2dSpec spec =
        new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER, 500, 2,
            new File("/nonexistent"));
    Raster2dPositionCalculator calc = new Raster2dPositionCalculator(spec);
    calc.topLeft(-0.45, -0.4);

    assertEquals(0, calc.centerHorizPos());
    assertEquals(0, calc.centerVertPos());
    assertEquals(0.0, calc.unscaledCenterHorizPos(), 1e-6);
    assertEquals(0.0, calc.unscaledCenterVertPos(), 1e-6);

    assertEquals(-450000, calc.topLeftHorizPos());
    assertEquals(-400000, calc.topLeftVertPos());
    assertEquals(-0.45, calc.unscaledTopLeftHorizPos(), 1e-6);
    assertEquals(-0.4, calc.unscaledTopLeftVertPos(), 1e-6);

    assertEquals(-300000, calc.topLeftCellHorizPos());
    assertEquals(-300000, calc.topLeftCellVertPos());
    assertEquals(-0.3, calc.unscaledTopLeftCellHorizPos(), 1e-6);
    assertEquals(-0.3, calc.unscaledTopLeftCellVertPos(), 1e-6);
  }

  @Test
  public void testCenter03() {
    Raster2dSpec spec =
        new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER, 500, 2,
            new File("/nonexistent"));
    Raster2dPositionCalculator calc = new Raster2dPositionCalculator(spec);
    calc.topLeftCell(-0.3, -0.3);

    assertEquals(0, calc.centerHorizPos());
    assertEquals(0, calc.centerVertPos());
    assertEquals(0.0, calc.unscaledCenterHorizPos(), 1e-6);
    assertEquals(0.0, calc.unscaledCenterVertPos(), 1e-6);

    assertEquals(-450000, calc.topLeftHorizPos());
    assertEquals(-400000, calc.topLeftVertPos());
    assertEquals(-0.45, calc.unscaledTopLeftHorizPos(), 1e-6);
    assertEquals(-0.4, calc.unscaledTopLeftVertPos(), 1e-6);

    assertEquals(-300000, calc.topLeftCellHorizPos());
    assertEquals(-300000, calc.topLeftCellVertPos());
    assertEquals(-0.3, calc.unscaledTopLeftCellHorizPos(), 1e-6);
    assertEquals(-0.3, calc.unscaledTopLeftCellVertPos(), 1e-6);
  }
}
