package imcacat.imcadr;

import java.io.File;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;

/**
 * I test <code>Raster2d</code>.
 */
public class Raster2dTest {
  @Test
  public void testComputeBestPositionIndex01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    int[][] scores = new int[][] {
      { 0, 25, 50},
      {90, 30, 55},
      {95, 35, 60}
    };
    /* @formatter:on */
    for (int r = 0; r < scores.length; r++)
      for (int c = 0; c < scores[0].length; c++)
        setScore(harness, c, r, scores[r][c]);
    assertEquals(new Raster2dIndex(0, 2), harness.computeBestPositionIndex());
  }

  @Test
  public void testComputeBestPositionIndex02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    int[][] scores = new int[][] {
      {20, 20, 20},
      {40, 40, 40},
      {40, 60, 60},
      {40, 60, 80}
    };
    /* @formatter:on */
    for (int r = 0; r < scores.length; r++)
      for (int c = 0; c < scores[0].length; c++)
        setScore(harness, c, r, scores[r][c]);
    assertEquals(new Raster2dIndex(2, 3), harness.computeBestPositionIndex());
  }

  @Test
  public void testComputeBestPositionIndex03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    int[][] scores = new int[][] {
      { 0},
      {90},
      {95}
    };
    /* @formatter:on */
    for (int r = 0; r < scores.length; r++)
      for (int c = 0; c < scores[0].length; c++)
        setScore(harness, c, r, scores[r][c]);
    assertEquals(new Raster2dIndex(0, 2), harness.computeBestPositionIndex());
  }

  @Test
  public void testComputeBestPositionIndex04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 90, 95}
    };
    /* @formatter:on */
    for (int r = 0; r < scores.length; r++)
      for (int c = 0; c < scores[0].length; c++)
        setScore(harness, c, r, scores[r][c]);
    assertEquals(new Raster2dIndex(2, 0), harness.computeBestPositionIndex());
  }

  /* Weighted best position: one row; one non-zero. */
  @Test
  public void testComputeBestPosition01() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 40, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: one row; two same score. */
  @Test
  public void testComputeBestPosition02() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 40, 40, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.2, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: one row; half-strength score to right. */
  @Test
  public void testComputeBestPosition03() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 40, 20, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.4 * (20.0 / 60.0), bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: one row; half-strength score on either side. */
  @Test
  public void testComputeBestPosition04() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 20, 40, 20, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: one row; half-strength score on one side,
   * quarter-strength on other.
   */
  @Test
  public void testComputeBestPosition05() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {39, 20, 40, 10, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals((-0.4 * (20.0 / 70.0)) + (0.4 * (10.0 / 70.0)), bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows. */
  @Test
  public void testComputeBestPosition06() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0,  0, 0, 0},
      {0, 0, 40, 0, 0},
      {0, 0,  0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; half-strength above. */
  @Test
  public void testComputeBestPosition07() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 20, 0, 0},
      {0, 0, 40, 0, 0},
      {0, 0,  0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (20.0 / 60.0), bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; half-strength above and below. */
  @Test
  public void testComputeBestPosition08() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 20, 0, 0},
      {0, 0, 40, 0, 0},
      {0, 0, 20, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; half-strength above; quarter-strength
   * below.
   */
  @Test
  public void testComputeBestPosition09() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 20, 0, 0},
      {0, 0, 40, 0, 0},
      {0, 0, 10, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals((-0.2 * (20.0 / 70.0)) + (0.2 * (10.0 / 70.0)), bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; half-strength score on top and side.
   */
  @Test
  public void testComputeBestPosition10() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0,  0, 20, 0, 0},
      {0, 20, 40, 0, 0},
      {0,  0,  0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.4 * (20.0 / 80.0), bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (20.0 / 80.0), bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; half-strength score on top-left, top,
   * and top-right.
   */
  @Test
  public void testComputeBestPosition11() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 20, 20, 20, 0},
      {0,  0, 40,  0, 0},
      {0,  0,  0,  0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (60.0 / 100.0), bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; almost full-strength score on
   * top-left, top, and top-right.
   */
  @Test
  public void testComputeBestPosition12() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 399, 399, 399, 0},
      {0,   0, 400,   0, 0},
      {0,   0,   0,   0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (119.7 / 159.7), bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; almost full-strength score on top. */
  @Test
  public void testComputeBestPosition13() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0,  0, 399, 0, 0},
      {0,  0, 400, 0, 0},
      {0,  0,   0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (399.0 / 799.0), bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; almost full-strength all around. */
  @Test
  public void testComputeBestPosition14() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0,  399, 399, 399, 0},
      {0,  399, 400, 399, 0},
      {0,  399, 399, 399, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; half-strength on diagonal. */
  @Test
  public void testComputeBestPosition15() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0,  20,  0,  0, 0},
      {0,   0, 40,  0, 0},
      {0,   0,  0, 20, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; half-strength top-left. */
  @Test
  public void testComputeBestPosition16() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0,  20,  0, 0, 0},
      {0,   0, 40, 0, 0},
      {0,   0,  0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.4 * (20.0 / 60.0), bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (20.0 / 60.0), bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; almost full-strength top-left. */
  @Test
  public void testComputeBestPosition17() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0,  399,   0, 0, 0},
      {0,    0, 400, 0, 0},
      {0,    0,   0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.4 * (399.0 / 799.0), bestPosition.getX(), 1e-6);
    assertEquals(-0.2 * (399.0 / 799.0), bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: three rows; best index on edge. */
  @Test
  public void testComputeBestPosition18() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 40, 0, 0},
      {0, 0,  0, 0, 0},
      {0, 0,  0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; best index on top edge; should not do
   * averaging.
   */
  @Test
  public void testComputeBestPosition19() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 400, 0, 0},
      {0, 0, 399, 0, 0},
      {0, 0,   0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; best index on top and right edges;
   * should not do averaging.
   */
  @Test
  public void testComputeBestPosition20() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 0, 30, 40},
      {0, 0, 0, 20, 30},
      {0, 0, 0,  0,  0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.8, bestPosition.getX(), 1e-6);
    assertEquals(-0.2, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: two rows; best index on edge; should not do
   * averaging.
   */
  @Test
  public void testComputeBestPosition21() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 0, 40, 30},
      {0, 0, 0, 20, 30}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.4, bestPosition.getX(), 1e-6);
    assertEquals(-0.1, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: three rows; best index on edge; should not do
   * averaging.
   */
  @Test
  public void testComputeBestPosition22() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 0, 40, 30},
      {0, 0, 0, 30, 30},
      {0, 0, 0, 30, 30}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.4, bestPosition.getX(), 1e-6);
    assertEquals(-0.2, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: raster size smaller than 3x3. */
  @Test
  public void testComputeBestPosition23() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {40, 30},
      {10, 20}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.2, bestPosition.getX(), 1e-6);
    assertEquals(-0.1, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: raster size is 1x1. */
  @Test
  public void testComputeBestPosition24() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {40}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: raster size is 2x1; should not average. */
  @Test
  public void testComputeBestPosition25() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {40, 20}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.2, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: raster size is 3x1; but best index is on edge so
   * should not average.
   */
  @Test
  public void testComputeBestPosition26() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {40, 20, 10}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.4, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: raster size is 1x2; should not average. */
  @Test
  public void testComputeBestPosition27() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {40},
      {20}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.1, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: raster size is 1x3; but best index is on edge so
   * should not average.
   */
  @Test
  public void testComputeBestPosition28() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {40},
      {20},
      {10}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.2, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: raster size is 3x2; should not average. */
  @Test
  public void testComputeBestPosition29() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {20, 40, 0},
      { 0, 20, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(-0.1, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: raster size is 2x3; should not average. */
  @Test
  public void testComputeBestPosition30() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {20, 20},
      {20, 40},
      {20,  0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.2, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: five columns; ignore outside averaging region. */
  @Test
  public void testComputeBestPosition31() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {399, 0,   0, 0, 0},
      {399, 0, 400, 0, 0},
      {399, 0,   0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: five rows; ignore outside averaging region. */
  @Test
  public void testComputeBestPosition32() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {399, 399, 399},
      {  0,   0,   0},
      {  0, 400,   0},
      {  0,   0,   0},
      {  0,   0,   0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.0, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: one row; positive best position; negative scores. */
  @Test
  public void testComputeBestPosition33() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 0, 0, 0, 0, -60, 40, -60, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.8, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: one row; negative best position; negative scores. */
  @Test
  public void testComputeBestPosition34() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, -60, 40, -60, 0, 0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.8, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /* Weighted best position: one row; negative best position; positive scores. */
  @Test
  public void testComputeBestPosition35() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 20, 40, 20, 0, 0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.4, 0.2);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-0.8, bestPosition.getX(), 1e-6);
    assertEquals(0.0, bestPosition.getY(), 1e-6);
  }

  /*
   * Weighted best position: high score in middle, lower on top, even lower on
   * bottom and bottom-right. But since there are two scores on the bottom that
   * are fairly large, they actually pull the average position closer to them,
   * even though the one cell above the high score cell has a score bigger than
   * either of the two below.
   */
  @Test
  public void testComputeBestPosition36() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {1,  74, 21},
      {0, 100, 89},
      {0,  60, 69}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.05, 0.025);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(0.02149758, bestPosition.getX(), 1e-6);
    assertEquals(0.00199275, bestPosition.getY(), 1e-6);
  }

  /*
   * Best position averaging.
   */
  @Test
  public void testComputeBestPosition37() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {0, 66, 42},
      {0, 71, 15},
      {0, 30,  0}
    };
    /* @formatter:on */
    Raster2d raster = createRaster(scores, 0.05, 0.025, -1.773, 0.295);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-1.76027679, bestPosition.getX(), 1e-6);
    assertEquals(0.28629464, bestPosition.getY(), 1e-6);
  }

  /*
   * Negative scores.
   */
  @Test
  public void testComputeBestPosition38() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {-208, 168, -208},
      {-208, 193,  -86},
      {-208,  67, -208}
    };
    /* @formatter:on */
    Raster2d raster =
        createRaster(scores, AnalysisPackage.DTREK, 0.05, 0.025, -1.707, 0.775);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-1.70180409, bestPosition.getX(), 1e-6);
    assertEquals(0.77284923, bestPosition.getY(), 1e-6);
  }

  /*
   * Negative scores.
   */
  @Test
  public void testComputeBestPosition39() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {-208, 305,  -82},
      {-208, 344, -208},
      {-208, 148, -208}
    };
    /* @formatter:on */
    Raster2d raster =
        createRaster(scores, AnalysisPackage.DTREK, 0.05, 0.025, -1.709, 0.795);
    Position2d bestPosition = raster.computeBestPosition();
    assertEquals(-1.70492760, bestPosition.getX(), 1e-6);
    assertEquals(0.79042663, bestPosition.getY(), 1e-6);
  }

  @Test
  public void testAssignPositionsCenteredOn01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0, 0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-0.3, -0.2}, {0, -0.2}, {0.3, -0.2}},
      {{-0.3,  0.0}, {0,  0.0}, {0.3,  0.0}},
      {{-0.3,  0.2}, {0,  0.2}, {0.3,  0.2}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsCenteredOn02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0, 0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-0.45, -0.3}, {-0.15, -0.3}, {0.15, -0.3}, {0.45, -0.3}},
      {{-0.45, -0.1}, {-0.15, -0.1}, {0.15, -0.1}, {0.45, -0.1}},
      {{-0.45,  0.1}, {-0.15,  0.1}, {0.15,  0.1}, {0.45,  0.1}},
      {{-0.45,  0.3}, {-0.15,  0.3}, {0.15,  0.3}, {0.45,  0.3}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsCenteredOn03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0.1, 0.2);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-0.2, 0.0}, {0.1, 0.0}, {0.4, 0.0}},
      {{-0.2, 0.2}, {0.1, 0.2}, {0.4, 0.2}},
      {{-0.2, 0.4}, {0.1, 0.4}, {0.4, 0.4}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsCenteredOn04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 30, 20, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0, 0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-30.0, -20.0}, {0.0, -20.0}, {30.0, -20.0}},
      {{-30.0,   0.0}, {0.0,   0.0}, {30.0,   0.0}},
      {{-30.0,  20.0}, {0.0,  20.0}, {30.0,  20.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsCenteredOn05() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 123, 223, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0, 0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-123.0, -223.0}, {0.0, -223.0}, {123.0, -223.0}},
      {{-123.0,    0.0}, {0.0,    0.0}, {123.0,    0.0}},
      {{-123.0,  223.0}, {0.0,  223.0}, {123.0,  223.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsCenteredOn06() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 100, 200, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0, 0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsCenteredOn07() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 100, 200, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsCenteredOn(0.2, 0.4);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.2, 0.4}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(-0.3, -0.2);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-0.3, -0.2}, {0.0, -0.2}, {0.3, -0.2}},
      {{-0.3,  0.0}, {0.0,  0.0}, {0.3,  0.0}},
      {{-0.3,  0.2}, {0.0,  0.2}, {0.3,  0.2}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(-0.45, -0.3);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-0.45, -0.3}, {-0.15, -0.3}, {0.15, -0.3}, {0.45, -0.3}},
      {{-0.45, -0.1}, {-0.15, -0.1}, {0.15, -0.1}, {0.45, -0.1}},
      {{-0.45,  0.1}, {-0.15,  0.1}, {0.15,  0.1}, {0.45,  0.1}},
      {{-0.45,  0.3}, {-0.15,  0.3}, {0.15,  0.3}, {0.45,  0.3}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(-0.2, 0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-0.2, 0.0}, {0.1, 0.0}, {0.4, 0.0}},
      {{-0.2, 0.2}, {0.1, 0.2}, {0.4, 0.2}},
      {{-0.2, 0.4}, {0.1, 0.4}, {0.4, 0.4}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 30, 20, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(-30, -20);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-30.0, -20.0}, {0.0, -20.0}, {30.0, -20.0}},
      {{-30.0,   0.0}, {0.0,   0.0}, {30.0,   0.0}},
      {{-30.0,  20.0}, {0.0,  20.0}, {30.0,  20.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell05() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 123, 223, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(-123, -223);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{-123.0, -223.0}, {0.0, -223.0}, {123.0, -223.0}},
      {{-123.0,    0.0}, {0.0,    0.0}, {123.0,    0.0}},
      {{-123.0,  223.0}, {0.0,  223.0}, {123.0,  223.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell06() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 100, 200, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(0.0, 0.0);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsRelativeToCell07() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 100, 200, LengthUnit.MICROMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsRelativeToCell(0.2, 0.4);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.2, 0.4}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositions01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1, 0, 0, 1, 4);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.4}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.6}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.8}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositions02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1, 1, 0, 2, 4);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.6, 0.2}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.6, 0.4}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.6, 0.6}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.6, 0.8}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositions03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1, 0, 0, 3, 1);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.6, 0.2}, {0.9, 0.2}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositions04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1, 0, 1, 3, 2);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.4}, {0.6, 0.4}, {0.9, 0.4}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositions05() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1, 1, 1, 3, 3);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.6, 0.4}, {0.9, 0.4}},
      {{0.0, 0.0}, {0.6, 0.6}, {0.9, 0.6}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.TOP);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.6, 0.2}, {0.9, 0.2}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.TOP_RIGHT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.6, 0.2}, {0.9, 0.2}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.4}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.6}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.8}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.RIGHT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.2}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.4}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.6}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.8}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.BOTTOM_RIGHT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.2}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.4}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.9, 0.6}},
      {{0.3, 0.8}, {0.6, 0.8}, {0.9, 0.8}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong05() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.BOTTOM);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.8}, {0.6, 0.8}, {0.9, 0.8}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong06() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.BOTTOM_LEFT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.4}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.6}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.8}, {0.6, 0.8}, {0.9, 0.8}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong07() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.LEFT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.4}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.6}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.8}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testAssignPositionsAlong08() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositionsAlong(0.15, 0.1, Raster2dEdge.TOP_LEFT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.2}, {0.6, 0.2}, {0.9, 0.2}},
      {{0.3, 0.4}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.6}, {0.0, 0.0}, {0.0, 0.0}},
      {{0.3, 0.8}, {0.0, 0.0}, {0.0, 0.0}}
    };
    /* @formatter:on */
    for (int c = 0; c < expectedPositions[0].length; c++) {
      for (int r = 0; r < expectedPositions.length; r++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
      }
    }
  }

  @Test
  public void testEdge01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dEdge.TOP, harness.edge(new Raster2dIndex(1, 0)));
    assertEquals(Raster2dEdge.TOP_RIGHT,
        harness.edge(new Raster2dIndex(harness.columnCount() - 1, 0)));
    assertEquals(Raster2dEdge.RIGHT,
        harness.edge(new Raster2dIndex(harness.columnCount() - 1, 1)));
    assertEquals(Raster2dEdge.BOTTOM_RIGHT,
        harness.edge(new Raster2dIndex(harness.columnCount() - 1, harness.rowCount() - 1)));
    assertEquals(Raster2dEdge.BOTTOM, harness.edge(new Raster2dIndex(1, harness.rowCount() - 1)));
    assertEquals(Raster2dEdge.BOTTOM_LEFT,
        harness.edge(new Raster2dIndex(0, harness.rowCount() - 1)));
    assertEquals(Raster2dEdge.LEFT, harness.edge(new Raster2dIndex(0, 1)));
    assertEquals(Raster2dEdge.TOP_LEFT, harness.edge(new Raster2dIndex(0, 0)));
    assertNull(harness.edge(new Raster2dIndex(1, 1)));
  }

  @Test
  public void testEdge02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dEdge.TOP, harness.edge(new Raster2dIndex(0, 0)));
    assertNull(harness.edge(new Raster2dIndex(0, 1)));
    assertEquals(Raster2dEdge.BOTTOM, harness.edge(new Raster2dIndex(0, 2)));
  }

  @Test
  public void testEdge03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dEdge.LEFT, harness.edge(new Raster2dIndex(0, 0)));
    assertNull(harness.edge(new Raster2dIndex(1, 0)));
    assertEquals(Raster2dEdge.RIGHT, harness.edge(new Raster2dIndex(2, 0)));
  }

  /*
   * The sole cell of a 1x1 raster is considered to be on the special ALL edge.
   */
  @Test
  public void testEdge04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dEdge.ALL, harness.edge(new Raster2dIndex(0, 0)));
  }

  @Test
  public void testIsOnEdge01() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertTrue(harness.isOnEdge(new Raster2dIndex(1, 0)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(harness.columnCount() - 1, 0)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(harness.columnCount() - 1, 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(harness.columnCount() - 1,
        harness.rowCount() - 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(1, harness.rowCount() - 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, harness.rowCount() - 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 0)));
    assertFalse(harness.isOnEdge(new Raster2dIndex(1, 1)));
  }

  @Test
  public void testIsOnEdge02() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 3, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 0)));
    assertFalse(harness.isOnEdge(new Raster2dIndex(0, 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 2)));
  }

  @Test
  public void testIsOnEdge03() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 0)));
    assertFalse(harness.isOnEdge(new Raster2dIndex(1, 0)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(2, 0)));
  }

  @Test
  public void testIsOnEdge04() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 2, 2, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 0)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(1, 0)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 1)));
    assertTrue(harness.isOnEdge(new Raster2dIndex(1, 1)));
  }

  /*
   * The sole cell of a 1x1 raster is considered to be on the special ALL edge.
   */
  @Test
  public void testIsOnEdge05() {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 1, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    assertTrue(harness.isOnEdge(new Raster2dIndex(0, 0)));
  }

  @Test
  public void testUnmaskedEdge_1x1() {
    Raster2d r = createRaster(1, 1, new Raster2dMask());
    assertEquals(Raster2dEdge.ALL, r.unmaskedEdge(new Raster2dIndex(0, 0)));
  }

  @Test
  public void testUnmaskedEdge_1x1Masked() {
    Raster2dMask m = new Raster2dMask();
    m.set(0, 0, true);
    Raster2d r = createRaster(1, 1, m);
    assertNull(r.unmaskedEdge(new Raster2dIndex(0, 0)));
  }

  @Test
  public void testUnmaskedEdge_top() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 1, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.TOP, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_leftTopRight() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {1, 1, 1, 1},
      {1, 1, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.TOP, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_topRight() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 1, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.TOP_RIGHT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_right() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.RIGHT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_topRightBottom() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 1, 1},
      {0, 0, 0, 1},
      {0, 0, 1, 1},
      {0, 0, 1, 1}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.RIGHT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_bottomRight() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 1, 1},
      {0, 0, 1, 1}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.BOTTOM_RIGHT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_bottom() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 1, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.BOTTOM, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_rightBottomLeft() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {1, 1, 0, 1},
      {1, 1, 1, 1},
      {1, 1, 1, 1}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.BOTTOM, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_bottomLeft() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {1, 1, 0, 0},
      {1, 1, 0, 0},
      {0, 0, 1, 1},
      {0, 0, 1, 1}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.BOTTOM_LEFT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_left() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.LEFT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_bottomLeftTop() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {1, 1, 1, 0},
      {1, 1, 0, 0},
      {0, 0, 1, 1},
      {0, 0, 1, 1}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.LEFT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testUnmaskedEdge_topLeft() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {1, 1, 1, 0},
      {1, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertEquals(Raster2dEdge.TOP_LEFT, r.unmaskedEdge(new Raster2dIndex(2, 1)));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_no() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    for (Raster2dEdge each : Raster2dEdge.values()) {
      assertFalse(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), each));
    }
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_top() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.TOP));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.TOP));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_topRight() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.TOP_RIGHT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.TOP_RIGHT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_right() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 1, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.RIGHT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 1, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.RIGHT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_bottomRight() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.BOTTOM_RIGHT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 1, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.BOTTOM_RIGHT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_bottom() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.BOTTOM));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.BOTTOM));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_bottomLeft() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.BOTTOM_LEFT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.BOTTOM_LEFT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_left() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 1, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.LEFT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 1, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.LEFT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_topLeft() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.TOP_LEFT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 1, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 1), Raster2dEdge.TOP_LEFT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_topOnTopEdge() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 1, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 0), Raster2dEdge.TOP));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 0), Raster2dEdge.TOP));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_rightOnRightEdge() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(3, 1), Raster2dEdge.RIGHT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(3, 1), Raster2dEdge.RIGHT));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_bottomOnBottomEdge() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 1, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 3), Raster2dEdge.BOTTOM));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 1}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(2, 3), Raster2dEdge.BOTTOM));
  }

  @Test
  public void testHasMaskedOrthogonalNeighbor_leftOnLeftEdge() {
    /* @formatter:off */
    int[][] ma = new int[][] {
      {1, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    Raster2d r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(0, 1), Raster2dEdge.LEFT));
    /* @formatter:off */
    ma = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */
    r = createRaster(ma[0].length, ma.length, createMask(ma));
    assertTrue(r.hasMaskedOrthogonalNeighbor(new Raster2dIndex(0, 1), Raster2dEdge.LEFT));
  }

  @Test
  public void testShitftToward01() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.TOP);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.0}, {0.6, 0.0}, {0.9, 0.0}},
      {{0.3, 0.2}, {0.6, 0.2}, {0.9, 0.2}},
      {{0.3, 0.4}, {0.6, 0.4}, {0.9, 0.4}},
      {{0.3, 0.6}, {0.6, 0.6}, {0.9, 0.6}}
    };
    Integer[][] expectedScores = new Integer[][] {
      { null, null, null},
      {   10,   11,   12},
      {   13,   14,   15},
      {   16,   17,   18}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward02() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.TOP_RIGHT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.6, 0.0}, {0.9, 0.0}, {1.2, 0.0}},
      {{0.6, 0.2}, {0.9, 0.2}, {1.2, 0.2}},
      {{0.6, 0.4}, {0.9, 0.4}, {1.2, 0.4}},
      {{0.6, 0.6}, {0.9, 0.6}, {1.2, 0.6}}
    };
    Integer[][] expectedScores = new Integer[][] {
      { null, null, null},
      {   11,   12, null},
      {   14,   15, null},
      {   17,   18, null}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward03() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.RIGHT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.6, 0.2}, {0.9, 0.2}, {1.2, 0.2}},
      {{0.6, 0.4}, {0.9, 0.4}, {1.2, 0.4}},
      {{0.6, 0.6}, {0.9, 0.6}, {1.2, 0.6}},
      {{0.6, 0.8}, {0.9, 0.8}, {1.2, 0.8}}
    };
    Integer[][] expectedScores = new Integer[][] {
      {11, 12, null},
      {14, 15, null},
      {17, 18, null},
      {20, 21, null}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward04() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.BOTTOM_RIGHT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.6, 0.4}, {0.9, 0.4}, {1.2, 0.4}},
      {{0.6, 0.6}, {0.9, 0.6}, {1.2, 0.6}},
      {{0.6, 0.8}, {0.9, 0.8}, {1.2, 0.8}},
      {{0.6, 1.0}, {0.9, 1.0}, {1.2, 1.0}}
    };
    Integer[][] expectedScores = new Integer[][] {
      {  14,   15, null},
      {  17,   18, null},
      {  20,   21, null},
      {null, null, null}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward05() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.BOTTOM);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.3, 0.4}, {0.6, 0.4}, {0.9, 0.4}},
      {{0.3, 0.6}, {0.6, 0.6}, {0.9, 0.6}},
      {{0.3, 0.8}, {0.6, 0.8}, {0.9, 0.8}},
      {{0.3, 1.0}, {0.6, 1.0}, {0.9, 1.0}}
    };
    Integer[][] expectedScores = new Integer[][] {
      {  13,   14,   15},
      {  16,   17,   18},
      {  19,   20,   21},
      {null, null, null}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward06() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.BOTTOM_LEFT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.4}, {0.3, 0.4}, {0.6, 0.4}},
      {{0.0, 0.6}, {0.3, 0.6}, {0.6, 0.6}},
      {{0.0, 0.8}, {0.3, 0.8}, {0.6, 0.8}},
      {{0.0, 1.0}, {0.3, 1.0}, {0.6, 1.0}}
    };
    Integer[][] expectedScores = new Integer[][] {
      {null,   13,   14},
      {null,   16,   17},
      {null,   19,   20},
      {null, null, null}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward07() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.LEFT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.2}, {0.3, 0.2}, {0.6, 0.2}},
      {{0.0, 0.4}, {0.3, 0.4}, {0.6, 0.4}},
      {{0.0, 0.6}, {0.3, 0.6}, {0.6, 0.6}},
      {{0.0, 0.8}, {0.3, 0.8}, {0.6, 0.8}}
    };
    Integer[][] expectedScores = new Integer[][] {
      {null, 10, 11},
      {null, 13, 14},
      {null, 16, 17},
      {null, 19, 20}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testShitftToward08() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);
    harness.assignPositions(0.15, 0.1);
    int score = 10;
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        setScore(harness, c, r, score);
        score++;
      }
    }
    harness.shiftToward(Raster2dEdge.TOP_LEFT);
    /* @formatter:off */
    double[][][] expectedPositions = new double[][][] {
      {{0.0, 0.0}, {0.3, 0.0}, {0.6, 0.0}},
      {{0.0, 0.2}, {0.3, 0.2}, {0.6, 0.2}},
      {{0.0, 0.4}, {0.3, 0.4}, {0.6, 0.4}},
      {{0.0, 0.6}, {0.3, 0.6}, {0.6, 0.6}}
    };
    Integer[][] expectedScores = new Integer[][] {
      {null, null, null},
      {null,   10,   11},
      {null,   13,   14},
      {null,   16,   17}
    };
    /* @formatter:on */
    for (int r = 0; r < harness.rowCount(); r++) {
      for (int c = 0; c < harness.columnCount(); c++) {
        assertEquals("horiz c=" + c + ",r=" + r, expectedPositions[r][c][0],
            harness.getCell(c, r).getHorizontalPosition(), 1e-6);
        assertEquals("vert c=" + c + ",r=" + r, expectedPositions[r][c][1], harness
            .getCell(c, r).getVerticalPosition(), 1e-6);
        assertEquals("score c=" + c + ",r=" + r, expectedScores[r][c], harness.getCell(c, r)
            .getScore());
      }
    }
  }

  @Test
  public void testRenameCellImageFile01() throws Exception {
    Raster2d harness =
        new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2, LengthUnit.MILLIMETER,
            500, 2, new File("/nonexistent")), ".cbf", false);

    File from = null;
    File to = null;
    boolean ok = false;

    try {
      from = File.createTempFile("imcadr-test-from", ".cbf");
      to = File.createTempFile("imcadr-test-to", ".cbf");
      harness.renameCellImageFile(from, to);
      assertFalse(from.exists());
      to.delete();
      ok = true;
    } finally {
      if (!ok) {
        try {
          from.delete();
        } catch (RuntimeException e) {
          /* IGNORED */
        }
        try {
          to.delete();
        } catch (RuntimeException e) {
          /* IGNORED */
        }
      }
    }
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor01() {
    new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 3, 123, 223, LengthUnit.METER, 500, 2,
        new File("/nonexistent")), ".cbf", false);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testConstructor02() {
    new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 0, 1, 123, 223, LengthUnit.MILLIMETER, 500,
        2, new File("/nonexistent")), ".cbf", false);
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_rowMajorEqualCellSize() {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dRowFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_rowMajorUnequalCellSize() {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 3, 0.3, 0.5,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dRowFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_rowMajorEqualNumRowAndNumCol() {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 4, 0.3, 0.5,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dRowFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_colMajorEqualCellSize() {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dColumnFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_colMajorUnequalCellSize() {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.5, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dColumnFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_colMajorEqualNumRowAndNumCol() {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 4, 0.5, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false);
    assertEquals(Raster2dColumnFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_zWeightOnZyPlane() {
    @SuppressWarnings("serial")
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 4, 0.3, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false) {
      @Override
      protected double getAxisFlyScanWeight(Raster2dPlane plane, Direction2d direction) {
        switch (getAxis(plane, direction)) {
          case Y:
            return 1.0;
          case Z:
            return 1.5;
          default:
            throw new BugException();
        }
      }
    };
    assertEquals(Raster2dRowFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_yWeightOnZyPlane() {
    @SuppressWarnings("serial")
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, 4, 4, 0.3, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false) {
      @Override
      protected double getAxisFlyScanWeight(Raster2dPlane plane, Direction2d direction) {
        switch (getAxis(plane, direction)) {
          case Y:
            return 1.5;
          case Z:
            return 1.0;
          default:
            throw new BugException();
        }
      }
    };
    assertEquals(Raster2dColumnFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_zWeightOnZxPlane() {
    @SuppressWarnings("serial")
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZX, 4, 4, 0.3, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false) {
      @Override
      protected double getAxisFlyScanWeight(Raster2dPlane plane, Direction2d direction) {
        switch (getAxis(plane, direction)) {
          case X:
            return 1.0;
          case Z:
            return 1.5;
          default:
            throw new BugException();
        }
      }
    };
    assertEquals(Raster2dRowFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  @Test
  public void testUnmaskedUnscoredFlyScanIterator_xWeightOnZxPlane() {
    @SuppressWarnings("serial")
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZX, 4, 4, 0.3, 0.3,
        LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), ".cbf", false) {
      @Override
      protected double getAxisFlyScanWeight(Raster2dPlane plane, Direction2d direction) {
        switch (getAxis(plane, direction)) {
          case X:
            return 1.5;
          case Z:
            return 1.0;
          default:
            throw new BugException();
        }
      }
    };
    assertEquals(Raster2dColumnFlyScanIterator.class,
        raster.unmaskedUnscoredFlyScanIterator(null, null).getClass());
  }

  protected static Raster2d createRaster(int[][] scores, double cellWidth, double cellHeight) {
    return createRaster(scores, cellWidth, cellHeight, 0.0, 0.0);
  }

  protected static Raster2d createRaster(int[][] scores, double cellWidth, double cellHeight,
      double centerHorizPos, double centerVertPos) {
    return createRaster(scores, null, cellWidth, cellHeight, centerHorizPos, centerVertPos);
  }

  protected static Raster2d createRaster(int[][] scores, AnalysisPackage pkg, double cellWidth,
      double cellHeight, double centerHorizPos, double centerVertPos) {
    Raster2d raster = new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, scores[0].length,
        scores.length, cellWidth, cellHeight, LengthUnit.MILLIMETER, 500, 2, true,
        new File("/nonexistent"), (new ImageAnalysisSpec.Builder()).analysisPackage(pkg).build()),
        ".cbf", false);
    raster.assignPositionsCenteredOn(centerHorizPos, centerVertPos);
    for (int r = 0; r < scores.length; r++)
      for (int c = 0; c < scores[0].length; c++)
        setScore(raster, c, r, scores[r][c]);
    return raster;
  }

  protected static Raster2d createRaster(int numColumns, int numRows, Raster2dMask mask) {
    return new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, numColumns, numRows, numColumns,
        numRows, mask, 0.3, 0.2, LengthUnit.MILLIMETER, 500L, 2, false, null, new File(
            "/nonexistent"), null), ".cbf", false);
  }

  protected static Raster2dMask createMask(int[][] maskArray) {
    return TestUtils.newMask(maskArray);
  }

  protected static void setScore(Raster2d raster, int column, int row, Integer score) {
    ImageAnalysisResult result = new ImageAnalysisResult();
    result.setScore(score);
    raster.loadResult(column, row, result);
  }
}
