package imcacat.imcadr;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.NoSuchElementException;

import org.junit.Test;

/**
 * I test <code>Raster2dUnmaskedIterator</code>.
 */
public class Raster2dUnmaskedIteratorTest {
  @Test
  public void testHasNext_zero() {
    Raster2d raster = newRaster(1, 1, newMask(0, 0));
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    assertFalse(iter.hasNext());
  }

  @Test(expected = NoSuchElementException.class)
  public void testNext_zero() {
    Raster2d raster = newRaster(1, 1, newMask(0, 0));
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    iter.next();
  }

  @Test
  public void testNext_one() {
    Raster2d raster = newRaster(1, 1, new Raster2dMask());
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(0, 0), iter.nextIndex());
    iter.next();
    assertFalse(iter.hasNext());
  }

  @Test
  public void testNext_oneOfThreeInRow() {
    Raster2d raster = newRaster(3, 1, newMask(0, 0, 1, 0));
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(2, 0), iter.nextIndex());
    iter.next();
    assertFalse(iter.hasNext());
  }

  @Test
  public void testNext_twoOfFourInRow() {
    Raster2d raster = newRaster(4, 1, newMask(0, 0, 2, 0));
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(1, 0), iter.nextIndex());
    iter.next();
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(3, 0), iter.nextIndex());
    iter.next();
    assertFalse(iter.hasNext());
  }

  @Test
  public void testNext_oneOfFourInTwoRows() {
    Raster2d raster = newRaster(2, 2, newMask(0, 0, 1, 0, 0, 1));
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(1, 1), iter.nextIndex());
    iter.next();
    assertFalse(iter.hasNext());
  }

  @Test
  public void testNext_twoOfNineInThreeRows() {
    Raster2d raster = newRaster(3, 3, newMask(0, 0, 1, 0, 0, 1, 1, 1, 2, 1, 0, 2, 2, 2));
    Raster2dIterator iter = new Raster2dUnmaskedIterator(raster);
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(2, 0), iter.nextIndex());
    iter.next();
    assertTrue(iter.hasNext());
    assertEquals(new Raster2dIndex(1, 2), iter.nextIndex());
    iter.next();
    assertFalse(iter.hasNext());
  }

  private static Raster2d newRaster(int numColumns, int numRows, Raster2dMask mask) {
    return new Raster2d(new Raster2dSpec(Raster2dPlane.ZY, numColumns, numRows, numColumns,
        numRows, mask, 0.3, 0.3, LengthUnit.MILLIMETER, 500L, 2, false, null, new File(
            "/nonexistent"), null), ".cbf", false);
  }

  /**
   * @param maskedIndices colA, rowA, colB, rowB, ...
   */
  private static Raster2dMask newMask(int... maskedIndices) {
    Raster2dMask result = new Raster2dMask();
    for (int i = 1; i < maskedIndices.length; i += 2) {
      result.set(maskedIndices[i - 1], maskedIndices[i], true);
    }
    return result;
  }
}
