package imcacat.imcadr;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Test;

/**
 * I test <code>Rasterizer2d</code>.
 */
public class Rasterizer2dTest {
  private static final int DEFAULT_RASTER_WIDTH = 3;
  private static final int DEFAULT_RASTER_HEIGHT = 4;
  private static final double DEFAULT_CELL_WIDTH = 0.3;
  private static final double DEFAULT_CELL_HEIGHT = 0.2;

  /* High score not on edge. */
  @Test
  public void testRun01() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40},
      {60, 60, 60},
      {60, 80, 60},
      {60, 60, 60}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 2;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on top edge. */
  @Test
  public void testRun02() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {40, 60, 60, 60, 40},
      {40, 60, 80, 60, 40},
      {40, 60, 60, 60, 40},
      {40, 40, 40, 40, 40},
      {20, 20, 20, 20, 20},
      { 0,  0,  0,  0,  0}
    };
    /* @formatter:on */
    int bestColumn = 2;
    int bestRow = 1;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on top right edge. */
  @Test
  public void testRun03() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {20, 40, 60, 60, 60},
      {20, 40, 60, 80, 60},
      {20, 40, 60, 60, 60},
      {20, 40, 40, 40, 40},
      {20, 20, 20, 20, 20},
      { 0,  0,  0,  0,  0}
    };
    /* @formatter:on */
    int bestColumn = 3;
    int bestRow = 1;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on right edge. */
  @Test
  public void testRun04() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {20, 40, 40, 40, 40},
      {20, 40, 60, 60, 60},
      {20, 40, 60, 80, 60},
      {20, 40, 60, 60, 60},
      {20, 40, 40, 40, 40},
      {20, 20, 20, 20, 20}
    };
    /* @formatter:on */
    int bestColumn = 3;
    int bestRow = 2;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on bottom right edge. */
  @Test
  public void testRun05() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      { 0,  0,  0,  0,  0},
      {20, 20, 20, 20, 20},
      {20, 40, 40, 40, 40},
      {20, 40, 60, 60, 60},
      {20, 40, 60, 80, 60},
      {20, 40, 60, 60, 60}
    };
    /* @formatter:on */
    int bestColumn = 3;
    int bestRow = 4;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on bottom edge. */
  @Test
  public void testRun06() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      { 0,  0,  0,  0,  0},
      {20, 20, 20, 20, 20},
      {40, 40, 40, 40, 40},
      {40, 60, 60, 60, 40},
      {40, 60, 80, 60, 40},
      {40, 60, 60, 60, 40}
    };
    /* @formatter:on */
    int bestColumn = 2;
    int bestRow = 4;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on bottom left edge. */
  @Test
  public void testRun07() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      { 0,  0,  0,  0,  0},
      {20, 20, 20, 20, 20},
      {40, 40, 40, 40, 20},
      {60, 60, 60, 40, 20},
      {60, 80, 60, 40, 20},
      {60, 60, 60, 40, 20}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 4;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on left edge. */
  @Test
  public void testRun08() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {20, 20, 20, 20, 20},
      {40, 40, 40, 40, 20},
      {60, 60, 60, 40, 20},
      {60, 80, 60, 40, 20},
      {60, 60, 60, 40, 20},
      {40, 40, 40, 40, 20}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 3;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on top left edge. */
  @Test
  public void testRun09() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {60, 60, 60, 40, 20},
      {60, 80, 60, 40, 20},
      {60, 60, 60, 40, 20},
      {40, 40, 40, 40, 20},
      {20, 20, 20, 20, 20},
      { 0,  0,  0,  0,  0}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 1;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * High score on top edge after one shift (i.e. requires two shifts toward
   * top to find).
   */
  @Test
  public void testRun10() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.7}, {-0.3, -0.7}, {0, -0.7}, {0.3, -0.7}, {0.6, -0.7}},
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {40, 60, 60, 60, 40},
      {40, 60, 80, 60, 40},
      {60, 60, 60, 60, 40},
      {40, 40, 40, 40, 40},
      {20, 20, 20, 20, 20},
      { 0,  0,  0,  0,  0},
      { 0,  0,  0,  0,  0}
    };
    /* @formatter:on */
    int bestColumn = 2;
    int bestRow = 1;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * High score on top edge after 4 top shifts when stops due to reaching
   * vertical shifts limit.
   */
  @Test
  public void testRun11() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -1.3}, {-0.3, -1.3}, {0, -1.3}, {0.3, -1.3}, {0.6, -1.3}},
      {{-0.6, -1.1}, {-0.3, -1.1}, {0, -1.1}, {0.3, -1.1}, {0.6, -1.1}},
      {{-0.6, -0.9}, {-0.3, -0.9}, {0, -0.9}, {0.3, -0.9}, {0.6, -0.9}},
      {{-0.6, -0.7}, {-0.3, -0.7}, {0, -0.7}, {0.3, -0.7}, {0.6, -0.7}},
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {60, 79, 99, 79, 60},
      {60, 79, 80, 79, 60},
      {55, 60, 60, 60, 55},
      {35, 55, 55, 55, 35},
      {30, 40, 40, 40, 30},
      {25, 30, 30, 30, 25},
      {15, 20, 20, 20, 15},
      { 5, 10, 10, 10,  5},
      { 4,  9,  9,  9,  4},
      { 2,  4,  4,  4,  2}
    };
    /* @formatter:on */
    int bestColumn = 2;
    int bestRow = 1;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score on edge with raster height of just one cell. */
  @Test
  public void testRun12() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, 0}, {0, 0}, {0.3, 0}, {0.6, 0}, {0.9, 0}}
    };
    int[][] scores = new int[][] {
      {10, 20, 40, 60, 40}
    };
    /* @formatter:on */
    int bestColumn = 3;
    int bestRow = 0;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 1, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* Prefer high score on edge when two cells share high score. */
  @Test
  public void testRun13() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, 0}, {0, 0}, {0.3, 0}, {0.6, 0}, {0.9, 0}}
    };
    int[][] scores = new int[][] {
      {10, 40, 40, 60, 40}
    };
    /* @formatter:on */
    int bestColumn = 3;
    int bestRow = 0;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 1, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * High score on bottom right edge after 3 bottom right shifts when stops due
   * to reaching vertical shifts limit.
   */
  @Test
  public void testRun14() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -1.3}, {-0.3, -1.3}, {0, -1.3}, {0.3, -1.3}, {0.6, -1.3},
            {0.9, -1.3}, {1.2, -1.3}, {1.5, -1.3}},
      {{-0.6, -1.1}, {-0.3, -1.1}, {0, -1.1}, {0.3, -1.1}, {0.6, -1.1},
            {0.9, -1.1}, {1.2, -1.1}, {1.5, -1.1}},
      {{-0.6, -0.9}, {-0.3, -0.9}, {0, -0.9}, {0.3, -0.9}, {0.6, -0.9},
            {0.9, -0.9}, {1.2, -0.9}, {1.5, -0.9}},
      {{-0.6, -0.7}, {-0.3, -0.7}, {0, -0.7}, {0.3, -0.7}, {0.6, -0.7},
            {0.9, -0.7}, {1.2, -0.7}, {1.5, -0.7}},
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5},
            {0.9, -0.5}, {1.2, -0.5}, {1.5, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3},
            {0.9, -0.3}, {1.2, -0.3}, {1.5, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1},
            {0.9, -0.1}, {1.2, -0.1}, {1.5, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1},
            {0.9,  0.1}, {1.2,  0.1}, {1.5,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3},
            {0.9,  0.3}, {1.2,  0.3}, {1.5,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5},
            {0.9,  0.5}, {1.2,  0.5}, {1.5,  0.5}}
    };
    int[][] scores = new int[][] {
      {20, 20, 20, 40, 40, 60, 60, 60},
      {40, 40, 40, 40, 60, 60, 60, 99},
      {40, 40, 60, 60, 60, 60, 95, 60},
      {40, 60, 60, 60, 60, 90, 60, 60},
      {40, 60, 60, 60, 80, 60, 40, 40},
      {40, 60, 60, 70, 60, 40, 40, 20},
      {40, 60, 60, 60, 40, 40, 20, 10},
      {40, 40, 40, 40, 40, 20, 20, 10},
      {20, 20, 20, 40, 20, 20,  0, 10},
      { 0,  0, 20, 20, 20,  0,  0,  0}
    };
    /* @formatter:on */
    int bestColumn = 6;
    int bestRow = 2;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * High score on top edge with raster height of two cells shifts up once and
   * does not oscillate.
   */
  @Test
  public void testRun15() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
        {20, 30, 20},
        {20, 60, 20},
        {20, 30, 20}
    };
    int bestCol = 1;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", "0",
        "0.5");
    int[][] maskArray = new int[][] {
        {0, 0, 0},
        {0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
  }

  /*
   * High score on top-right corner with raster height of two cells shifts up
   * and right once and does not oscillate.
   */
  @Test
  public void testRun16() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
        {20, 20, 20, 20},
        {20, 20, 60, 20},
        {20, 20, 20, 20}
    };
    int bestCol = 2;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", "-0.5",
        "0.5");
    int[][] maskArray = new int[][] {
        {0, 0, 0},
        {0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
  }

  /*
   * High score on left edge with raster width of two cells shifts left once
   * and does not oscillate.
   */
  @Test
  public void testRun17() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
        {20, 20, 20},
        {20, 60, 20},
        {20, 20, 20}
    };
    int bestCol = 1;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", "0.5",
        "0");
    int[][] maskArray = new int[][] {
        {0, 0},
        {0, 0},
        {0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
  }

  /*
   * High score on right edge of 2x1 raster shifts right once and does not
   * oscillate.
   */
  @Test
  public void testRun18_right() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
        {20, 60, 20}
    };
    int bestCol = 1;
    int bestRow = 0;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", "-0.5",
        "0");
    int[][] maskArray = new int[][] {
        {0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
  }

  /*
   * High score on bottom edge of 1x2 raster shifts down once and does not
   * oscillate.
   */
  @Test
  public void testRun18_bottom() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
        {20},
        {60},
        {20}
    };
    int bestCol = 0;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", "0",
        "-0.5");
    int[][] maskArray = new int[][] {
        {0},
        {0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
  }

  /* Ensure Phi gets driven to 0 degrees before ZY raster. */
  @Test
  public void testRun19() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40},
      {60, 60, 60},
      {60, 80, 60},
      {60, 60, 60}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    MockBeamlineDriver driver = new MockBeamlineDriver();
    driver.movePhi(45);
    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), driver, false,
            positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertArrayEquals(new double[] {
        0,
        45,
        0}, driver.phiRbvHistory(), 1e-6);
  }

  /* High score on top edge, but no shift because chasing is not enabled. */
  @Test
  public void testRun20() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.7}, {-0.3, -0.7}, {0, -0.7}, {0.3, -0.7}, {0.6, -0.7}},
      {{-0.6, -0.5}, {-0.3, -0.5}, {0, -0.5}, {0.3, -0.5}, {0.6, -0.5}},
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {40, 60, 60, 60, 40},
      {40, 60, 90, 60, 40},
      {40, 60, 80, 60, 40},
      {40, 60, 60, 60, 40},
      {40, 40, 40, 40, 40},
      {20, 20, 20, 20, 20},
      { 0,  0,  0,  0,  0}
    };
    /* @formatter:on */
    int bestColumn = 2;
    int bestRow = 2;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, false, new File("/nonexistent"), null), false,
            positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * High score in middle of vertical raster of width 1. Should not shift.
   */
  @Test
  public void testRun21() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{0, -0.3}},
      {{0, -0.1}},
      {{0, +0.1}},
      {{0, +0.3}}
    };
    int[][] scores = new int[][] {
      {20},
      {60},
      {20},
      {40}
    };
    /* @formatter:on */
    int bestColumn = 0;
    int bestRow = 1;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * High score at bottom of vertical raster of width 1. Should shift down.
   */
  @Test
  public void testRun22() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{0, -0.3}},
      {{0, -0.1}},
      {{0, +0.1}},
      {{0, +0.3}},
      {{0, +0.5}}
    };
    int[][] scores = new int[][] {
      {20},
      {30},
      {40},
      {50},
      {40}
    };
    /* @formatter:on */
    int bestColumn = 0;
    int bestRow = 3;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 1, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* High score <= cutoff. */
  @Test
  public void testRun23() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {10, 11, 10},
      {11, 12, 13},
      {10, 25, 15},
      {11, 14, 13}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3, 0.2,
            LengthUnit.MILLIMETER, 500, 2, true, 25, new File("/nonexistent"), null), false,
            positionScores);
    rasterizer.doNotLogException();

    rasterizer.run();

    assertTrue(rasterizer.hasException());
    assertEquals(BestScoreCutoffException.class, rasterizer.getException().getClass());
  }

  /* Mask with high score cell on top edge without shifting. */
  @Test
  public void testRun24() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {90, 80, 90},
      { 0,  0,  0},
      {60, 60, 60},
      {60, 60, 60}
    };
    int bestCol = 1;
    int bestRow = 0;
    int[][] maskArray = new int[][] {
      {1, 0, 1},
      {0, 0, 0},
      {0, 0, 0},
      {0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, false, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* Mask with high score cell on top-right corner without shifting. */
  @Test
  public void testRun25() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {90, 90, 80},
      { 0,  0,  0},
      {60, 60, 60},
      {60, 60, 60}
    };
    int bestCol = 2;
    int bestRow = 0;
    int[][] maskArray = new int[][] {
      {1, 1, 0},
      {0, 0, 0},
      {0, 0, 0},
      {0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, false, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* Mask with high score cell in interior without shifting. */
  @Test
  public void testRun26() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40},
      {60, 90, 60},
      {60, 80, 60},
      {60,  0, 60}
    };
    int bestCol = 1;
    int bestRow = 2;
    int[][] maskArray = new int[][] {
      {0, 0, 0},
      {0, 1, 0},
      {0, 0, 0},
      {0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, false, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with surrounded high score cell on top edge with shifting results in
   * top shift.
   */
  @Test
  public void testRun27() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60},
      {60, 65, 60},
      {60, 75, 60},
      {60, 65, 60},
      {60, 60, 60},
      {60, 60, 60}
    };
    int bestCol = 1;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 0, 1);
    int[][] maskArray = new int[][] {
      {1, 1, 1},
      {0, 0, 0},
      {0, 0, 0},
      {0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with surrounded high score cell on right edge with shifting results
   * in right shift.
   */
  @Test
  public void testRun28() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 65, 75, 65, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 3;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 0, 0);
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with surrounded high score cell on bottom edge with shifting results
   * in bottom shift.
   */
  @Test
  public void testRun29() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60},
      {60, 60, 60},
      {60, 65, 60},
      {60, 75, 60},
      {60, 65, 60},
      {60, 60, 60}
    };
    int bestCol = 1;
    int bestRow = 3;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 0, -1);
    int[][] maskArray = new int[][] {
      {0, 0, 0},
      {0, 0, 0},
      {0, 0, 0},
      {1, 1, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with surrounded high score cell on left edge with shifting results in
   * left shift.
   */
  @Test
  public void testRun30() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 65, 75, 65, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 2;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, 0);
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-to-right high score cell on top edge with shifting
   * results in top and right shift.
   */
  @Test
  public void testRun31() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 65, 60},
      {60, 60, 60, 75, 60, 60},
      {60, 60, 65, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 3;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", -1, 1);
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-to-left high score cell on top edge with shifting
   * results in top and left shift.
   */
  @Test
  public void testRun32() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 65, 60, 60, 60, 60},
      {60, 60, 75, 60, 60, 60},
      {60, 60, 60, 65, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 2;
    int bestRow = 1;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, 1);
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-above high score cell on right edge with shifting
   * results in top and right shift.
   */
  @Test
  public void testRun33() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 65, 60},
      {60, 60, 60, 75, 60, 60},
      {60, 60, 65, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 3;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", -1, 1);
    int[][] maskArray = new int[][] {
      {1, 1, 1, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-below high score cell on right edge with shifting
   * results in bottom and right shift.
   */
  @Test
  public void testRun34() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 65, 60, 60},
      {60, 60, 60, 60, 75, 60},
      {60, 60, 60, 60, 60, 65},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 4;
    int bestRow = 3;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", -1, -1);
    int[][] maskArray = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 1, 1, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-to-right high score cell on bottom edge with shifting
   * results in bottom and right shift.
   */
  @Test
  public void testRun35() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 65, 60, 60, 60},
      {60, 60, 60, 75, 60, 60},
      {60, 60, 60, 60, 65, 60}
    };
    int bestCol = 3;
    int bestRow = 4;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", -1, -1);
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-to-left high score cell on bottom edge with shifting
   * results in bottom and left shift.
   */
  @Test
  public void testRun36() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 65, 60, 60},
      {60, 60, 75, 60, 60, 60},
      {60, 65, 60, 60, 60, 60}
    };
    int bestCol = 2;
    int bestRow = 4;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, -1);
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-above high score cell on left edge with shifting results
   * in top and left shift.
   */
  @Test
  public void testRun37() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {65, 60, 60, 60, 60, 60},
      {60, 75, 60, 60, 60, 60},
      {60, 60, 65, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 1;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, 1);
    int[][] maskArray = new int[][] {
      {1, 1, 1, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-below high score cell on left edge with shifting results
   * in bottom and left shift.
   */
  @Test
  public void testRun38() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 65, 60, 60, 60},
      {60, 75, 60, 60, 60, 60},
      {65, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 1;
    int bestRow = 3;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, -1);
    int[][] maskArray = new int[][] {
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 1, 1, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-to-top-and-right high score cell on top-right-unmasked
   * edge with shifting results in top and right shift.
   */
  @Test
  public void testRun39() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 65, 60},
      {60, 60, 60, 75, 60, 60},
      {60, 60, 65, 60, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 3;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", -1, 1);
    int[][] maskArray = new int[][] {
      {1, 1, 1, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1},
      {0, 0, 0, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with nothing-to-top-and-left high score cell on top-left-unmasked
   * edge with shifting results in top and right shift.
   */
  @Test
  public void testRun40() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 65, 60, 60, 60, 60},
      {60, 60, 75, 60, 60, 60},
      {60, 60, 60, 65, 60, 60},
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 60, 60, 60}
    };
    int bestCol = 2;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, 1);
    int[][] maskArray = new int[][] {
      {1, 1, 1, 1},
      {1, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestCol][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestCol][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /*
   * Mask with solo high score cell on right edge with shifting results in
   * right shift and polygon expansion to include cell above and below.
   */
  @Test
  public void testRun42() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 60, 65, 60, 60},
      {60, 60, 60, 60, 75,  0},
      {60, 60, 60, 60, 60, 65}
    };
    int bestCol = 4;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", -1, 0);
    int[][] maskArray = new int[][] {
      {0, 0, 0, 1},
      {0, 0, 0, 0},
      {0, 0, 0, 1},
      {0, 0, 0, 1}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    assertTrue(rasterizer.getRaster().isMasked(3, 0));
    assertTrue(rasterizer.getRaster().isMasked(3, 2));

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
    assertFalse(rasterizer.getRaster().isMasked(3, 0));
    assertFalse(rasterizer.getRaster().isMasked(3, 2));
  }

  /*
   * Mask with solo high score cell on left edge with shifting results in left
   * shift and polygon expansion to include cell above and below.
   */
  @Test
  public void testRun44() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {60, 60, 60, 60, 60, 60},
      {60, 60, 65, 60, 60, 60},
      { 0, 75, 60, 60, 60, 60},
      {65, 60, 60, 60, 60, 60}
    };
    int bestCol = 1;
    int bestRow = 2;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 1, 0);
    int[][] maskArray = new int[][] {
      {1, 0, 0, 0},
      {0, 0, 0, 0},
      {1, 0, 0, 0},
      {1, 0, 0, 0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    assertTrue(rasterizer.getRaster().isMasked(0, 0));
    assertTrue(rasterizer.getRaster().isMasked(0, 2));

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
    assertFalse(rasterizer.getRaster().isMasked(0, 0));
    assertFalse(rasterizer.getRaster().isMasked(0, 2));
  }

  /*
   * A 1x1 raster should not be shifted.
   */
  @Test
  public void testRun45() throws Exception {
    /* @formatter:off */
    int[][] scores = new int[][] {
      {75}
    };
    int bestCol = 0;
    int bestRow = 0;
    double[][][] positions = newPositions(scores[0].length, scores.length, "0.3", "0.2", 0, 0);
    int[][] maskArray = new int[][] {
      {0}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = newPositionScoresMap(positions, scores);

    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, maskArray[0].length,
            maskArray.length, null, null, newMask(maskArray), 0.3, 0.2, LengthUnit.MILLIMETER,
            500L, 2, true, null, new File("/nonexistent"), null), false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    Raster2dIndex actualBest =
        findPosition(positions, rasterizer.getBestHorizontalPosition(),
            rasterizer.getBestVerticalPosition(), 1e-6);
    assertNotNull("Couldn't find position (" + rasterizer.getBestHorizontalPosition() + ", " +
        rasterizer.getBestVerticalPosition() + ")", actualBest);
    assertEquals(bestCol, actualBest.getColumn());
    assertEquals(bestRow, actualBest.getRow());
    assertFalse(rasterizer.getRaster().isMasked(0, 0));
  }

  /* Fly-scan by column; high score not on edge. */
  @Test
  public void testRun46() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40},
      {60, 60, 60},
      {60, 80, 60},
      {60, 60, 60}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 2;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer = new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3,
        0.2, LengthUnit.MILLIMETER, 500, 2, true, true, 25, new File("/nonexistent"), null), false,
        positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* Fly-scan by row; high score not on edge. */
  @Test
  public void testRun47() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40, 40, 40},
      {60, 60, 60, 40, 40},
      {60, 80, 60, 40, 40},
      {60, 60, 60, 40, 40}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 2;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer = new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3,
        0.2, LengthUnit.MILLIMETER, 500, 2, true, true, 25, new File("/nonexistent"), null), false,
        positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* Fly-scan by row; high score on bottom edge, so fly-scan edge scan on
   * bottom. */
  @Test
  public void testRun48() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.6, -0.3}, {-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}, {0.6, -0.3}},
      {{-0.6, -0.1}, {-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}, {0.6, -0.1}},
      {{-0.6,  0.1}, {-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}, {0.6,  0.1}},
      {{-0.6,  0.3}, {-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}, {0.6,  0.3}},
      {{-0.6,  0.5}, {-0.3,  0.5}, {0,  0.5}, {0.3,  0.5}, {0.6,  0.5}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40, 40, 40},
      {60, 60, 60, 40, 40},
      {60, 60, 60, 40, 40},
      {60, 80, 60, 40, 40},
      {60, 60, 60, 40, 40}
    };
    /* @formatter:on */
    int bestColumn = 1;
    int bestRow = 3;

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    Rasterizer2d rasterizer = new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 5, 4, 0.3,
        0.2, LengthUnit.MILLIMETER, 500, 2, true, true, 25, new File("/nonexistent"), null), false,
        positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    assertEquals(positions[bestRow][bestColumn][0], rasterizer.getBestHorizontalPosition(), 1e-6);
    assertEquals(positions[bestRow][bestColumn][1], rasterizer.getBestVerticalPosition(), 1e-6);
  }

  /* Ensure correct image-taken publishing for fly-scans. */
  @Test
  public void testRun49() throws Exception {
    /* @formatter:off */
    double[][][] positions = new double[][][] {
      {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
      {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
      {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
      {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
    };
    int[][] scores = new int[][] {
      {40, 40, 40},
      {60, 60, 60},
      {60, 80, 60},
      {60, 60, 60}
    };
    /* @formatter:on */

    Map<Position2d, Integer> positionScores = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++)
      for (int c = 0; c < positions[0].length; c++)
        positionScores.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);

    MockBeamlineDriver driver = new MockBeamlineDriver();
    Rasterizer2d rasterizer = new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, 3, 4, 0.3,
        0.2, LengthUnit.MILLIMETER, 500, 2, true, true, 25, new File("/nonexistent"), null), driver,
        false, positionScores);

    rasterizer.run();

    assertFalse(rasterizer.hasException());
    List<File> published = driver.publishedDetectorImagesTaken();
    assertEquals(12, published.size());
  }

  /* Experiment state should be restored if interrupted. */
  @Test
  public void testInterrupt01() throws Exception {
    MockBeamlineDriver driver = new MockBeamlineDriver();
    driver.setRealisticExposureTimeEnabled(true);
    Map<Position2d, Integer> scores = createDefaultPositionScoresMap();
    Rasterizer2d rasterizer =
        new MockRasterizer2d(new Raster2dSpec(Raster2dPlane.ZY, DEFAULT_RASTER_WIDTH,
            DEFAULT_RASTER_HEIGHT, DEFAULT_CELL_WIDTH, DEFAULT_CELL_HEIGHT,
            LengthUnit.MILLIMETER, 500, 2, new File("/nonexistent")), driver, false, scores);
    rasterizer.doNotLogException();

    assertEquals(ExperimentState.IDLE, driver.experimentState());

    Thread t = new Thread(rasterizer);
    try {
      t.start();
      Thread.sleep(1000L);
    } finally {
      t.interrupt();
      t.join();
    }

    assertEquals(ExperimentState.IDLE, driver.experimentState());
  }

  private static Map<Position2d, Integer> createDefaultPositionScoresMap() {
    return newPositionScoresMap(
/* @formatter:off */
      new double[][][] {
        {{-0.3, -0.3}, {0, -0.3}, {0.3, -0.3}},
        {{-0.3, -0.1}, {0, -0.1}, {0.3, -0.1}},
        {{-0.3,  0.1}, {0,  0.1}, {0.3,  0.1}},
        {{-0.3,  0.3}, {0,  0.3}, {0.3,  0.3}}
      },
      new int[][] {
        {40, 40, 40},
        {60, 60, 60},
        {60, 80, 60},
        {60, 60, 60}
      });
      /* @formatter:on */
  }

  /**
   * @param positions [row][column][x,y] positions in mm
   * @param scores [row][column]
   */
  private static Map<Position2d, Integer> newPositionScoresMap(double[][][] positions,
      int[][] scores) {
    Map<Position2d, Integer> result = new HashMap<Position2d, Integer>();
    for (int r = 0; r < positions.length; r++) {
      for (int c = 0; c < positions[0].length; c++) {
        result.put(Position2d.mm(positions[r][c][0], positions[r][c][1]), scores[r][c]);
      }
    }
    return result;
  }

  /**
   * @param maskArray [row][column] mask array where nonzero is {@code true}
   */
  private static Raster2dMask newMask(int[][] maskArray) {
    return TestUtils.newMask(maskArray);
  }

  /**
   * @param rasterWidth width of raster in cells
   * @param rasterHeight height of raster in cells
   * @param cellWidth width of cell in mm
   * @param cellHeight height of cell in mm
   * @param rasterCenterOffsetX X offset of raster center in cells
   * @param rasterCenterOffsetY Y offset of raster center in cells
   *
   * @return positions [row][column][x,y] in mm
   */
  private static double[][][] newPositions(int rasterWidth, int rasterHeight, String cellWidth,
      String cellHeight, String rasterCenterOffsetX, String rasterCenterOffsetY) {
    return newPositions(BigDecimal.valueOf(rasterWidth), BigDecimal.valueOf(rasterHeight),
        new BigDecimal(cellWidth), new BigDecimal(cellHeight), new BigDecimal(
            rasterCenterOffsetX), new BigDecimal(rasterCenterOffsetY));
  }

  /**
   * @param rasterWidth width of raster in cells
   * @param rasterHeight height of raster in cells
   * @param cellWidth width of cell in mm
   * @param cellHeight height of cell in mm
   * @param rasterCenterOffsetX X offset of raster center in cells
   * @param rasterCenterOffsetY Y offset of raster center in cells
   *
   * @return positions [row][column][x,y] in mm
   */
  private static double[][][] newPositions(int rasterWidth, int rasterHeight, String cellWidth,
      String cellHeight, int rasterCenterOffsetX, int rasterCenterOffsetY) {
    return newPositions(BigDecimal.valueOf(rasterWidth), BigDecimal.valueOf(rasterHeight),
        new BigDecimal(cellWidth), new BigDecimal(cellHeight),
        BigDecimal.valueOf(rasterCenterOffsetX), BigDecimal.valueOf(rasterCenterOffsetY));
  }

  /**
   * @param rasterWidth width of raster in cells
   * @param rasterHeight height of raster in cells
   * @param cellWidth width of cell in mm
   * @param cellHeight height of cell in mm
   * @param rasterCenterOffsetX X offset of raster center in cells
   * @param rasterCenterOffsetY Y offset of raster center in cells
   *
   * @return positions [row][column][x,y] in mm
   */
  private static double[][][] newPositions(BigDecimal rasterWidth, BigDecimal rasterHeight,
      BigDecimal cellWidth, BigDecimal cellHeight, BigDecimal rasterCenterOffsetX,
      BigDecimal rasterCenterOffsetY) {
    final BigDecimal half = new BigDecimal("0.5");
    final int numCols = rasterWidth.intValue();
    final int numRows = rasterHeight.intValue();
    double[][][] result = new double[numRows][numCols][2];
    BigDecimal minX =
        cellWidth.multiply(rasterWidth.multiply(half)).negate().add(cellWidth.multiply(half))
            .add(rasterCenterOffsetX.multiply(cellWidth).negate());
    BigDecimal minY =
        cellHeight.multiply(rasterHeight.multiply(half)).negate().add(cellHeight.multiply(half))
            .add(rasterCenterOffsetY.multiply(cellHeight).negate());
    for (int r = 0; r < numRows; r++) {
      double y = minY.add(cellHeight.multiply(BigDecimal.valueOf(r))).doubleValue();
      for (int c = 0; c < numCols; c++) {
        result[r][c][0] = minX.add(cellWidth.multiply(BigDecimal.valueOf(c))).doubleValue();
        result[r][c][1] = y;
      }
    }
    return result;
  }

  /**
   * @param positions [row][column][x,y] in mm
   * @param x X position to find
   * @param y Y position to find
   * @param delta consider positions to be equal if the absolute value of their
   *          difference is <= {@code delta}
   *
   * @return index of position found or {@code null} if not found
   */
  private static Raster2dIndex findPosition(double[][][] positions, double x, double y,
      double delta) {
    for (int r = 0; r < positions.length; r++) {
      for (int c = 0; c < positions[0].length; c++) {
        if ((Math.abs(positions[r][c][0] - x) <= delta) &&
            (Math.abs(positions[r][c][1] - y) <= delta)) {
          return new Raster2dIndex(c, r);
        }
      }
    }
    return null;
  }
}
