package imcacat.imcadr.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

/**
 * I test <code>MathUtils</code>.
 */
public class MathUtilsTest {
  @Test
  public void testScale01() {
    assertEquals(50, MathUtils.scale(5, 0, 10, 0, 100));
    assertEquals(50, MathUtils.scale(0, -5, 5, 0, 100));
    assertEquals(0, MathUtils.scale(0, 0, 10, 0, 100));
    assertEquals(100, MathUtils.scale(10, 0, 10, 0, 100));
    assertEquals(25, MathUtils.scale(1, 0, 4, 0, 100));
    assertEquals(3, MathUtils.scale(75, 0, 100, 0, 4));
  }

  @Test
  public void testScale02() {
    assertEquals(50.0, MathUtils.scale(5.0, 0.0, 10.0, 0.0, 100.0), 1e-8);
    assertEquals(50.0, MathUtils.scale(0.0, -5.0, 5.0, 0.0, 100.0), 1e-8);
    assertEquals(0.0, MathUtils.scale(0.0, 0.0, 10.0, 0.0, 100.0), 1e-8);
    assertEquals(100.0, MathUtils.scale(10.0, 0.0, 10.0, 0.0, 100.0), 1e-8);
    assertEquals(25.0, MathUtils.scale(1.0, 0.0, 4.0, 0.0, 100.0), 1e-8);
    assertEquals(3.0, MathUtils.scale(75.0, 0.0, 100.0, 0.0, 4.0), 1e-8);
  }

  /* Value out of range. */
  @Test
  public void testScale03() {
    assertEquals(510.0, MathUtils.scale(123456789.0, 0.0, 150.0, 0.0, 510.0), 1e-8);
    assertEquals(510.0, MathUtils.scale(1234567890.0, 0.0, 150.0, 0.0, 510.0), 1e-8);
    assertEquals(510.0, MathUtils.scale(Double.MAX_VALUE, 0.0, 150.0, 0.0, 510.0), 1e-8);
  }

  @Test
  public void testEven01() {
    boolean even = true;
    for (int i = -6; i <= 6; i++) {
      if (even) assertTrue(MathUtils.even(i));
      else assertFalse(MathUtils.even(i));
      even = !even;
    }
  }

  @Test
  public void testOdd01() {
    boolean even = true;
    for (int i = -6; i <= 6; i++) {
      if (even) assertFalse(MathUtils.odd(i));
      else assertTrue(MathUtils.odd(i));
      even = !even;
    }
  }

  @Test
  public void testRound01() {
    assertEquals(7.595, MathUtils.round(7.595001, 3), 0.0);
    assertEquals(7.596, MathUtils.round(7.595601, 3), 0.0);
  }
}
