package imcacat.imcadr.util;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

/**
 * I test <code>NumberUtils</code>.
 */
public class NumberUtilsTest {
  /* Reduced precision. */
  @Test
  public void testToDecimalNotationString01() {
    assertEquals("1.234", NumberUtils.toDecimalNotationString(1.234, 3, 5));
    assertEquals("1.23", NumberUtils.toDecimalNotationString(1.234, 2, 5));
    assertEquals("1.2", NumberUtils.toDecimalNotationString(1.234, 1, 5));
    assertEquals("1", NumberUtils.toDecimalNotationString(1.234, 0, 5));
  }

  /* Reduced max character count. */
  @Test
  public void testToDecimalNotationString02() {
    assertEquals("1.23", NumberUtils.toDecimalNotationString(1.234, 3, 4));
    assertEquals("1.2", NumberUtils.toDecimalNotationString(1.234, 3, 3));
    assertEquals("1", NumberUtils.toDecimalNotationString(1.234, 3, 2));
    assertEquals("1", NumberUtils.toDecimalNotationString(1.234, 3, 1));
  }

  /* Greater than max character count, but cannot be shortened anymore. */
  @Test
  public void testToDetestToDecimalNotationString03() {
    assertEquals("123456", NumberUtils.toDecimalNotationString(123456.0, 0, 5));
  }

  /* Zero. */
  @Test
  public void testToDetestToDecimalNotationString04() {
    assertEquals("0.000", NumberUtils.toDecimalNotationString(0.0, 3, 5));
    assertEquals("0", NumberUtils.toDecimalNotationString(0.0, 0, 5));
  }

  /* Negative. */
  @Test
  public void testToDetestToDecimalNotationString05() {
    assertEquals("-1.234", NumberUtils.toDecimalNotationString(-1.234, 3, 6));
    assertEquals("-1.23", NumberUtils.toDecimalNotationString(-1.234, 3, 5));
    assertEquals("-1.2", NumberUtils.toDecimalNotationString(-1.234, 3, 4));
    assertEquals("-1", NumberUtils.toDecimalNotationString(-1.234, 3, 3));
    assertEquals("-1", NumberUtils.toDecimalNotationString(-1.234, 3, 2));
    assertEquals("-1", NumberUtils.toDecimalNotationString(-1.234, 3, 1));
  }

  /* Reduced max character count. */
  @Test
  public void testToScientificNotationString01() {
    assertEquals("1.234e0", NumberUtils.toScientificNotationString(1.234, 3, 7));
    assertEquals("1.23e0", NumberUtils.toScientificNotationString(1.234, 3, 6));
    assertEquals("1.2e0", NumberUtils.toScientificNotationString(1.234, 3, 5));
    assertEquals("1e0", NumberUtils.toScientificNotationString(1.234, 3, 4));
    assertEquals("1e0", NumberUtils.toScientificNotationString(1.234, 3, 3));
    assertEquals("1e0", NumberUtils.toScientificNotationString(1.234, 3, 2));
    assertEquals("1e0", NumberUtils.toScientificNotationString(1.234, 3, 1));
    assertEquals("1.234e12", NumberUtils.toScientificNotationString(1.234e12, 3, 8));
    assertEquals("1.23e12", NumberUtils.toScientificNotationString(1.234e12, 3, 7));
    assertEquals("1.2e12", NumberUtils.toScientificNotationString(1.234e12, 3, 6));
    assertEquals("1e12", NumberUtils.toScientificNotationString(1.234e12, 3, 5));
    assertEquals("1e12", NumberUtils.toScientificNotationString(1.234e12, 3, 4));
    assertEquals("1e12", NumberUtils.toScientificNotationString(1.234e12, 3, 3));
  }

  /* Negative. */
  @Test
  public void testToScientificNotationString02() {
    assertEquals("-1.234e0", NumberUtils.toScientificNotationString(-1.234, 3, 8));
  }

  /* Negative power. */
  @Test
  public void testToScientificNotationString03() {
    assertEquals("-1.234e-1", NumberUtils.toScientificNotationString(-1.234e-1, 3, 9));
    assertEquals("-1.234e-12", NumberUtils.toScientificNotationString(-1.234e-12, 3, 10));
  }

  /* Special. */
  @Test
  public void testToScientificNotationString04() {
    assertEquals("NaN", NumberUtils.toScientificNotationString(Double.NaN, 3, 3));
    assertEquals("-Infinity",
        NumberUtils.toScientificNotationString(Double.NEGATIVE_INFINITY, 3, 9));
    assertEquals("Infinity",
        NumberUtils.toScientificNotationString(Double.POSITIVE_INFINITY, 3, 9));
  }

  /* Basic. */
  @Test
  public void testToString01() {
    assertEquals("9999", NumberUtils.toString(9999.0, 0, 10000.0, 4));
    assertEquals("1e4", NumberUtils.toString(10000.0, 0, 10000.0, 4));
    assertEquals("1.2e4", NumberUtils.toString(12000.0, 1, 10000.0, 5));
  }
}
