package imcacat.imcadr.util;

import static org.junit.Assert.assertEquals;

import java.util.Arrays;

import org.junit.Test;

/**
 * I test <code>SimpleStatistics</code>.
 */
public class SimpleStatisticsTest {
  @Test
  public void testCompute_empty() {
    Double[] values = new Double[0];
    SimpleStatistics s = SimpleStatistics.compute(Arrays.asList(values));
    assertEquals(0, s.count());
    assertEquals(0.0, s.min(), 0.0);
    assertEquals(0.0, s.max(), 0.0);
    assertEquals(0.0, s.sum(), 0.0);
    assertEquals(0.0, s.mean(), 0.0);
    assertEquals(0.0, s.stddev(), 0.0);
  }

  @Test
  public void testCompute_oneValue() {
    /* @formatter:off */
    Double[] values = new Double[] {2.3};
    /* @formatter:on */
    SimpleStatistics s = SimpleStatistics.compute(Arrays.asList(values));
    assertEquals(1, s.count());
    assertEquals(2.3, s.min(), 0.0);
    assertEquals(2.3, s.max(), 0.0);
    assertEquals(2.3, s.sum(), 0.0);
    assertEquals(2.3, s.mean(), 0.0);
    assertEquals(0.0, s.stddev(), 0.0);
  }

  @Test
  public void testCompute_basic() {
    /* @formatter:off */
    Double[] values = new Double[] {600.0, 470.0, 170.0, 430.0, 300.0};
    /* @formatter:on */
    SimpleStatistics s = SimpleStatistics.compute(Arrays.asList(values));
    assertEquals(5, s.count());
    assertEquals(170.0, s.min(), 0.0);
    assertEquals(600.0, s.max(), 0.0);
    assertEquals(1970.0, s.sum(), 0.0);
    assertEquals(394.0, s.mean(), 0.0);
    assertEquals(147.3, s.stddev(), 0.1);
  }

  @Test
  public void testCompute_basicWithNegative() {
    /* @formatter:off */
    Double[] values = new Double[] {-5.0, 1.0, 8.0, 7.0, 2.0};
    /* @formatter:on */
    SimpleStatistics s = SimpleStatistics.compute(Arrays.asList(values));
    assertEquals(5, s.count());
    assertEquals(-5.0, s.min(), 0.0);
    assertEquals(8.0, s.max(), 0.0);
    assertEquals(13.0, s.sum(), 0.0);
    assertEquals(2.6, s.mean(), 0.0);
    assertEquals(4.7, s.stddev(), 0.1);
  }
}
