package imcacat.imcadr;

import java.awt.Color;
import java.io.PrintWriter;

import org.apache.commons.lang.StringEscapeUtils;

/**
 * I am a base class for rasterizer printers printing to a file as HTML.
 */
public abstract class BaseRasterizerPrinter {
  protected static final Color TEXT_COLOR = Color.BLACK;
  protected static final Color BACKGROUND_COLOR = Color.WHITE;

  protected PrintWriter writer;

  public BaseRasterizerPrinter(PrintWriter writer) {
    this.writer = writer;
  }

  protected void printHeader() {
    writer.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\"");
    writer.println("  \"http://www.w3.org/TR/html4/strict.dtd\">");
    writer.println("<html lang=\"en-us\">");
    writer.println("<head>");
    writer.println("<title>Imcadr Result</title>");
    writer.println("<meta http-equiv=\"Content-Type\" "
        + "content=\"text/html; charset=UTF-8\">");
    writer.println("<style type=\"text/css\">");
    writer.println("body {");
    writer.format("  color: #%02x%02x%02x;%n", TEXT_COLOR.getRed(), TEXT_COLOR.getGreen(),
        TEXT_COLOR.getBlue());
    writer.format("  background-color: #%02x%02x%02x;%n", BACKGROUND_COLOR.getRed(),
        BACKGROUND_COLOR.getGreen(), BACKGROUND_COLOR.getBlue());
    writer.println("}");
    writer.println();
    writer.println("div.Diagnostics {");
    writer.println("  color: gray;");
    writer.println("  font-size: smaller;");
    writer.println("}");
    writer.println();
    writer.println("div.Warning {");
    writer.println("  border-style: solid;");
    writer.println("  border-color: red;");
    writer.println("  border-width: 1px;");
    writer.println("  padding: 1em 1em 1em 0em;");
    writer.println("  font-family: monospace;");
    writer.println("  color: red;");
    writer.println("}");
    writer.println();
    writer.println("div.Warning > ul {");
    writer.println("  margin: 0px;");
    writer.println("}");
    writer.println();
    writer.println("h1 {");
    writer.println("  font-size: x-large;");
    writer.println("}");
    writer.println();
    writer.println("hr {");
    writer.println("  border: 1px solid silver;");
    writer.println("}");
    writer.println();
    writer.println("table.Raster {");
    writer.println("  table-layout: fixed;");
    writer.println("  margin-bottom: 1em;");
    writer.println("  border-collapse: collapse;");
    writer.println("}");
    writer.println();
    writer.println("table.Raster td {");
    writer.println("  border-style: solid;");
    writer.println("  border-color: black;");
    writer.println("  border-width: 1px;");
    writer.println("  color: white;");
    writer.println("  text-align: center;");
    writer.println("  vertical-align: middle;");
    writer.println("  padding-top: 2px; /* For better middle "
        + "vertical-alignment in Firefox 3.6 */");
    writer.println("}");
    writer.println("</style>");
    writer.println("</head>");
    writer.println("<body>");
    writer.println();
  }

  protected void printFooter() {
    writer.println();
    writer.println("</body>");
    writer.println("</html>");
  }

  protected void printWarningHeader() {
    writer.println("<div class=\"Warning\">");
    writer.println("<ul>");
  }

  protected void printWarningFooter() {
    writer.println("</ul>");
    writer.println("</div>");
  }

  protected void printDiagnosticsHeader() {
    writer.println("<div class=\"Diagnostics\">");
    writer.println("<p>Diagnostics</p>");
  }

  protected void printDiagnosticsFooter() {
    writer.println("</div>");
  }

  protected void printUnsuccessfulWarning(Rasterizer r) {
    writer.print("<li>Note: Rasterization did not complete successfully.");
    if (r.hasException()) {
      Throwable t = r.getException();
      String m = t.getMessage();
      if (m != null) {
        writer.print(" The error was, <q>");
        writer.print(escape(m));
        writer.print(".</q>");
      }
    }
    writer.println("</li>");
  }

  protected void printDivider() {
    writer.println();
    writer.println("<hr>");
    writer.println();
  }

  protected String escape(String value) {
    return StringEscapeUtils.escapeHtml(value);
  }
}
