package imcacat.imcadr;

/**
 * An exception indicating that the best score in a raster is less than or
 * equal to a best-score cutoff.
 */
public class BestScoreCutoffException extends Exception {
  private static final long serialVersionUID = 1L;

  private final Integer score;
  private final Integer cutoff;

  public BestScoreCutoffException(Integer score, Integer cutoff) {
    super(createMessage(score, cutoff));
    if (score != null && cutoff != null && score > cutoff) {
      throw new IllegalArgumentException("Score must be <= cutoff");
    }
    this.score = score;
    this.cutoff = cutoff;
  }

  public Integer score() {
    return score;
  }

  public Integer cutoff() {
    return cutoff;
  }

  private static String createMessage(Integer score, Integer cutoff) {
    StringBuilder result = new StringBuilder();
    result.append("Best score");
    if (score != null) {
      result.append(" (");
      result.append(score);
      result.append(")");
    }
    result.append(" <= cutoff");
    if (cutoff != null) {
      result.append(" (");
      result.append(cutoff);
      result.append(")");
    }
    return result.toString();
  }
}
