package imcacat.imcadr;

import java.awt.Dimension;
import java.io.File;

/**
 * I model this application's command line options.
 */
public class CliArguments {
  private boolean help = false;
  private boolean version = false;
  private boolean server = false;
  private File configurationFile;
  private File debugFile;
  private Double exposureTime;
  private Dimension cellSize;
  private Dimension zySize;
  private Dimension zyPreClipSize;
  private Raster2dMask zyMask;
  private Dimension zxSize;
  private Dimension zxPreClipSize;
  private Raster2dMask zxMask;
  private boolean zxFirst = false;
  private boolean flyScanEnabled = true;
  private boolean chasingEnabled = true;
  private Integer bestScoreCutoff;
  private AnalysisPackage analysisPackage;
  private boolean weakDiffracter = false;
  private Double highResolutionCutoff;
  private Double lowResolutionCutoff;
  private Integer minimumSpotArea;
  private Double minimumSignalHeight;
  private File imagesDirectory;

  public File configurationFile() {
    return configurationFile;
  }

  public void configurationFile(File configurationFile) {
    this.configurationFile = configurationFile;
  }

  public File debugFile() {
    return debugFile;
  }

  public void debugFile(File debugFile) {
    this.debugFile = debugFile;
  }

  public boolean help() {
    return help;
  }

  public void help(boolean help) {
    this.help = help;
  }

  public boolean version() {
    return version;
  }

  public void version(boolean version) {
    this.version = version;
  }

  public boolean server() {
    return server;
  }

  public void server(boolean server) {
    this.server = server;
  }

  public Double exposureTime() {
    return exposureTime;
  }

  public void exposureTime(Double exposureTime) {
    this.exposureTime = exposureTime;
  }

  public Dimension cellSize() {
    return cellSize;
  }

  public void cellSize(Dimension cellSize) {
    this.cellSize = cellSize;
  }

  public Dimension zySize() {
    return zySize;
  }

  public void zySize(Dimension zySize) {
    this.zySize = zySize;
  }

  public Dimension zyPreClipSize() {
    return zyPreClipSize;
  }

  public void zyPreClipSize(Dimension zyPreClipSize) {
    this.zyPreClipSize = zyPreClipSize;
  }

  public Raster2dMask zyMask() {
    return zyMask;
  }

  public void zyMask(Raster2dMask zyMask) {
    this.zyMask = zyMask;
  }

  public Dimension zxSize() {
    return zxSize;
  }

  public void zxSize(Dimension zxSize) {
    this.zxSize = zxSize;
  }

  public Dimension zxPreClipSize() {
    return zxPreClipSize;
  }

  public void zxPreClipSize(Dimension zxPreClipSize) {
    this.zxPreClipSize = zxPreClipSize;
  }

  public Raster2dMask zxMask() {
    return zxMask;
  }

  public void zxMask(Raster2dMask zxMask) {
    this.zxMask = zxMask;
  }

  public boolean zxFirst() {
    return zxFirst;
  }

  public void zxFirst(boolean zxFirst) {
    this.zxFirst = zxFirst;
  }

  public boolean flyScanEnabled() {
    return flyScanEnabled;
  }

  public void flyScanEnabled(boolean flyScanEnabled) {
    this.flyScanEnabled = flyScanEnabled;
  }

  public boolean chasingEnabled() {
    return chasingEnabled;
  }

  public void chasingEnabled(boolean chasingEnabled) {
    this.chasingEnabled = chasingEnabled;
  }

  public Integer bestScoreCutoff() {
    return bestScoreCutoff;
  }

  public void bestScoreCutoff(Integer bestScoreCutoff) {
    this.bestScoreCutoff = bestScoreCutoff;
  }

  public AnalysisPackage analysisPackage() {
    return analysisPackage;
  }

  public void analysisPackage(AnalysisPackage analysisPackage) {
    this.analysisPackage = analysisPackage;
  }

  public boolean weakDiffracter() {
    return weakDiffracter;
  }

  public void weakDiffracter(boolean weakDiffracter) {
    this.weakDiffracter = weakDiffracter;
  }

  public Double highResolutionCutoff() {
    return highResolutionCutoff;
  }

  public void highResolutionCutoff(Double highResolutionCutoff) {
    this.highResolutionCutoff = highResolutionCutoff;
  }

  public Double lowResolutionCutoff() {
    return lowResolutionCutoff;
  }

  public void lowResolutionCutoff(Double lowResolutionCutoff) {
    this.lowResolutionCutoff = lowResolutionCutoff;
  }

  public Integer minimumSpotArea() {
    return minimumSpotArea;
  }

  public void minimumSpotArea(Integer minimumSpotArea) {
    this.minimumSpotArea = minimumSpotArea;
  }

  public Double minimumSignalHeight() {
    return minimumSignalHeight;
  }

  public void minimumSignalHeight(Double minimumSignalHeight) {
    this.minimumSignalHeight = minimumSignalHeight;
  }

  public File imagesDirectory() {
    return imagesDirectory;
  }

  public void imagesDirectory(File imagesDirectory) {
    this.imagesDirectory = imagesDirectory;
  }

  public int count() {
    int count = 0;
    if (help) count++;
    if (version) count++;
    if (server) count++;
    if (configurationFile != null) count++;
    if (debugFile != null) count++;
    if (exposureTime != null) count++;
    if (cellSize != null) count++;
    if (zySize != null) count++;
    if (zyPreClipSize != null) count++;
    if (zyMask != null) count++;
    if (zxSize != null) count++;
    if (zxPreClipSize != null) count++;
    if (zxMask != null) count++;
    if (!flyScanEnabled) count++;
    if (!chasingEnabled) count++;
    if (bestScoreCutoff != null) count++;
    if (analysisPackage != null) count++;
    if (weakDiffracter) count++;
    if (highResolutionCutoff != null) count++;
    if (lowResolutionCutoff != null) count++;
    if (minimumSpotArea != null) count++;
    if (minimumSignalHeight != null) count++;
    if (imagesDirectory != null) count++;
    return count;
  }

  public boolean isEmpty() {
    return count() == 0;
  }

  public boolean onlyHasImagesDirectory() {
    return count() == 1 && imagesDirectory != null;
  }

  public ImcadrConfiguration toConfiguration() {
    return ImcadrConfiguration.Builder.newWithDefaults().build();
  }
}
