package imcacat.imcadr;

import java.util.Objects;

/**
 * I am a node in a computer cluster.
 */
public class ClusterNode {
  private final String hostname;
  private final int jobCapacity;

  /**
   * @param hostname the network hostname of this node
   * @param jobCapacity the maximum number of jobs that this node can be
   *          executing at once
   *
   * @throws NullPointerException if {@code hostname} is {@code null}
   * @throws IllegalArgumentException if {@code hostname} is empty or
   *           {@code jobCapacity} is less than 1
   */
  public ClusterNode(String hostname, int jobCapacity) {
    super();
    Objects.requireNonNull(hostname);
    if (hostname.isEmpty()) {
      throw new IllegalArgumentException("hostname is empty");
    }
    this.hostname = hostname;
    if (jobCapacity < 1) {
      throw new IllegalArgumentException("jobCapacity < 1");
    }
    this.jobCapacity = jobCapacity;
  }

  public String hostname() {
    return hostname;
  }

  public int jobCapacity() {
    return jobCapacity;
  }

  @Override
  public boolean equals(Object o) {
    if (o == this) return true;
    if (!(o instanceof ClusterNode)) return false;
    ClusterNode n = (ClusterNode)o;
    if (!hostname.equals(n.hostname)) return false;
    return jobCapacity == n.jobCapacity;
  }

  @Override
  public int hashCode() {
    int result = 17;
    result = 31 * result + hostname.hashCode();
    result = 31 * result + jobCapacity;
    return result;
  }

  @Override
  public String toString() {
    return (new StringBuilder())
        .append("{")
        .append("hostname=")
        .append(hostname)
        .append(", ")
        .append("jobCapacity=")
        .append(jobCapacity)
        .append("}")
        .toString();
  }
}
