package imcacat.imcadr;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import imcacat.imcadr.util.ShellUtils;

/**
 * I am an {@link ImageAnalysis} that uses
 * <a href="https://github.com/tlhrigaku/d-star-trek">d*TREK</a>.
 * <p>
 * I run the <tt>dtrek-dtranker</tt> helper program as if running
 * <tt>dtranker</tt>. The working directory is unspecified, so any file or
 * directory paths will be absolute. <tt>dtranker</tt> is eventually executed
 * with the arguments given to <tt>dtrek-dtranker</tt>.
 */
public class DTrekImageAnalysis extends ImageAnalysis {
  private static final Double DEFAULT_HIGH_RES_CUTOFF = Double.valueOf(0.5);
  private static final Double DEFAULT_LOW_RES_CUTOFF = Double.valueOf(99.0);

  public DTrekImageAnalysis(ImageAnalysisSpec spec, File image) {
    super(spec, image);
  }

  public DTrekImageAnalysis(ImageAnalysisSpec spec, File image, File clusterKeyFile,
      String clusterNode) {
    super(spec, image, clusterKeyFile, clusterNode);
  }

  @Override
  protected List<String> createCommand() {
    ArrayList<String> result = new ArrayList<String>();
    result.add(Resources.getInstance().getDTrekDtrankerHelper().getPath());
    Map<String, String> varNamesToValues = new HashMap<>(1);
    varNamesToValues.put("image", image.getAbsolutePath());
    for (String each : spec.getAnalysisProgramArguments(AnalysisProgram.DTREK_DTRANKER)) {
      if (each.equals("$more") || each.equals("${more}")) {
        if (spec.getHighResolutionCutoff() != null || spec.getLowResolutionCutoff() != null) {
          Double high = spec.getHighResolutionCutoff();
          if (high == null) high = DEFAULT_HIGH_RES_CUTOFF;
          Double low = spec.getLowResolutionCutoff();
          if (low == null) low = DEFAULT_LOW_RES_CUTOFF;
          result.add(format("reso=%.2f %.2f", low, high));
        }
      } else {
        result.add(ShellUtils.expand(each, varNamesToValues));
      }
    }
    return result;
  }

  @Override
  protected ImageAnalysisParser newAnalysisParser(InputStream stream) {
    return new DTrekDtrankerParser(stream);
  }
}
