package imcacat.imcadr;

import java.io.File;

/**
 * I am an interface for controlling a detector.
 */
public interface DetectorDriver {
  /**
   * Gets the readout time in seconds with the specified binning enablement.
   *
   * @param binningEnabled whether the readout time returned is for the
   *          binning-enabled mode of the detector; {@code true} indicates the
   *          binning-enabled mode, and {@code false} indicates the
   *          binning-disabled mode (i.e. no binning)
   *
   * @return the readout time in seconds
   */
  double getReadoutTime(boolean binningEnabled);

  /**
   * Gets the additional time to wait for an exposure to end after it should
   * have ended.
   *
   * @return the exposure timeout in seconds
   */
  double getExposureTimeout();

  void setRotationAxis(CrystalRotationAxis axis) throws InterruptedException, DriverException;

  void setBinningEnabled(boolean enabled) throws InterruptedException, DriverException;

  double getExposureTime() throws InterruptedException, DriverException;

  void setExposureTime(double exposureTime) throws InterruptedException, DriverException;

  double getExposurePeriod() throws InterruptedException, DriverException;

  void setExposurePeriod(double exposurePeriod) throws InterruptedException, DriverException;

  int getNumberOfImages() throws InterruptedException, DriverException;

  void setNumberOfImages(int numberOfImages) throws InterruptedException, DriverException;

  void setImageFilePath(File imageFilePath) throws InterruptedException, DriverException;

  void setPhi(double position) throws InterruptedException, DriverException;

  void setDetectorDistance(double position) throws InterruptedException, DriverException;

  void setDetector2theta(double position) throws InterruptedException, DriverException;

  void setWavelength(double position) throws InterruptedException, DriverException;

  void setAttenuation(double attenuation) throws InterruptedException, DriverException;

  void setImageWidth(double width) throws InterruptedException, DriverException;

  void lastImage() throws InterruptedException, DriverException;

  void startDataset() throws InterruptedException, DriverException;

  void stopDataset() throws InterruptedException, DriverException;

  void startExposing() throws InterruptedException, DriverException;

  boolean stopExposing() throws InterruptedException, DriverException;

  boolean waitForExposureToEnd(long timeout) throws InterruptedException, DriverException;
}
