package imcacat.imcadr;

import java.io.File;
import java.text.ParseException;

import javax.swing.text.DefaultFormatter;

/**
 * I am a formatter that formats a <code>File</code> object representing a file
 * system directory as a <code>String</code>. I am suitable for use by a GUI
 * component that wants to display a <code>File</code> object.
 */
public class DirectoryFormatter extends DefaultFormatter {
  private static final long serialVersionUID = 1;

  protected boolean isEmptyAllowed;

  public DirectoryFormatter() {
    this(false);
  }

  public DirectoryFormatter(boolean isEmptyAllowed) {
    super();
    this.isEmptyAllowed = isEmptyAllowed;
  }

  public Object stringToValue(String directoryString) throws ParseException {
    File directory = new File(directoryString);
    if (isEmptyAllowed && directoryString.equals("")) return directory;
    if (directory.isDirectory()) return directory;
    throw new ParseException("Directory does not exist", directoryString.length());
  }

  public String valueToString(Object directory) throws ParseException {
    return directory == null ? "" : ((File)directory).getPath();
  }
}
