package imcacat.imcadr;

import javax.swing.SwingUtilities;

import com.jgoodies.binding.beans.Model;

/**
 * I am just like {@link com.jgoodies.binding.beans.Model} except I am able to
 * fire my property changes on the EDT (Event Dispatch Thread) if so
 * configured.
 */
public class EdtAwareModel extends Model {
  private static final long serialVersionUID = 1;

  protected boolean firePropertyChangesOnEdtEnabled;

  public EdtAwareModel(boolean firePropertyChangesOnEdtEnabled) {
    super();
    this.firePropertyChangesOnEdtEnabled = firePropertyChangesOnEdtEnabled;
  }

  /**
   * Provides behavior like {@link #firePropertyChange(String, Object, Object)}
   * , but fires the property change notification from the Event Dispatch
   * Thread if this instance is configured to do so.
   */
  protected void notifyChanged(String propertyName, Object oldValue, Object newValue) {
    if (firePropertyChangesOnEdtEnabled && !SwingUtilities.isEventDispatchThread()) {
      notifyChangedOnEdt(propertyName, oldValue, newValue);
    } else {
      firePropertyChange(propertyName, oldValue, newValue);
    }
  }

  /**
   * Invokes {@link #firePropertyChange(String, Object, Object)} from the Event
   * Dispatch Thread.
   */
  protected void notifyChangedOnEdt(final String propertyName, final Object oldValue,
      final Object newValue) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        firePropertyChange(propertyName, oldValue, newValue);
      }
    });
  }
}
