package imcacat.imcadr;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;
import java.util.regex.Pattern;

/**
 * I parse the output of an {@link ImageAnalysis}.
 */
public abstract class ImageAnalysisParser {
  protected final InputStream in;

  protected ImageAnalysisParser(InputStream in) {
    super();
    this.in = in;
  }

  public abstract ImageAnalysisResult run();

  protected static Pattern newPattern(String re) {
    return Pattern.compile(re, Pattern.CASE_INSENSITIVE);
  }

  protected static ImageAnalysisResult newAnalysisResult(String errorMessage, Throwable cause) {
    ImageAnalysisResult result = new ImageAnalysisResult();
    result.setError(errorMessage, cause);
    return result;
  }

  protected static void closeSilently(Closeable closeable) {
    if (closeable == null) return;
    try {
      closeable.close();
    } catch (InterruptedIOException e) {
      Thread.currentThread().interrupt();
    } catch (IOException | RuntimeException e) {
      /* IGNORED */
    }
  }
}
