package imcacat.imcadr;

import java.io.File;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import imcacat.epicsca4j.util.ObjectUtilities;

import com.jgoodies.binding.beans.Model;

/**
 * I am the model for this application.
 */
public class Imcadr extends Model {
  private static final long serialVersionUID = 1;

  private static final ExecutorService RASTERIZER_EXECUTOR_SERVICE;

  public static final String PROPNAME_OPERATION = "operation";
  public static final String PROPNAME_RASTERIZER_WORKING_DIRECTORY =
      "rasterizerWorkingDirectory";
  public static final String PROPNAME_STATUS = "status";

  static {
    RASTERIZER_EXECUTOR_SERVICE = Executors.newSingleThreadExecutor();
    Runtime.getRuntime().addShutdownHook(new Thread() {
      @Override
      public void run() {
        RASTERIZER_EXECUTOR_SERVICE.shutdownNow();
        try {
          RASTERIZER_EXECUTOR_SERVICE.awaitTermination(15L, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
          /* INTERRUPTED */
        }
      }
    });
  }

  protected Operation operation;
  protected File rasterizerWorkingDirectory;
  protected ImcadrSettings settings;
  protected BeamlineDriver beamlineDriver;
  protected String status;

  public Imcadr() {
    this(new BaseBeamlineDriver());
  }

  public Imcadr(BeamlineDriver driver) {
    operation = Operation.IDLE;
    rasterizerWorkingDirectory = Resources.getInstance().getRasterizerWorkingDirectory();
    settings = new ImcadrSettings();
    beamlineDriver = driver;
    status = "";
  }

  public Operation getOperation() {
    return operation;
  }

  public void setOperation(Operation newValue) {
    Operation oldValue = operation;
    operation = newValue;
    if (!ObjectUtilities.areEqual(oldValue, newValue)) {
      firePropertyChange(Imcadr.PROPNAME_OPERATION, oldValue, newValue);
    }
  }

  public File getRasterizerWorkingDirectory() {
    return rasterizerWorkingDirectory;
  }

  public void setRasterizerWorkingDirectory(File newValue) {
    File oldValue = rasterizerWorkingDirectory;
    rasterizerWorkingDirectory = newValue;
    if (!ObjectUtilities.areEqual(oldValue, newValue)) {
      firePropertyChange(Imcadr.PROPNAME_RASTERIZER_WORKING_DIRECTORY, oldValue, newValue);
    }
  }

  public ImcadrSettings getSettings() {
    return settings;
  }

  public String getStatus() {
    return status;
  }

  public void setStatus(String newStatus) {
    String oldStatus = status;
    status = newStatus;
    if (!ObjectUtilities.areEqual(oldStatus, newStatus)) {
      firePropertyChange(Imcadr.PROPNAME_STATUS, oldStatus, newStatus);
    }
  }

  public BeamlineDriver getBeamlineDriver() {
    return beamlineDriver;
  }

  public Future<?> submit(Runnable task) {
    return RASTERIZER_EXECUTOR_SERVICE.submit(task);
  }
}
