package imcacat.imcadr;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.StringUtilities;
import imcacat.imcadr.ChooseDirectoryAction;
import imcacat.imcadr.ComponentFactory;
import imcacat.imcadr.DirectoryFormatter;
import imcacat.imcadr.ProgressBarMediator;
import imcacat.imcadr.util.Pair;
import imcacat.imcadr.Resources;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ConstantSize;
import com.jgoodies.forms.layout.FormLayout;

import net.roydesign.app.Application;

/**
 * I am the main frame (i.e.&nbsp;window) for this application.
 */
public class ImcadrFrame extends JFrame {
  public static final String FRAME_TITLE = Resources.getApplicationName();

  private static final long serialVersionUID = 1;

  protected ImcadrPresentationModel model;

  protected Action aboutAction;
  protected Action preferencesAction;
  protected Action quitAction;

  protected JDialog aboutDialog;
  protected JButton chooseRasterizerWorkingDirButton;
  protected ProgressBarMediator rasterizerProgressBarMediator;

  public ImcadrFrame() throws InterruptedException, EpicsCaException {
    this(new ImcadrPresentationModel());
  }

  public ImcadrFrame(ImcadrPresentationModel model) throws InterruptedException,
      EpicsCaException {
    super();
    this.model = model;

    initializeActions();
    initializeSpecialMenuBar();
    initializeFrame();
  }

  public void release() {
    rasterizerProgressBarMediator.release();
  }

  public void requestPreferredInitialFocus() {
    chooseRasterizerWorkingDirButton.requestFocusInWindow();
  }

  protected void initializeActions() {
    aboutAction = new AbstractAction() {
      private static final long serialVersionUID = 1;

      public void actionPerformed(ActionEvent e) {
        aboutActionPerformed();
      }
    };
    aboutAction.putValue(Action.NAME, "About " + Resources.getApplicationName());

    preferencesAction =
        new OpenImcadrSettingsDialogAction(this, "Preferences...", "Preferences", model
            .getBean().getSettings());

    quitAction = new AbstractAction() {
      private static final long serialVersionUID = 1;

      public void actionPerformed(ActionEvent e) {
        quitActionPerformed();
      }
    };
    if (Resources.isPlatformWindows()) {
      quitAction.putValue(Action.NAME, "Exit");
    } else {
      quitAction.putValue(Action.NAME, "Quit");
      quitAction.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Q, Toolkit
          .getDefaultToolkit().getMenuShortcutKeyMask()));
    }
  }

  protected JMenuBar buildMenuBar() {
    JMenuBar menuBar = new JMenuBar();

    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic(KeyEvent.VK_F);
    JMenuItem fileQuitMenuItem = new JMenuItem(quitAction);
    if (!Resources.isPlatformWindows()) fileQuitMenuItem.setMnemonic(KeyEvent.VK_Q);
    fileMenu.add(fileQuitMenuItem);
    menuBar.add(fileMenu);

    JMenu editMenu = new JMenu("Edit");
    editMenu.setMnemonic(KeyEvent.VK_E);
    JMenuItem editPreferencesMenuItem = new JMenuItem(preferencesAction);
    editPreferencesMenuItem.setMnemonic(KeyEvent.VK_P);
    editMenu.add(editPreferencesMenuItem);
    menuBar.add(editMenu);

    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic(KeyEvent.VK_H);
    JMenuItem helpAboutMenuItem = new JMenuItem(aboutAction);
    helpAboutMenuItem.setMnemonic(KeyEvent.VK_A);
    helpMenu.add(helpAboutMenuItem);
    menuBar.add(helpMenu);

    return menuBar;
  }

  /*
   * If running on Mac OS X, this makes the special Mac OS X Application About
   * and Quit menu items work as expected.
   */
  protected void initializeSpecialMenuBar() {
    Application mrjApp = Application.getInstance();
    mrjApp.getAboutJMenuItem().addActionListener(aboutAction);
    mrjApp.getPreferencesJMenuItem().addActionListener(preferencesAction);
    mrjApp.getQuitJMenuItem().addActionListener(quitAction);
  }

  protected void initializeFrame() throws InterruptedException, EpicsCaException {
    setTitle(getFrameTitle());
    setIconImage(getFrameIconImage());
    setJMenuBar(buildMenuBar());
    setContentPane(buildContentPane());
    setPreferredSize(new Dimension(600, 575));

    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        quitActionPerformed();
      }
    });
  }

  protected JComponent buildContentPane() {
    /* @formatter:off */
    FormLayout layout = new FormLayout(
        /* Columns */
        "fill:default:grow",
        /* Rows */
        "pref, fill:default:grow, pref");
    /* @formatter:on */
    PanelBuilder builder = new PanelBuilder(layout);
    CellConstraints cc = new CellConstraints();

    builder.add(buildControlPanel(), cc.xy(1, 1));
    builder.add(buildRasterizerPanel(), cc.xy(1, 2));
    builder.add(buildStatusPanel(), cc.xy(1, 3));

    return builder.getPanel();
  }

  protected JComponent buildControlPanel() {
    /* @formatter:off */
    FormLayout layout = new FormLayout(
        /* Columns */
        "right:default, 4dlu, fill:max(45dlu;default):grow, 4dlu, max(40dlu;default)",
        /* Rows */
        "pref, 2dlu, pref");
    layout.setRowGroups(
        new int[][] {
            /* Fields */
            {1, 3}
        });
    /* @formatter:on */
    PanelBuilder builder = new PanelBuilder(layout);
    builder.setBorder(Borders.DLU4_BORDER);
    CellConstraints cc = new CellConstraints();

    builder.addLabel("Raster images folder:", cc.xy(1, 1));
    JFormattedTextField rasterizerWorkingDirTextField =
        ComponentFactory.createFormattedTextField(
            model.getModel(Imcadr.PROPNAME_RASTERIZER_WORKING_DIRECTORY),
            new DirectoryFormatter());
    rasterizerWorkingDirTextField.setEditable(false);
    builder.add(rasterizerWorkingDirTextField, cc.xy(3, 1));
    Action chooseRasterizerWorkingDirAction =
        new ChooseDirectoryAction(this, "Choose...", "Raster images folder",
            model.getModel(Imcadr.PROPNAME_RASTERIZER_WORKING_DIRECTORY));
    chooseRasterizerWorkingDirButton = new JButton(chooseRasterizerWorkingDirAction);
    builder.add(chooseRasterizerWorkingDirButton, cc.xy(5, 1));

    Pair<JProgressBar, ProgressBarMediator> pair =
        ComponentFactory.createProgressBar(
            model.getRasterizerPresentationModel().getModel(Rasterizer3d.PROPNAME_PROGRESS),
            model.getRasterizerPresentationModel().getModel(Rasterizer3d.PROPNAME_STATUS));
    JProgressBar rasterizerProgressBar = pair.getFirst();
    rasterizerProgressBarMediator = pair.getSecond();
    builder.add(rasterizerProgressBar, cc.xy(3, 3));
    Action startStopAction = model.getRasterizerStartStopAction();
    builder.add(new JButton(startStopAction), cc.xy(5, 3));

    return builder.getPanel();
  }

  protected JComponent buildRasterizerPanel() {
    Rasterizer3dPanel rasterizerPanel =
        new Rasterizer3dPanel(model.getRasterizerPresentationModel());

    /* @formatter:off */
    FormLayout layout = new FormLayout(
        /* Columns */
        "left:pref:grow",
        /* Rows */
        "top:pref:grow");
    /* @formatter:on */
    PanelBuilder builder = new PanelBuilder(layout);
    CellConstraints cc = new CellConstraints();
    builder.add(rasterizerPanel, cc.xy(1, 1));

    JScrollPane rasterizerPane = new JScrollPane(builder.getPanel());
    rasterizerPane.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0, Color.GRAY));
    return rasterizerPane;
  }

  protected JComponent buildStatusPanel() {
    /* @formatter:off */
    FormLayout layout = new FormLayout(
        /* Columns */
        "left:default:grow",
        /* Rows */
        "center:max(10dlu;pref)");
    /* @formatter:on */
    PanelBuilder builder = new PanelBuilder(layout);
    builder.setBorder(Borders.createEmptyBorder(new ConstantSize(1, ConstantSize.DLUY),
        new ConstantSize(4, ConstantSize.DLUX), new ConstantSize(1, ConstantSize.DLUY),
        new ConstantSize(1, ConstantSize.DLUX)));
    CellConstraints cc = new CellConstraints();

    JLabel statusValueLabel =
        ComponentFactory.createLabel(model.getModel(Imcadr.PROPNAME_STATUS));
    builder.add(statusValueLabel, cc.xy(1, 1));

    return builder.getPanel();
  }

  protected String getFrameTitle() {
    return Resources.getApplicationName();
  }

  protected Image getFrameIconImage() {
    return Resources.getAppIcon().getImage();
  }

  protected void tellUserCannotExitBecauseOperationInProgress(Operation operation) {
    String operationName = Resources.getInstance().getDisplayString(operation);
    JOptionPane.showMessageDialog(this, StringUtilities.wordWrap(
        (StringUtilities.startsWithVowel(operationName) ? "An " : "A ") + operationName +
            " operation is still in progress. Please stop the " + "operation before quitting.",
        78, "\n"), "Operation in progress", JOptionPane.WARNING_MESSAGE);
  }

  protected void aboutActionPerformed() {
    if (aboutDialog == null) {
      aboutDialog =
          new AboutDialog(this, Resources.getApplicationName(),
              Resources.getApplicationVersion(), Resources.getLargeAppIcon());
      aboutDialog.setLocationRelativeTo(this);
    }
    aboutDialog.setVisible(true);
  }

  protected void quitActionPerformed() {
    Operation operation = (Operation)model.getModel(Imcadr.PROPNAME_OPERATION).getValue();
    if (Operation.IDLE.equals(operation)) basicExit(ExitStatus.SUCCESS.getStatus());
    else tellUserCannotExitBecauseOperationInProgress(operation);
  }

  protected String getApplicationVersion() {
    return Resources.getApplicationVersion();
  }

  protected static void basicExit(int exitStatus) {
    ImcadrSystem.basicExit(exitStatus);
  }

  public static ImcadrFrame onImagesDirectory(File imagesDir) throws InterruptedException,
      EpicsCaException {
    if (imagesDir == null) return new ImcadrFrame();
    Imcadr model = new Imcadr();
    model.setRasterizerWorkingDirectory(imagesDir);
    return new ImcadrFrame(new ImcadrPresentationModel(model));
  }
}
