package imcacat.imcadr;

import imcacat.epicsca4j.ca.EpicsCaException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.concurrent.Future;

import javax.swing.Action;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.beans.PropertyAdapter;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link ImcadrFrame}.
 */
public class ImcadrPresentationModel extends PresentationModel<Imcadr> {
  private static final long serialVersionUID = 1;

  protected static final String RASTERIZER_COMMAND_START = "start";
  protected static final String RASTERIZER_COMMAND_STOP = "stop";

  protected boolean rasterizerRunning = false;

  protected ValueModel rasterizerBeanChannel;
  protected ValueModel rasterizerBeanRunningModel;

  protected Rasterizer3dPresentationModel rasterizerPresentationModel;

  protected Future<?> rasterizerFuture;

  protected TwoStateButtonAction rasterizerStartStopAction;

  public ImcadrPresentationModel() throws InterruptedException, EpicsCaException {
    this(new Imcadr());
  }

  public ImcadrPresentationModel(Imcadr bean) throws InterruptedException, EpicsCaException {
    super(bean);

    rasterizerBeanChannel = new ValueHolder(createRasterizer(), true);
    rasterizerBeanRunningModel =
        new PropertyAdapter<Rasterizer>(rasterizerBeanChannel, Rasterizer.PROPNAME_RUNNING, true);
    rasterizerBeanRunningModel.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        setRasterizerRunning((Boolean)event.getNewValue());
      }
    });

    rasterizerStartStopAction = new TwoStateButtonAction("Start", "Stop");
    rasterizerStartStopAction.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        if (event.getActionCommand().equals(TwoStateButtonAction.STATE_1_COMMAND)) {
          startRasterizer();
        } else if (event.getActionCommand().equals(TwoStateButtonAction.STATE_2_COMMAND)) {
          stopRasterizer();
        }
      }
    });

    rasterizerPresentationModel =
        new Rasterizer3dPresentationModel(rasterizerBeanChannel,
            getModel(Imcadr.PROPNAME_STATUS));
  }

  public Action getRasterizerStartStopAction() {
    return rasterizerStartStopAction;
  }

  public Rasterizer3dPresentationModel getRasterizerPresentationModel() {
    return rasterizerPresentationModel;
  }

  protected boolean isRasterizerRunning() {
    return rasterizerRunning;
  }

  protected void setRasterizerRunning(boolean newRasterizerRunning) {
    if (rasterizerRunning == newRasterizerRunning) return;
    rasterizerRunning = newRasterizerRunning;
    if (rasterizerRunning) {
      rasterizerStartStopAction.setStateTo2();
      getBean().setOperation(Operation.RASTERIZE);
    } else {
      rasterizerStartStopAction.setStateTo1();
      getBean().setOperation(Operation.IDLE);
    }
  }

  protected void startRasterizer() {
    Rasterizer3d rasterizer = createRasterizer();
    rasterizerBeanChannel.setValue(rasterizer);
    rasterizerFuture = getBean().submit(rasterizer);
  }

  protected void stopRasterizer() {
    rasterizerFuture.cancel(true);
  }

  protected Rasterizer3d createRasterizer() {
    return new Rasterizer3d(createRasterSpec(), getBean().getBeamlineDriver(), true);
  }

  protected Raster3dSpec createRasterSpec() {
    ImcadrSettings settings = getBean().getSettings();
    ImageAnalysisSpec analysisSpec = createAnalysisSpec();
    return new Raster3dSpec(new Raster2dSpec(Raster2dPlane.ZY, settings.getZyRasterWidth(),
        settings.getZyRasterHeight(), settings.getRasterCellWidth(),
        settings.getRasterCellHeight(), LengthUnit.MICROMETER, settings.getExposureTime(),
        settings.getMaxNumConcurrentAnalyzers(), true, settings.isChasingEnabled(), getBean()
            .getRasterizerWorkingDirectory(), analysisSpec), new Raster2dSpec(Raster2dPlane.ZX,
        settings.getZxRasterWidth(), settings.getZxRasterHeight(),
        settings.getRasterCellWidth(), settings.getRasterCellHeight(), LengthUnit.MICROMETER,
        settings.getExposureTime(), settings.getMaxNumConcurrentAnalyzers(), true,
        settings.isChasingEnabled(), getBean().getRasterizerWorkingDirectory(), analysisSpec));
  }

  protected ImageAnalysisSpec createAnalysisSpec() {
    ImageAnalysisSpec.Builder b = new ImageAnalysisSpec.Builder();
    ImcadrSettings s = getBean().getSettings();
    if (s.isWeakDiffracter()) b.highResolutionCutoff(s.getHighResolutionCutoff());
    return b.build();
  }
}
