package imcacat.imcadr;

/**
 * I am the result of {@code ImcadrServer} running a {@code Rasterizer}. My
 * values are updated as the {@code Rasterizer} runs, not just once when it
 * finishes.
 * <p>
 * Units: progress [0-100], scores [0-100], estimated resolution 1 [angstroms],
 * estimated resolution 2 [angstroms], mosaicity [degrees], unit cell [lengths:
 * angstroms; angles: degrees], time [milliseconds].
 */
public class ImcadrServerResult {
  protected Rectangle sizeZy;
  protected Rectangle sizeZx;
  protected int progress;
  protected ImcadrServerState state;
  protected String message;
  protected Position2d originZy;
  protected Position2d originZx;
  protected Integer[][] scoresZy;
  protected Integer[][] scoresZx;
  protected Integer[][] spotTotalsZy;
  protected Integer[][] spotTotalsZx;
  protected Integer[][] inResTotalsZy;
  protected Integer[][] inResTotalsZx;
  protected Integer[][] goodBraggTotalsZy;
  protected Integer[][] goodBraggTotalsZx;
  protected Integer[][] iceRingTotalsZy;
  protected Integer[][] iceRingTotalsZx;
  protected Double[][] estimatedRes1Zy;
  protected Double[][] estimatedRes1Zx;
  protected Double[][] estimatedRes2Zy;
  protected Double[][] estimatedRes2Zx;
  protected Integer[][] totalIntSignalsZy;
  protected Integer[][] totalIntSignalsZx;
  protected Double[][] mosaicitiesZy;
  protected Double[][] mosaicitiesZx;
  protected SpaceGroup[][] spaceGroupsZy;
  protected SpaceGroup[][] spaceGroupsZx;
  protected UnitCell[][] unitCellsZy;
  protected UnitCell[][] unitCellsZx;
  protected Position2d centerZy;
  protected Position2d centerZx;
  protected Position3d center;
  protected Integer time;

  public ImcadrServerResult() {
    this(null, null);
  }

  public ImcadrServerResult(Rectangle sizeZy, Rectangle sizeZx) {
    this.sizeZy = (sizeZy == null) ? null : sizeZy.convertTo(LengthUnit.CELL);
    this.sizeZx = (sizeZx == null) ? null : sizeZx.convertTo(LengthUnit.CELL);
    progress = 0;
    state = ImcadrServerState.IDLE;
    message = null;
    originZy = (sizeZy == null) ? null : new Position2d(LengthUnit.MILLIMETER);
    originZx = (sizeZx == null) ? null : new Position2d(LengthUnit.MILLIMETER);
    scoresZy = create2dIntegerArray(this.sizeZy);
    scoresZx = create2dIntegerArray(this.sizeZx);
    spotTotalsZy = create2dIntegerArray(this.sizeZy);
    spotTotalsZx = create2dIntegerArray(this.sizeZx);
    inResTotalsZy = create2dIntegerArray(this.sizeZy);
    inResTotalsZx = create2dIntegerArray(this.sizeZx);
    goodBraggTotalsZy = create2dIntegerArray(this.sizeZy);
    goodBraggTotalsZx = create2dIntegerArray(this.sizeZx);
    iceRingTotalsZy = create2dIntegerArray(this.sizeZy);
    iceRingTotalsZx = create2dIntegerArray(this.sizeZx);
    estimatedRes1Zy = create2dDoubleArray(this.sizeZy);
    estimatedRes1Zx = create2dDoubleArray(this.sizeZx);
    estimatedRes2Zy = create2dDoubleArray(this.sizeZy);
    estimatedRes2Zx = create2dDoubleArray(this.sizeZx);
    totalIntSignalsZy = create2dIntegerArray(this.sizeZy);
    totalIntSignalsZx = create2dIntegerArray(this.sizeZx);
    mosaicitiesZy = create2dDoubleArray(this.sizeZy);
    mosaicitiesZx = create2dDoubleArray(this.sizeZx);
    spaceGroupsZy = create2dSpaceGroupArray(this.sizeZy);
    spaceGroupsZx = create2dSpaceGroupArray(this.sizeZx);
    unitCellsZy = create2dUnitCellArray(this.sizeZy);
    unitCellsZx = create2dUnitCellArray(this.sizeZx);
    centerZy = (sizeZy == null) ? null : new Position2d(LengthUnit.MILLIMETER);
    centerZx = (sizeZx == null) ? null : new Position2d(LengthUnit.MILLIMETER);
    center = (sizeZy == null || sizeZx == null) ? null : new Position3d(LengthUnit.MILLIMETER);
    time = null;
  }

  public synchronized ImcadrServerResult snapshot(ImcadrServerResult dest) {
    ImcadrServerResult result = dest;
    if (result == null) result = new ImcadrServerResult(sizeZy, sizeZx);
    result.progress = progress;
    result.state = state;
    result.message = message;
    result.originZy = originZy;
    result.originZx = originZx;
    result.scoresZy = copy(scoresZy, result.scoresZy);
    result.scoresZx = copy(scoresZx, result.scoresZx);
    result.spotTotalsZy = copy(spotTotalsZy, result.spotTotalsZy);
    result.spotTotalsZx = copy(spotTotalsZx, result.spotTotalsZx);
    result.inResTotalsZy = copy(inResTotalsZy, result.inResTotalsZy);
    result.inResTotalsZx = copy(inResTotalsZx, result.inResTotalsZx);
    result.goodBraggTotalsZy = copy(goodBraggTotalsZy, result.goodBraggTotalsZy);
    result.goodBraggTotalsZx = copy(goodBraggTotalsZx, result.goodBraggTotalsZx);
    result.iceRingTotalsZy = copy(iceRingTotalsZy, result.iceRingTotalsZy);
    result.iceRingTotalsZx = copy(iceRingTotalsZx, result.iceRingTotalsZx);
    result.estimatedRes1Zy = copy(estimatedRes1Zy, result.estimatedRes1Zy);
    result.estimatedRes1Zx = copy(estimatedRes1Zx, result.estimatedRes1Zx);
    result.estimatedRes2Zy = copy(estimatedRes2Zy, result.estimatedRes2Zy);
    result.estimatedRes2Zx = copy(estimatedRes2Zx, result.estimatedRes2Zx);
    result.totalIntSignalsZy = copy(totalIntSignalsZy, result.totalIntSignalsZy);
    result.totalIntSignalsZx = copy(totalIntSignalsZx, result.totalIntSignalsZx);
    result.mosaicitiesZy = copy(mosaicitiesZy, result.mosaicitiesZy);
    result.mosaicitiesZx = copy(mosaicitiesZx, result.mosaicitiesZx);
    result.spaceGroupsZy = copy(spaceGroupsZy, result.spaceGroupsZy);
    result.spaceGroupsZx = copy(spaceGroupsZx, result.spaceGroupsZx);
    result.unitCellsZy = copy(unitCellsZy, result.unitCellsZy);
    result.unitCellsZx = copy(unitCellsZx, result.unitCellsZx);
    result.centerZy = centerZy;
    result.centerZx = centerZx;
    result.center = center;
    result.time = time;
    return result;
  }

  public synchronized int getProgress() {
    return progress;
  }

  public synchronized void setProgress(int progress) {
    this.progress = progress;
  }

  public synchronized ImcadrServerState getState() {
    return state;
  }

  public synchronized void setState(ImcadrServerState state) {
    if (state == null) return;
    this.state = state;
  }

  public synchronized String getMessage() {
    return message;
  }

  public synchronized void setMessage(String message) {
    this.message = message;
  }

  public synchronized Position2d getOriginZy() {
    return originZy;
  }

  public synchronized void setOriginZy(Position2d originZy) {
    this.originZy = (originZy == null) ? null : originZy.convertTo(LengthUnit.MILLIMETER);
  }

  public synchronized Position2d getOriginZx() {
    return originZx;
  }

  public synchronized void setOriginZx(Position2d originZx) {
    this.originZx = (originZx == null) ? null : originZx.convertTo(LengthUnit.MILLIMETER);
  }

  public synchronized Integer[][] getScoresZy() {
    return scoresZy;
  }

  public synchronized void setScoresZy(Integer[][] scoresZy) {
    this.scoresZy = scoresZy;
  }

  public synchronized void setScoreZy(int col, int row, Integer score) {
    scoresZy[row][col] = score;
  }

  public synchronized Integer[][] getScoresZx() {
    return scoresZx;
  }

  public synchronized void setScoresZx(Integer[][] scoresZx) {
    this.scoresZx = scoresZx;
  }

  public synchronized void setScoreZx(int col, int row, Integer score) {
    scoresZx[row][col] = score;
  }

  public synchronized Integer[][] getSpotTotalsZy() {
    return spotTotalsZy;
  }

  public synchronized void setSpotTotalsZy(Integer[][] spotTotalsZy) {
    this.spotTotalsZy = spotTotalsZy;
  }

  public synchronized void setSpotTotalZy(int col, int row, Integer spotTotal) {
    spotTotalsZy[row][col] = spotTotal;
  }

  public synchronized Integer[][] getSpotTotalsZx() {
    return spotTotalsZx;
  }

  public synchronized void setSpotTotalsZx(Integer[][] spotTotalsZx) {
    this.spotTotalsZx = spotTotalsZx;
  }

  public synchronized void setSpotTotalZx(int col, int row, Integer spotTotal) {
    spotTotalsZx[row][col] = spotTotal;
  }

  public synchronized Integer[][] getInResolutionTotalsZy() {
    return inResTotalsZy;
  }

  public synchronized void setInResolutionTotalsZy(Integer[][] inResTotalsZy) {
    this.inResTotalsZy = inResTotalsZy;
  }

  public synchronized void setInResolutionTotalZy(int col, int row, Integer inResTotal) {
    inResTotalsZy[row][col] = inResTotal;
  }

  public synchronized Integer[][] getInResolutionTotalsZx() {
    return inResTotalsZx;
  }

  public synchronized void setInResolutionTotalsZx(Integer[][] inResTotalsZx) {
    this.inResTotalsZx = inResTotalsZx;
  }

  public synchronized void setInResolutionTotalZx(int col, int row, Integer inResTotal) {
    inResTotalsZx[row][col] = inResTotal;
  }

  public synchronized Integer[][] getGoodBraggTotalsZy() {
    return goodBraggTotalsZy;
  }

  public synchronized void setGoodBraggTotalsZy(Integer[][] goodBraggTotalsZy) {
    this.goodBraggTotalsZy = goodBraggTotalsZy;
  }

  public synchronized void setGoodBraggTotalZy(int col, int row, Integer goodBraggTotal) {
    goodBraggTotalsZy[row][col] = goodBraggTotal;
  }

  public synchronized Integer[][] getGoodBraggTotalsZx() {
    return goodBraggTotalsZx;
  }

  public synchronized void setGoodBraggTotalsZx(Integer[][] goodBraggTotalsZx) {
    this.goodBraggTotalsZx = goodBraggTotalsZx;
  }

  public synchronized void setGoodBraggTotalZx(int col, int row, Integer goodBraggTotal) {
    goodBraggTotalsZx[row][col] = goodBraggTotal;
  }

  public synchronized Integer[][] getIceRingTotalsZy() {
    return iceRingTotalsZy;
  }

  public synchronized void setIceRingTotalsZy(Integer[][] iceRingTotalsZy) {
    this.iceRingTotalsZy = iceRingTotalsZy;
  }

  public synchronized void setIceRingTotalZy(int col, int row, Integer iceRingTotal) {
    iceRingTotalsZy[row][col] = iceRingTotal;
  }

  public synchronized Integer[][] getIceRingTotalsZx() {
    return iceRingTotalsZx;
  }

  public synchronized void setIceRingTotalsZx(Integer[][] iceRingTotalsZx) {
    this.iceRingTotalsZx = iceRingTotalsZx;
  }

  public synchronized void setIceRingTotalZx(int col, int row, Integer iceRingTotal) {
    iceRingTotalsZx[row][col] = iceRingTotal;
  }

  public synchronized Double[][] getEstimatedResolutions1Zy() {
    return estimatedRes1Zy;
  }

  public synchronized void setEstimatedResolutions1Zy(Double[][] estimatedRes1Zy) {
    this.estimatedRes1Zy = estimatedRes1Zy;
  }

  public synchronized void setEstimatedResolution1Zy(int col, int row, Double estimatedRes) {
    estimatedRes1Zy[row][col] = estimatedRes;
  }

  public synchronized Double[][] getEstimatedResolutions1Zx() {
    return estimatedRes1Zx;
  }

  public synchronized void setEstimatedResolutions1Zx(Double[][] estimatedRes1Zx) {
    this.estimatedRes1Zx = estimatedRes1Zx;
  }

  public synchronized void setEstimatedResolution1Zx(int col, int row, Double estimatedRes) {
    estimatedRes1Zx[row][col] = estimatedRes;
  }

  public synchronized Double[][] getEstimatedResolutions2Zy() {
    return estimatedRes2Zy;
  }

  public synchronized void setEstimatedResolutions2Zy(Double[][] estimatedRes2Zy) {
    this.estimatedRes2Zy = estimatedRes2Zy;
  }

  public synchronized void setEstimatedResolution2Zy(int col, int row, Double estimatedRes) {
    estimatedRes2Zy[row][col] = estimatedRes;
  }

  public synchronized Double[][] getEstimatedResolutions2Zx() {
    return estimatedRes2Zx;
  }

  public synchronized void setEstimatedResolutions2Zx(Double[][] estimatedRes2Zx) {
    this.estimatedRes2Zx = estimatedRes2Zx;
  }

  public synchronized void setEstimatedResolution2Zx(int col, int row, Double estimatedRes) {
    estimatedRes2Zx[row][col] = estimatedRes;
  }

  public synchronized Integer[][] getTotalIntegratedSignalsZy() {
    return totalIntSignalsZy;
  }

  public synchronized void setTotalIntegratedSignalsZy(Integer[][] totalIntSignalsZy) {
    this.totalIntSignalsZy = totalIntSignalsZy;
  }

  public synchronized void setTotalIntegratedSignalZy(int col, int row, Integer totalIntSignal) {
    totalIntSignalsZy[row][col] = totalIntSignal;
  }

  public synchronized Integer[][] getTotalIntegratedSignalsZx() {
    return totalIntSignalsZx;
  }

  public synchronized void setTotalIntegratedSignalsZx(Integer[][] totalIntSignalsZx) {
    this.totalIntSignalsZx = totalIntSignalsZx;
  }

  public synchronized void setTotalIntegratedSignalZx(int col, int row, Integer totalIntSignal) {
    totalIntSignalsZx[row][col] = totalIntSignal;
  }

  public synchronized Double[][] getMosaicitiesZy() {
    return mosaicitiesZy;
  }

  public synchronized void setMosaicitiesZy(Double[][] mosaicitiesZy) {
    this.mosaicitiesZy = mosaicitiesZy;
  }

  public synchronized void setMosaicityZy(int col, int row, Double mosaicity) {
    mosaicitiesZy[row][col] = mosaicity;
  }

  public synchronized Double[][] getMosaicitiesZx() {
    return mosaicitiesZx;
  }

  public synchronized void setMosaicitiesZx(Double[][] mosaicitiesZx) {
    this.mosaicitiesZx = mosaicitiesZx;
  }

  public synchronized void setMosaicityZx(int col, int row, Double mosaicity) {
    mosaicitiesZx[row][col] = mosaicity;
  }

  public synchronized SpaceGroup[][] getSpaceGroupsZy() {
    return spaceGroupsZy;
  }

  public synchronized void setSpaceGroupsZy(SpaceGroup[][] spaceGroupsZy) {
    this.spaceGroupsZy = spaceGroupsZy;
  }

  public synchronized void setSpaceGroupZy(int col, int row, SpaceGroup spaceGroup) {
    spaceGroupsZy[row][col] = spaceGroup;
  }

  public synchronized SpaceGroup[][] getSpaceGroupsZx() {
    return spaceGroupsZx;
  }

  public synchronized void setSpaceGroupsZx(SpaceGroup[][] spaceGroupsZx) {
    this.spaceGroupsZx = spaceGroupsZx;
  }

  public synchronized void setSpaceGroupZx(int col, int row, SpaceGroup spaceGroup) {
    spaceGroupsZx[row][col] = spaceGroup;
  }

  public synchronized UnitCell[][] getUnitCellsZy() {
    return unitCellsZy;
  }

  public synchronized void setUnitCellsZy(UnitCell[][] unitCellsZy) {
    this.unitCellsZy = unitCellsZy;
  }

  public synchronized void setUnitCellZy(int col, int row, UnitCell unitCell) {
    unitCellsZy[row][col] = unitCell;
  }

  public synchronized UnitCell[][] getUnitCellsZx() {
    return unitCellsZx;
  }

  public synchronized void setUnitCellsZx(UnitCell[][] unitCellsZx) {
    this.unitCellsZx = unitCellsZx;
  }

  public synchronized void setUnitCellZx(int col, int row, UnitCell unitCell) {
    unitCellsZx[row][col] = unitCell;
  }

  public synchronized Position2d getCenterZy() {
    return centerZy;
  }

  public synchronized void setCenterZy(Position2d centerZy) {
    this.centerZy = (centerZy == null) ? null : centerZy.convertTo(LengthUnit.MILLIMETER);
  }

  public synchronized Position2d getCenterZx() {
    return centerZx;
  }

  public synchronized void setCenterZx(Position2d centerZx) {
    this.centerZx = (centerZx == null) ? null : centerZx.convertTo(LengthUnit.MILLIMETER);
  }

  public synchronized Position3d getCenter() {
    return center;
  }

  public synchronized void setCenter(Position3d center) {
    this.center = (center == null) ? null : center.convertTo(LengthUnit.MILLIMETER);
  }

  public synchronized Integer getTime() {
    return time;
  }

  public synchronized void setTime(Integer time) {
    this.time = time;
  }

  protected static Integer[][] create2dIntegerArray(Rectangle size) {
    return (size == null) ? null : new Integer[(int)size.getHeight()][(int)size.getWidth()];
  }

  protected static Double[][] create2dDoubleArray(Rectangle size) {
    return (size == null) ? null : new Double[(int)size.getHeight()][(int)size.getWidth()];
  }

  protected static SpaceGroup[][] create2dSpaceGroupArray(Rectangle size) {
    return (size == null) ? null : new SpaceGroup[(int)size.getHeight()][(int)size.getWidth()];
  }

  protected static UnitCell[][] create2dUnitCellArray(Rectangle size) {
    return (size == null) ? null : new UnitCell[(int)size.getHeight()][(int)size.getWidth()];
  }

  protected static Integer[][] copy(Integer[][] src, Integer[][] dest) {
    if (src == null) return null;
    Integer[][] result = dest;
    if (result == null) result = new Integer[src.length][(src.length == 0) ? 0 : src[0].length];
    for (int i = 0; i < src.length; i++) {
      System.arraycopy(src[i], 0, result[i], 0, src[i].length);
    }
    return result;
  }

  protected static Double[][] copy(Double[][] src, Double[][] dest) {
    if (src == null) return null;
    Double[][] result = dest;
    if (result == null) result = new Double[src.length][(src.length == 0) ? 0 : src[0].length];
    for (int i = 0; i < src.length; i++) {
      System.arraycopy(src[i], 0, result[i], 0, src[i].length);
    }
    return result;
  }

  protected static SpaceGroup[][] copy(SpaceGroup[][] src, SpaceGroup[][] dest) {
    if (src == null) return null;
    SpaceGroup[][] result = dest;
    if (result == null) result =
        new SpaceGroup[src.length][(src.length == 0) ? 0 : src[0].length];
    for (int i = 0; i < src.length; i++) {
      System.arraycopy(src[i], 0, result[i], 0, src[i].length);
    }
    return result;
  }

  protected static UnitCell[][] copy(UnitCell[][] src, UnitCell[][] dest) {
    if (src == null) return null;
    UnitCell[][] result = dest;
    if (result == null) result = new UnitCell[src.length][(src.length == 0) ? 0 : src[0].length];
    for (int i = 0; i < src.length; i++) {
      System.arraycopy(src[i], 0, result[i], 0, src[i].length);
    }
    return result;
  }
}
