package imcacat.imcadr;

import imcacat.epicsca4j.format.DoubleDecimalFormat;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DecimalFormat;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.text.NumberFormatter;

import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * I am an editor for an {@link ImcadrSettings}.
 */
public class ImcadrSettingsDialog extends JDialog {
  private static final long serialVersionUID = 1;

  protected ImcadrSettingsPresentationModel model;

  protected Action doneAction;

  public ImcadrSettingsDialog(Frame owner, String title, boolean modal,
      ImcadrSettingsPresentationModel model) {
    super(owner, title, modal);
    this.model = model;

    initializeActions();
    initializeDialog();
  }

  protected void initializeActions() {
    doneAction = new AbstractAction() {
      private static final long serialVersionUID = 1;

      public void actionPerformed(ActionEvent e) {
        doneActionPerformed();
      }
    };
    doneAction.putValue(Action.NAME, "Done");
  }

  protected void initializeDialog() {
    setContentPane(buildContentPane());

    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        doneActionPerformed();
      }
    });
  }

  protected JComponent buildContentPane() {
    /* @formatter:off */
    FormLayout layout = new FormLayout(
        /* Columns */
        "right:max(50dlu;pref), 4dlu, fill:max(30dlu;pref), 4dlu, left:pref, 4dlu, " +
            "fill:max(30dlu;pref), 4dlu, fill:max(30dlu;pref), 4dlu, left:pref",
        /* Rows: Raster */
        "pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 4dlu, pref, 4dlu," +
        /* Rows: Acquisition */
        "pref, 4dlu, pref, 4dlu, " +
        /* Rows: Scoring */
        "pref, 4dlu, pref, 4dlu, pref, 2dlu, pref"
    );
    layout.setRowGroups(
        new int[][] {
            {1, 11, 15},
            {3, 5, 7, 13, 17, 21}
        });
    layout.setColumnGroups(
        new int[][] {
            {3, 7}
        });
    /* @formatter:on */
    PanelBuilder builder = new PanelBuilder(layout);
    builder.setDefaultDialogBorder();
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("Raster", cc.xyw(1, 1, 9));

    builder.addLabel("Cell size:", cc.xy(1, 3));
    NumberFormatter rasterCellWidthTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    rasterCellWidthTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField rasterCellWidthTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_RASTER_CELL_WIDTH),
            rasterCellWidthTextFieldFormatter);
    rasterCellWidthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(rasterCellWidthTextField, cc.xy(3, 3));
    builder.add(new JLabel("\u03BCm wide by"), cc.xy(5, 3));
    NumberFormatter rasterCellHeightTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    rasterCellHeightTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField rasterCellHeightTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_RASTER_CELL_HEIGHT),
            rasterCellHeightTextFieldFormatter);
    rasterCellHeightTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(rasterCellHeightTextField, cc.xy(7, 3));
    builder.add(new JLabel("\u03BCm high"), cc.xy(9, 3));

    builder.addLabel("ZY (\u03A6 at 0\u00B0) raster size:", cc.xy(1, 5));
    NumberFormatter zyRasterWidthTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    zyRasterWidthTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField zyRasterWidthTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_ZY_RASTER_WIDTH),
            zyRasterWidthTextFieldFormatter);
    zyRasterWidthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(zyRasterWidthTextField, cc.xy(3, 5));
    builder.add(new JLabel("cells wide by"), cc.xy(5, 5));
    NumberFormatter zyRasterHeightTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    zyRasterHeightTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField zyRasterHeightTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_ZY_RASTER_HEIGHT),
            zyRasterHeightTextFieldFormatter);
    zyRasterHeightTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(zyRasterHeightTextField, cc.xy(7, 5));
    builder.add(new JLabel("cells high"), cc.xyw(9, 5, 2));

    builder.addLabel("ZX (\u03A6 at 90\u00B0) raster size:", cc.xy(1, 7));
    NumberFormatter zxRasterWidthTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    zxRasterWidthTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField zxRasterWidthTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_ZX_RASTER_WIDTH),
            zxRasterWidthTextFieldFormatter);
    zxRasterWidthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(zxRasterWidthTextField, cc.xy(3, 7));
    builder.add(new JLabel("cells wide by"), cc.xy(5, 7));
    NumberFormatter zxRasterHeightTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    zxRasterHeightTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField zxRasterHeightTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_ZX_RASTER_HEIGHT),
            zxRasterHeightTextFieldFormatter);
    zxRasterHeightTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(zxRasterHeightTextField, cc.xy(7, 7));
    builder.add(new JLabel("cells high"), cc.xyw(9, 7, 2));

    builder.add(BasicComponentFactory.createCheckBox(
        model.getModel(ImcadrSettings.PROPNAME_CHASING_ENABLED), "Chase best score if on edge"),
        cc.xyw(3, 9, 9));

    builder.addSeparator("Acquisition", cc.xyw(1, 11, 9));

    builder.addLabel("Exposure time:", cc.xy(1, 13));
    NumberFormatter exposureTimeTextFieldFormatter = new NumberFormatter(new DecimalFormat("0"));
    exposureTimeTextFieldFormatter.setMinimum(new Long(1));
    JFormattedTextField exposureTimeTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_EXPOSURE_TIME),
            exposureTimeTextFieldFormatter);
    exposureTimeTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(exposureTimeTextField, cc.xy(3, 13));
    builder.add(new JLabel("msec"), cc.xy(5, 13));

    builder.addSeparator("Score", cc.xyw(1, 15, 9));

    builder.addLabel("Score at most:", cc.xy(1, 17));
    NumberFormatter maxNumConcurrentAnalyzersTextFieldFormatter =
        new NumberFormatter(new IntegerDecimalFormat("0"));
    maxNumConcurrentAnalyzersTextFieldFormatter.setMinimum(new Integer(1));
    JFormattedTextField maxNumConcurrentAnalyzersTextField =
        BasicComponentFactory.createFormattedTextField(
            model.getModel(ImcadrSettings.PROPNAME_MAX_NUM_CONCURRENT_ANALYZERS),
            maxNumConcurrentAnalyzersTextFieldFormatter);
    maxNumConcurrentAnalyzersTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(maxNumConcurrentAnalyzersTextField, cc.xy(3, 17));
    builder.add(new JLabel("image(s) at the same time"), cc.xyw(5, 17, 7));

    builder.add(
        BasicComponentFactory.createCheckBox(
            model.getModel(ImcadrSettings.PROPNAME_WEAK_DIFFRACTER),
            "Weak diffraction is expected"), cc.xyw(3, 19, 9));

    builder.addLabel("High resolution cutoff:", cc.xyw(5, 21, 3, "r, d"));
    NumberFormatter highResCutoffTextFieldFormatter =
        new NumberFormatter(new DoubleDecimalFormat("#0.0#"));
    highResCutoffTextFieldFormatter.setMinimum(Double.valueOf(Resources.getInstance()
        .getHighResolutionCutoffMin()));
    highResCutoffTextFieldFormatter.setMaximum(Double.valueOf(Resources.getInstance()
        .getHighResolutionCutoffMax()));
    JFormattedTextField highResCutoffTextField =
        ComponentFactory.createFormattedTextField(
            model.getComponentModel(ImcadrSettings.PROPNAME_HIGH_RES_CUTOFF),
            highResCutoffTextFieldFormatter);
    highResCutoffTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(highResCutoffTextField, cc.xy(9, 21));
    builder.add(new JLabel("\u00C5"), cc.xy(11, 21));

    return builder.getPanel();
  }

  protected void doneActionPerformed() {
    dispose();
  }
}
