package imcacat.imcadr;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import imcacat.imcadr.util.ShellUtils;

/**
 * I am an {@link ImageAnalysis} that uses
 * <a href="http://cci.lbl.gov/labelit/">LABELIT</a>.
 * <p>
 * I run the <tt>labelit-distl.signal_strength</tt> helper program as if
 * running <tt>distl.signal_strength</tt>. The working directory is
 * unspecified, so any file or directory paths will be absolute.
 * <tt>distl.signal_strength</tt> is eventually executed with the arguments
 * given to <tt>labelit-distl.signal_strength</tt>.
 */
public class LabelitImageAnalysis extends ImageAnalysis {
  public LabelitImageAnalysis(ImageAnalysisSpec spec, File image) {
    super(spec, image);
  }

  public LabelitImageAnalysis(ImageAnalysisSpec spec, File image, File clusterKeyFile,
      String clusterNode) {
    super(spec, image, clusterKeyFile, clusterNode);
  }

  @Override
  protected List<String> createCommand() {
    ArrayList<String> result = new ArrayList<String>();
    result.add(Resources.getInstance().getLabelitDistlSignalStrengthHelper().getPath());
    Map<String, String> varNamesToValues = new HashMap<>(1);
    varNamesToValues.put("image", image.getAbsolutePath());
    for (String each : spec
        .getAnalysisProgramArguments(AnalysisProgram.LABELIT_DISTL_SIGNAL_STRENGTH)) {
      if (each.equals("$more") || each.equals("${more}")) {
        if (spec.getHighResolutionCutoff() != null) {
          result.add(format("distl.res.outer=%.2f", spec.getHighResolutionCutoff()));
        }
        if (spec.getLowResolutionCutoff() != null) {
          result.add(format("distl.res.inner=%.2f", spec.getLowResolutionCutoff()));
        }
        if (spec.getMinimumSpotArea() != null) {
          result.add(format("distl.minimum_spot_area=%d", spec.getMinimumSpotArea()));
        }
        if (spec.getMinimumSignalHeight() != null) {
          result.add(format("distl.minimum_signal_height=%.2f", spec.getMinimumSignalHeight()));
        }
      } else {
        result.add(ShellUtils.expand(each, varNamesToValues));
      }
    }
    return result;
  }

  @Override
  protected ImageAnalysisParser newAnalysisParser(InputStream stream) {
    return new LabelitDistSignalStrengthParser(stream);
  }
}
