package imcacat.imcadr;

/**
 * I model a unit of length.
 */
public enum LengthUnit {
  ARBITRARY("a.u."),
  METER("m"),
  MILLIMETER("mm"),
  MICROMETER("\u03BCm"),
  NANOMETER("nm"),
  PICOMETER("pm"),
  CELL("cell");

  protected String abbreviation;

  LengthUnit(String abbreviation) {
    this.abbreviation = abbreviation;
  }

  public String getAbbreviation() {
    return abbreviation;
  }

  /**
   * Returns the conversion ratio for converting from one unit to another.
   *
   * For example, the conversion ratio for converting from meters to
   * millimeters is 1000.0.
   *
   * @param src unit to convert from
   * @param dst unit to convert to
   *
   * @return conversion ratio or {@code null} if none is known
   */
  public static Double conversionRatio(LengthUnit src, LengthUnit dst) {
    if (src.equals(dst)) return 1.0;
    double srcToMeters;
    switch (src) {
      case METER:
        srcToMeters = 1.0;
        break;
      case MILLIMETER:
        srcToMeters = 1e-3;
        break;
      case MICROMETER:
        srcToMeters = 1e-6;
        break;
      case NANOMETER:
        srcToMeters = 1e-9;
        break;
      case PICOMETER:
        srcToMeters = 1e-12;
        break;
      default:
        return null;
    }
    double metersToDst;
    switch (dst) {
      case METER:
        metersToDst = 1.0;
        break;
      case MILLIMETER:
        metersToDst = 1e3;
        break;
      case MICROMETER:
        metersToDst = 1e6;
        break;
      case NANOMETER:
        metersToDst = 1e9;
        break;
      case PICOMETER:
        metersToDst = 1e12;
        break;
      default:
        return null;
    }
    return new Double(srcToMeters * metersToDst);
  }
}
