package imcacat.imcadr;

import java.util.Locale;

/**
 * I am thrown when a move does not reach the desired position.
 */
public class PositionErrorException extends DriverException {
  private static final long serialVersionUID = 1;

  private final String motorName;
  private final double desiredPosition;
  private final double actualPosition;
  private final double positionErrorMax;

  public PositionErrorException(String motorName, double desiredPosition, double actualPosition,
      double positionErrorMax) {
    super(createMessage(motorName, desiredPosition, actualPosition, positionErrorMax));
    this.motorName = motorName;
    this.desiredPosition = desiredPosition;
    this.actualPosition = actualPosition;
    this.positionErrorMax = positionErrorMax;
  }

  public String getMotorName() {
    return motorName;
  }

  public double getDesiredPosition() {
    return desiredPosition;
  }

  public double getActualPosition() {
    return actualPosition;
  }

  public double getPositionErrorMax() {
    return positionErrorMax;
  }

  private static final String createMessage(String motorName, double desiredPosition,
      double actualPosition, double positionErrorMax) {
    StringBuilder b = new StringBuilder();
    b.append(motorName);
    b.append(" move did not reach desired position (");
    b.append(String.format(Locale.US, "desired=%.9f, ", desiredPosition));
    b.append(String.format(Locale.US, "actual=%.9f, ", actualPosition));
    b.append(String.format(Locale.US, "positionErrorMax=%.9f", positionErrorMax));
    b.append(")");
    return b.toString();
  }
}
