package imcacat.imcadr;

import java.util.Iterator;
import java.util.NoSuchElementException;

import imcacat.imcadr.util.MathUtils;

/**
 * I am an iterator that iterates over a raster by column fly-scans, skipping
 * exterior masked and exterior scored cells.
 */
public class Raster2dColumnFlyScanIterator implements Iterator<Raster2dFlyScan> {
  private final Raster2d raster;
  private final int minCol;
  private final int maxCol;
  private int startRow;
  private int endRow;
  private int col;
  private boolean atEnd;

  public Raster2dColumnFlyScanIterator(Raster2d raster) {
    this.raster = raster;
    minCol = findMinCol(raster);
    maxCol = (minCol < 0) ? -1 : findMaxCol(raster);
    col = minCol;
    startRow = (col < 0) ? -1 : findMinRow(raster, col);
    endRow = (col < 0) ? -1 : findMaxRow(raster, col);
    atEnd = col < 0;
  }

  @Override
  public boolean hasNext() {
    return !atEnd;
  }

  @Override
  public Raster2dFlyScan next() {
    if (atEnd) throw new NoSuchElementException();
    Raster2dFlyScan result = new Raster2dFlyScan(raster, new Raster2dIndex(col, startRow),
        new Raster2dIndex(col, endRow));
    step();
    return result;
  }

  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }

  private void step() {
    col++;
    if (col > maxCol) {
      atEnd = true;
      return;
    }
    int minRow = findMinRow(raster, col);
    int maxRow = findMaxRow(raster, col);
    if (MathUtils.even(minCol + col)) {
      startRow = minRow;
      endRow = maxRow;
    } else {
      startRow = maxRow;
      endRow = minRow;
    }
  }

  private static int findMinCol(Raster2d raster) {
    int colCount = raster.columnCount();
    for (int col = 0; col < colCount; col++) {
      int row = findMinRow(raster, col);
      if (row >= 0) return col;
    }
    return -1;
  }

  private static int findMaxCol(Raster2d raster) {
    for (int col = raster.columnCount() - 1; col >= 0; col--) {
      int row = findMinRow(raster, col);
      if (row >= 0) return col;
    }
    return -1;
  }

  private static int findMinRow(Raster2d raster, int col) {
    int rowCount = raster.rowCount();
    for (int row = 0; row < rowCount; row++) {
      if (!raster.isMasked(col, row) && raster.getCell(col, row).getScore() == null) return row;
    }
    return -1;
  }

  private static int findMaxRow(Raster2d raster, int col) {
    for (int row = raster.rowCount() - 1; row >= 0; row--) {
      if (!raster.isMasked(col, row) && raster.getCell(col, row).getScore() == null) return row;
    }
    return -1;
  }
}
