package imcacat.imcadr;

import java.util.NoSuchElementException;

/**
 * I iterate along one column of a {@link Raster2d}.
 */
public class Raster2dColumnIterator implements Raster2dIterator {
  private final Raster2d raster;
  private final int startRow;
  private final int endRow;
  private final int col;
  private int row;
  private boolean atEnd;

  public Raster2dColumnIterator(Raster2d raster, Raster2dIndex start, Raster2dIndex end) {
    this.raster = raster;
    startRow = start.getRow();
    endRow = end.getRow();
    col = start.getColumn();
    row = startRow;
    atEnd = false;
  }

  @Override
  public boolean hasNext() {
    return !atEnd;
  }

  @Override
  public RasterCell next() {
    if (atEnd) throw new NoSuchElementException();
    RasterCell result = raster.getCell(col, row);
    step();
    return result;
  }

  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }

  @Override
  public int nextColumn() {
    if (atEnd) throw new NoSuchElementException();
    return col;
  }

  @Override
  public int nextRow() {
    if (atEnd) throw new NoSuchElementException();
    return row;
  }

  @Override
  public Raster2dIndex nextIndex() {
    if (atEnd) throw new NoSuchElementException();
    return new Raster2dIndex(col, row);
  }

  private void step() {
    if (endRow >= startRow) {
      row++;
      if (row > endRow) atEnd = true;
    } else {
      row--;
      if (row < endRow) atEnd = true;
    }
  }
}
