package imcacat.imcadr;

import java.util.EnumSet;
import java.util.Set;

/**
 * I model a two-dimensional raster edge.
 */
public enum Raster2dEdge {
  ALL,
  TOP,
  TOP_RIGHT,
  RIGHT,
  BOTTOM_RIGHT,
  BOTTOM,
  BOTTOM_LEFT,
  LEFT,
  TOP_LEFT;

  public boolean involvesTop() {
    return equals(TOP) || equals(TOP_LEFT) || equals(TOP_RIGHT) || equals(ALL);
  }

  public boolean involvesBottom() {
    return equals(BOTTOM) || equals(BOTTOM_LEFT) || equals(BOTTOM_RIGHT) || equals(ALL);
  }

  public boolean involvesLeft() {
    return equals(LEFT) || equals(TOP_LEFT) || equals(BOTTOM_LEFT) || equals(ALL);
  }

  public boolean involvesRight() {
    return equals(RIGHT) || equals(TOP_RIGHT) || equals(BOTTOM_RIGHT) || equals(ALL);
  }

  public Raster2dEdge minusTopAndBottom() {
    if (equals(TOP_LEFT) || equals(BOTTOM_LEFT)) return LEFT;
    if (equals(TOP_RIGHT) || equals(BOTTOM_RIGHT)) return RIGHT;
    if (equals(TOP) || equals(BOTTOM)) return null;
    if (equals(ALL)) return null;
    return this;
  }

  public Raster2dEdge minusLeftAndRight() {
    if (equals(TOP_LEFT) || equals(TOP_RIGHT)) return TOP;
    if (equals(BOTTOM_LEFT) || equals(BOTTOM_RIGHT)) return BOTTOM;
    if (equals(LEFT) || equals(RIGHT)) return null;
    if (equals(ALL)) return null;
    return this;
  }

  public Raster2dEdge opposite() {
    switch (this) {
      case ALL:
        return ALL;
      case TOP:
        return BOTTOM;
      case TOP_RIGHT:
        return BOTTOM_LEFT;
      case RIGHT:
        return LEFT;
      case BOTTOM_RIGHT:
        return TOP_LEFT;
      case BOTTOM:
        return TOP;
      case BOTTOM_LEFT:
        return TOP_RIGHT;
      case LEFT:
        return RIGHT;
      case TOP_LEFT:
        return BOTTOM_RIGHT;
      default:
        throw new BugException();
    }
  }

  public Set<Raster2dEdge> components() {
    switch (this) {
      case ALL:
        return EnumSet.<Raster2dEdge>of(TOP, RIGHT, BOTTOM, LEFT);
      case TOP:
        return EnumSet.<Raster2dEdge>of(TOP);
      case TOP_RIGHT:
        return EnumSet.<Raster2dEdge>of(TOP, RIGHT);
      case RIGHT:
        return EnumSet.<Raster2dEdge>of(RIGHT);
      case BOTTOM_RIGHT:
        return EnumSet.<Raster2dEdge>of(BOTTOM, RIGHT);
      case BOTTOM:
        return EnumSet.<Raster2dEdge>of(BOTTOM);
      case BOTTOM_LEFT:
        return EnumSet.<Raster2dEdge>of(BOTTOM, LEFT);
      case LEFT:
        return EnumSet.<Raster2dEdge>of(LEFT);
      case TOP_LEFT:
        return EnumSet.<Raster2dEdge>of(TOP, LEFT);
      default:
        throw new BugException();
    }
  }
}
