package imcacat.imcadr;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * I am an iterator that iterates over an edge of a raster by fly-scans,
 * skipping exterior masked and exterior scored cells.
 */
public class Raster2dEdgeFlyScanIterator implements Iterator<Raster2dFlyScan> {
  private final Iterator<Raster2dFlyScan> delegate;

  public Raster2dEdgeFlyScanIterator(Raster2d raster, Raster2dEdge edge, Raster2dIndex edgeIndex) {
    super();
    delegate = computeScans(raster, edge, edgeIndex).iterator();
  }

  @Override
  public boolean hasNext() {
    return delegate.hasNext();
  }

  @Override
  public Raster2dFlyScan next() {
    return delegate.next();
  }

  @Override
  public void remove() {
    delegate.remove();
  }

  private static List<Raster2dFlyScan> computeScans(Raster2d raster, Raster2dEdge edge,
      Raster2dIndex edgeIndex) {
    List<Raster2dFlyScan> result = new ArrayList<>();
    Raster2dEdgeIterator iterator = new Raster2dEdgeIterator(raster, edge, edgeIndex);
    RasterCell start = null;
    RasterCell end = start;
    while (iterator.hasNext()) {
      RasterCell each = iterator.next();
      if (each.getScore() != null) continue;
      if (raster.isMasked(each.getColumn(), each.getRow())) continue;
      if (start == null) {
        start = each;
        end = start;
      } else {
        if (start.getColumn() == each.getColumn() || start.getRow() == each.getRow()) {
          end = each;
        } else {
          result.add(new Raster2dFlyScan(raster, start.getIndex(), end.getIndex()));
          start = each;
          end = start;
        }
      }
    }
    if (start != null) {
      result.add(new Raster2dFlyScan(raster, start.getIndex(), end.getIndex()));
    }
    return result;
  }
}
