package imcacat.imcadr;

import java.util.Iterator;

/**
 * I represent a fly-scan of a row or column of a {@link Raster2d}.
 */
public class Raster2dFlyScan implements Iterable<RasterCell> {
  private final Raster2d raster;
  private final Raster2dIndex start;
  private final Raster2dIndex end;

  public Raster2dFlyScan(Raster2d raster, Raster2dIndex start, Raster2dIndex end) {
    super();
    this.raster = raster;
    this.start = start;
    this.end = end;
  }

  public Direction2d direction() {
    return (start.getRow() == end.getRow()) ? Direction2d.HORIZONTAL : Direction2d.VERTICAL;
  }

  public boolean isHorizontal() {
    return direction().isHorizontal();
  }

  public boolean isVertical() {
    return !isHorizontal();
  }

  public RasterCell startCell() {
    return raster.getCell(start);
  }

  public RasterCell endCell() {
    return raster.getCell(end);
  }

  public int imageCount() {
    if (isHorizontal()) {
      return Math.abs(end.getColumn() - start.getColumn()) + 1;
    } else {
      return Math.abs(end.getRow() - start.getRow()) + 1;
    }
  }

  public void setAcquiring(boolean newAcquiring) {
    for (RasterCell each : this) {
      each.setAcquiring(newAcquiring);
    }
  }

  @Override
  public Iterator<RasterCell> iterator() {
    return isHorizontal() ? new Raster2dRowIterator(raster, start, end)
        : new Raster2dColumnIterator(raster, start, end);
  }
}
