package imcacat.imcadr;

import java.util.Iterator;
import java.util.NoSuchElementException;

import imcacat.imcadr.util.MathUtils;

/**
 * I am an iterator that iterates over a raster by row fly-scans, skipping
 * exterior masked and exterior scored cells.
 */
public class Raster2dRowFlyScanIterator implements Iterator<Raster2dFlyScan> {
  private final Raster2d raster;
  private final int minRow;
  private final int maxRow;
  private int startCol;
  private int endCol;
  private int row;
  private boolean atEnd;

  public Raster2dRowFlyScanIterator(Raster2d raster) {
    this.raster = raster;
    minRow = findMinRow(raster);
    maxRow = (minRow < 0) ? -1 : findMaxRow(raster);
    row = minRow;
    startCol = (row < 0) ? -1 : findMinCol(raster, row);
    endCol = (row < 0) ? -1 : findMaxCol(raster, row);
    atEnd = row < 0;
  }

  @Override
  public boolean hasNext() {
    return !atEnd;
  }

  @Override
  public Raster2dFlyScan next() {
    if (atEnd) throw new NoSuchElementException();
    Raster2dFlyScan result = new Raster2dFlyScan(raster, new Raster2dIndex(startCol, row),
        new Raster2dIndex(endCol, row));
    step();
    return result;
  }

  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }

  private void step() {
    row++;
    if (row > maxRow) {
      atEnd = true;
      return;
    }
    int minCol = findMinCol(raster, row);
    int maxCol = findMaxCol(raster, row);
    if (MathUtils.even(minRow + row)) {
      startCol = minCol;
      endCol = maxCol;
    } else {
      startCol = maxCol;
      endCol = minCol;
    }
  }

  private static int findMinRow(Raster2d raster) {
    int rowCount = raster.rowCount();
    for (int row = 0; row < rowCount; row++) {
      int col = findMinCol(raster, row);
      if (col >= 0) return row;
    }
    return -1;
  }

  private static int findMaxRow(Raster2d raster) {
    for (int row = raster.rowCount() - 1; row >= 0; row--) {
      int col = findMinCol(raster, row);
      if (col >= 0) return row;
    }
    return -1;
  }

  private static int findMinCol(Raster2d raster, int row) {
    int colCount = raster.columnCount();
    for (int col = 0; col < colCount; col++) {
      if (!raster.isMasked(col, row) && raster.getCell(col, row).getScore() == null) return col;
    }
    return -1;
  }

  private static int findMaxCol(Raster2d raster, int row) {
    for (int col = raster.columnCount() - 1; col >= 0; col--) {
      if (!raster.isMasked(col, row) && raster.getCell(col, row).getScore() == null) return col;
    }
    return -1;
  }
}
