package imcacat.imcadr;

import java.util.NoSuchElementException;

/**
 * I iterate along one row of a {@link Raster2d}.
 */
public class Raster2dRowIterator implements Raster2dIterator {
  private final Raster2d raster;
  private final int startCol;
  private final int endCol;
  private final int row;
  private int col;
  private boolean atEnd;

  public Raster2dRowIterator(Raster2d raster, Raster2dIndex start, Raster2dIndex end) {
    this.raster = raster;
    startCol = start.getColumn();
    endCol = end.getColumn();
    row = start.getRow();
    col = startCol;
    atEnd = false;
  }

  @Override
  public boolean hasNext() {
    return !atEnd;
  }

  @Override
  public RasterCell next() {
    if (atEnd) throw new NoSuchElementException();
    RasterCell result = raster.getCell(col, row);
    step();
    return result;
  }

  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }

  @Override
  public int nextColumn() {
    if (atEnd) throw new NoSuchElementException();
    return col;
  }

  @Override
  public int nextRow() {
    if (atEnd) throw new NoSuchElementException();
    return row;
  }

  @Override
  public Raster2dIndex nextIndex() {
    if (atEnd) throw new NoSuchElementException();
    return new Raster2dIndex(col, row);
  }

  private void step() {
    if (endCol >= startCol) {
      col++;
      if (col > endCol) atEnd = true;
    } else {
      col--;
      if (col < endCol) atEnd = true;
    }
  }
}
