package imcacat.imcadr;

import java.util.NoSuchElementException;

/**
 * I am an iterator that iterates over only unmasked cells of a
 * {@link Raster2d}.
 */
public class Raster2dUnmaskedIterator implements Raster2dIterator {
  private final Raster2d raster;
  private final Raster2dIterator rasterIterator;
  private boolean steppedOnce;
  private RasterCell next;
  private Raster2dIndex nextIndex;
  private boolean atEnd;

  public Raster2dUnmaskedIterator(Raster2d raster) {
    this(raster, new Raster2dAllIterator(raster));
  }

  public Raster2dUnmaskedIterator(Raster2d raster, Raster2dIterator rasterIterator) {
    super();
    this.raster = raster;
    this.rasterIterator = rasterIterator;
    steppedOnce = false;
    next = null;
    nextIndex = null;
    atEnd = false;
  }

  @Override
  public boolean hasNext() {
    stepOnce();
    return !atEnd;
  }

  @Override
  public RasterCell next() {
    stepOnce();
    if (atEnd) throw new NoSuchElementException();
    RasterCell result = next;
    step();
    return result;
  }

  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }

  @Override
  public int nextColumn() {
    stepOnce();
    if (atEnd) throw new NoSuchElementException();
    return nextIndex.getColumn();
  }

  @Override
  public int nextRow() {
    stepOnce();
    if (atEnd) throw new NoSuchElementException();
    return nextIndex.getRow();
  }

  @Override
  public Raster2dIndex nextIndex() {
    stepOnce();
    if (atEnd) throw new NoSuchElementException();
    return nextIndex;
  }

  private void stepOnce() {
    if (steppedOnce) return;
    step();
    steppedOnce = true;
  }

  private void step() {
    while (rasterIterator.hasNext()) {
      Raster2dIndex candidateIndex = rasterIterator.nextIndex();
      RasterCell candidate = rasterIterator.next();
      if (raster.isMasked(candidateIndex)) continue;
      next = candidate;
      nextIndex = candidateIndex;
      return;
    }
    atEnd = true;
  }
}
