package imcacat.imcadr;

import imcacat.epicsca4j.util.ObjectUtilities;

import java.io.File;

/**
 * I am a specification for a three-dimensional raster.
 */
public class Raster3dSpec {
  private final Raster2dSpec zySpec;
  private final Raster2dSpec zxSpec;
  private final boolean zxFirst;

  public Raster3dSpec(Raster2dSpec zySpec, Raster2dSpec zxSpec) {
    this(zySpec, zxSpec, false);
  }

  public Raster3dSpec(Raster2dSpec zySpec, Raster2dSpec zxSpec, boolean zxFirst) {
    if (!ObjectUtilities.areEqual(zySpec.getImagesDirectory(), zySpec.getImagesDirectory())) {
      throw new IllegalArgumentException(
          "ZY spec and ZX spec images directories must be the same");
    }
    this.zySpec = zySpec;
    this.zxSpec = zxSpec;
    this.zxFirst = zxFirst;
  }

  public Raster2dSpec getZySpec() {
    return zySpec;
  }

  public Raster2dSpec getZxSpec() {
    return zxSpec;
  }

  public boolean isZxFirst() {
    return zxFirst;
  }

  public LengthUnit getCellDimensionUnit() {
    return zySpec.getCellDimensionUnit();
  }

  public File getImagesDirectory() {
    return zySpec.getImagesDirectory();
  }
}
