package imcacat.imcadr;

import java.io.File;
import java.util.Objects;

/**
 * I model a cell of a raster.
 */
public class RasterCell extends EdtAwareModel {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_SCORE = "score";
  public static final String PROPNAME_NORMALIZED_SCORE = "normalizedScore";
  public static final String PROPNAME_SPOT_TOTAL = "spotTotal";
  public static final String PROPNAME_IN_RES_TOTAL = "inResolutionTotal";
  public static final String PROPNAME_GOOD_BRAGG_TOTAL = "goodBraggTotal";
  public static final String PROPNAME_ICE_RING_TOTAL = "iceRingTotal";
  public static final String PROPNAME_ESTIMATED_RES1 = "estimatedResolution1";
  public static final String PROPNAME_ESTIMATED_RES2 = "estimatedResolution2";
  public static final String PROPNAME_TOTAL_INT_SIGNAL = "totalIntegratedSignal";
  public static final String PROPNAME_MOSAICITY = "mosaicity";
  public static final String PROPNAME_SPACE_GROUP = "spaceGroup";
  public static final String PROPNAME_UNIT_CELL = "unitCell";
  public static final String PROPNAME_ACQUIRING = "acquiring";
  public static final String PROPNAME_ACQUIRED = "acquired";
  public static final String PROPNAME_POSITION = "position";

  public static final double MIN_NORMALIZED_SCORE = 0.0;
  public static final double MAX_NORMALIZED_SCORE = 1.0;

  protected final File imageFilePath;
  protected Integer score = null;
  protected Double normalizedScore = null;
  protected Integer spotTotal = null;
  protected Integer inResTotal = null;
  protected Integer goodBraggTotal = null;
  protected Integer iceRingTotal = null;
  protected Double estimatedRes1 = null;
  protected Double estimatedRes2 = null;
  protected Integer totalIntSignal = null;
  protected Double mosaicity = null;
  protected SpaceGroup spaceGroup = null;
  protected UnitCell unitCell = null;
  protected int column = 0;
  protected int row = 0;
  protected Position2d position = null;
  protected boolean acquiring = false;
  protected boolean acquired = false;

  public RasterCell(int column, int row, Position2d position, File imageFilePath,
      boolean firePropertyChangesOnEdtEnabled) {
    super(firePropertyChangesOnEdtEnabled);
    this.column = column;
    this.row = row;
    this.position = position;
    this.imageFilePath = imageFilePath;
  }

  public void reset() {
    setScore(null);
    setNormalizedScore(null);
    setSpotTotal(null);
    setInResolutionTotal(null);
    setGoodBraggTotal(null);
    setIceRingTotal(null);
    setEstimatedResolution1(null);
    setEstimatedResolution2(null);
    setTotalIntegratedSignal(null);
    setMosaicity(null);
    setSpaceGroup(null);
    setUnitCell(null);
    setPosition(0.0, 0.0);
    setAcquiring(false);
    setAcquired(false);
  }

  public File getImageFilePath() {
    return imageFilePath;
  }

  public synchronized Integer getScore() {
    return score;
  }

  public synchronized void setScore(Integer newScore) {
    if (Objects.equals(score, newScore)) return;
    Integer oldScore = score;
    score = newScore;
    notifyChanged(PROPNAME_SCORE, oldScore, newScore);
  }

  public synchronized Double getNormalizedScore() {
    return normalizedScore;
  }

  public synchronized void setNormalizedScore(Double newNormalizedScore) {
    if (Objects.equals(normalizedScore, newNormalizedScore)) return;
    if (newNormalizedScore != null &&
        (newNormalizedScore < MIN_NORMALIZED_SCORE || newNormalizedScore > MAX_NORMALIZED_SCORE)) {
      throw new IllegalArgumentException("Normalized score out of range");
    }
    Double oldNormalizedScore = normalizedScore;
    normalizedScore = newNormalizedScore;
    notifyChanged(PROPNAME_NORMALIZED_SCORE, oldNormalizedScore, newNormalizedScore);
  }

  public synchronized Integer getSpotTotal() {
    return spotTotal;
  }

  public synchronized void setSpotTotal(Integer newSpotTotal) {
    if (Objects.equals(spotTotal, newSpotTotal)) return;
    Integer oldSpotTotal = spotTotal;
    spotTotal = newSpotTotal;
    notifyChanged(PROPNAME_SPOT_TOTAL, oldSpotTotal, newSpotTotal);
  }

  public synchronized Integer getInResolutionTotal() {
    return inResTotal;
  }

  public synchronized void setInResolutionTotal(Integer newInResTotal) {
    if (Objects.equals(inResTotal, newInResTotal)) return;
    Integer oldInResTotal = inResTotal;
    inResTotal = newInResTotal;
    notifyChanged(PROPNAME_IN_RES_TOTAL, oldInResTotal, newInResTotal);
  }

  public synchronized Integer getGoodBraggTotal() {
    return goodBraggTotal;
  }

  public synchronized void setGoodBraggTotal(Integer newGoodBraggTotal) {
    if (Objects.equals(goodBraggTotal, newGoodBraggTotal)) return;
    Integer oldGoodBraggTotal = goodBraggTotal;
    goodBraggTotal = newGoodBraggTotal;
    notifyChanged(PROPNAME_GOOD_BRAGG_TOTAL, oldGoodBraggTotal, newGoodBraggTotal);
  }

  public synchronized Integer getIceRingTotal() {
    return iceRingTotal;
  }

  public synchronized void setIceRingTotal(Integer newIceRingTotal) {
    if (Objects.equals(iceRingTotal, newIceRingTotal)) return;
    Integer oldIceRingTotal = iceRingTotal;
    iceRingTotal = newIceRingTotal;
    notifyChanged(PROPNAME_ICE_RING_TOTAL, oldIceRingTotal, newIceRingTotal);
  }

  public synchronized Double getEstimatedResolution1() {
    return estimatedRes1;
  }

  public synchronized void setEstimatedResolution1(Double newEstimatedRes1) {
    if (Objects.equals(estimatedRes1, newEstimatedRes1)) return;
    Double oldEstimatedRes1 = estimatedRes1;
    estimatedRes1 = newEstimatedRes1;
    notifyChanged(PROPNAME_ESTIMATED_RES1, oldEstimatedRes1, newEstimatedRes1);
  }

  public synchronized Double getEstimatedResolution2() {
    return estimatedRes2;
  }

  public synchronized void setEstimatedResolution2(Double newEstimatedRes2) {
    if (Objects.equals(estimatedRes2, newEstimatedRes2)) return;
    Double oldEstimatedRes2 = estimatedRes2;
    estimatedRes2 = newEstimatedRes2;
    notifyChanged(PROPNAME_ESTIMATED_RES2, oldEstimatedRes2, newEstimatedRes2);
  }

  public synchronized Integer getTotalIntegratedSignal() {
    return totalIntSignal;
  }

  public synchronized void setTotalIntegratedSignal(Integer newTotalIntSignal) {
    if (Objects.equals(totalIntSignal, newTotalIntSignal)) return;
    Integer oldTotalIntSignal = totalIntSignal;
    totalIntSignal = newTotalIntSignal;
    notifyChanged(PROPNAME_TOTAL_INT_SIGNAL, oldTotalIntSignal, newTotalIntSignal);
  }

  public synchronized Double getMosaicity() {
    return mosaicity;
  }

  public synchronized void setMosaicity(Double newMosaicity) {
    if (Objects.equals(mosaicity, newMosaicity)) return;
    Double oldMosaicity = mosaicity;
    mosaicity = newMosaicity;
    notifyChanged(PROPNAME_MOSAICITY, oldMosaicity, newMosaicity);
  }

  public synchronized SpaceGroup getSpaceGroup() {
    return spaceGroup;
  }

  public synchronized void setSpaceGroup(SpaceGroup newSpaceGroup) {
    if (Objects.equals(spaceGroup, newSpaceGroup)) return;
    SpaceGroup oldSpaceGroup = spaceGroup;
    spaceGroup = newSpaceGroup;
    notifyChanged(PROPNAME_SPACE_GROUP, oldSpaceGroup, newSpaceGroup);
  }

  public synchronized UnitCell getUnitCell() {
    return unitCell;
  }

  public synchronized void setUnitCell(UnitCell newUnitCell) {
    if (Objects.equals(unitCell, newUnitCell)) return;
    UnitCell oldUnitCell = unitCell;
    unitCell = newUnitCell;
    notifyChanged(PROPNAME_UNIT_CELL, oldUnitCell, newUnitCell);
  }

  public int getColumn() {
    return column;
  }

  public int getRow() {
    return row;
  }

  public Raster2dIndex getIndex() {
    return new Raster2dIndex(column, row);
  }

  public Position2d getPosition() {
    return position;
  }

  public double getHorizontalPosition() {
    return position.getX();
  }

  public double getVerticalPosition() {
    return position.getY();
  }

  public double getPosition(Direction2d d) {
    return d.isHorizontal() ? getHorizontalPosition() : getVerticalPosition();
  }

  public void setPosition(double horizontal, double vertical) {
    setPosition(new Position2d(horizontal, vertical, position.getUnit()));
  }

  public void setPosition(Position2d newPosition) {
    if (position == null && newPosition == null) return;
    if (position != null && position.equals(newPosition)) return;
    Position2d oldPosition = position;
    position = newPosition;
    notifyChanged(PROPNAME_POSITION, oldPosition, newPosition);
  }

  public synchronized boolean isAcquiring() {
    return acquiring;
  }

  public synchronized void setAcquiring(boolean newAcquiring) {
    if (acquiring == newAcquiring) return;
    boolean oldAcquiring = acquiring;
    acquiring = newAcquiring;
    notifyChanged(PROPNAME_ACQUIRING, oldAcquiring, newAcquiring);
    if (newAcquiring == false) setAcquired(true);
  }

  public synchronized boolean isAcquired() {
    return acquired;
  }

  protected synchronized void setAcquired(boolean newAcquired) {
    if (acquired == newAcquired) return;
    boolean oldAcquired = acquired;
    acquired = newAcquired;
    notifyChanged(PROPNAME_ACQUIRED, oldAcquired, newAcquired);
  }

  @Override
  public String toString() {
    return (score == null) ? "" : score.toString();
  }
}
