package imcacat.imcadr;

import imcacat.imcadr.util.ColorUtils;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

/**
 * I display a {@link RasterCell}.
 */
public class RasterCellLabel extends JLabel {
  private static final long serialVersionUID = 1;

  protected static final Border LIGHT_GRAY_BORDER = BorderFactory.createCompoundBorder(
      BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1),
      BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1));
  protected static final Border YELLOW_BORDER = BorderFactory.createCompoundBorder(
      BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1),
      BorderFactory.createLineBorder(Color.YELLOW, 1));
  protected static final Border BLACK_BORDER = BorderFactory.createCompoundBorder(
      BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1),
      BorderFactory.createLineBorder(Color.BLACK, 1));

  public RasterCellLabel(final RasterCellPresentationModel model) {
    super();
    setHorizontalAlignment(SwingConstants.CENTER);
    setVerticalAlignment(SwingConstants.CENTER);
    setForeground(Color.WHITE);
    setOpaque(true);
    setBorder(LIGHT_GRAY_BORDER);

    RasterCell cell = model.getBean();
    updateForScore(cell.getScore());
    updateForNormalizedScore(cell.getNormalizedScore());

    model.getModel(RasterCell.PROPNAME_SCORE).addValueChangeListener(
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent event) {
            updateForScore((Integer)event.getNewValue());
          }
        });

    model.getModel(RasterCell.PROPNAME_NORMALIZED_SCORE).addValueChangeListener(
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent event) {
            updateForNormalizedScore((Double)event.getNewValue());
          }
        });

    model.getModel(RasterCell.PROPNAME_ACQUIRING).addValueChangeListener(
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent event) {
            setBorder((Boolean)event.getNewValue() ? YELLOW_BORDER : BLACK_BORDER);
          }
        });

    JPopupMenu popupMenu = new JPopupMenu();
    JMenuItem openImageMenuItem =
        new JMenuItem("Open Image in " +
            Resources.getInstance().getDisplayImageFileProgramName());
    openImageMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        model.openImageMenuItemSelected();
      }
    });
    popupMenu.add(openImageMenuItem);
    setComponentPopupMenu(popupMenu);

    addMouseListener(new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent event) {
        if (event.getButton() != MouseEvent.BUTTON1) return;
        if (event.getClickCount() != 2) return;
        model.mouseDoubleClicked();
      }

      @Override
      public void mouseEntered(MouseEvent event) {
        model.mouseEntered();
      }

      @Override
      public void mouseExited(MouseEvent event) {
        model.mouseExited();
      }
    });
  }

  protected void updateForScore(Integer score) {
    setText(printScoreAsString(score));
  }

  protected void updateForNormalizedScore(Double normalizedScore) {
    setBackground(computeColorForScore(normalizedScore));
  }

  protected static String printScoreAsString(Integer score) {
    if (score == null) return "";
    return format("%d", score);
  }

  protected static Color computeColorForScore(Double normalizedScore) {
    return ColorUtils.getTemperature(normalizedScore, RasterCell.MIN_NORMALIZED_SCORE,
        RasterCell.MAX_NORMALIZED_SCORE);
  }

  protected static String format(String format, Object... args) {
    return String.format(Locale.US, format, args);
  }
}
