package imcacat.imcadr;

import imcacat.procrun.NullInputStreamConsumer;
import imcacat.procrun.ProcessRunner;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link RasterCell}.
 */
public class RasterCellPresentationModel extends PresentationModel<RasterCell> {
  private static final long serialVersionUID = 1;

  protected ValueModel mouseOverString;

  public RasterCellPresentationModel(RasterCell bean) {
    this(bean, new ValueHolder(""));
  }

  public RasterCellPresentationModel(RasterCell bean, ValueModel mouseOverString) {
    super(bean);
    this.mouseOverString = mouseOverString;
  }

  public void openImageMenuItemSelected() {
    openImage();
  }

  public void mouseDoubleClicked() {
    openImage();
  }

  public void mouseEntered() {
    mouseOverString.setValue("Cell: (" + (getBean().getColumn() + 1) + ", " +
        (getBean().getRow() + 1) + ") " + "Position: (" +
        getBean().getPosition().toDisplayString(3) + ")");
  }

  public void mouseExited() {
    mouseOverString.setValue("");
  }

  protected void openImage() {
    File image = getBean().getImageFilePath();
    if (image == null) return;
    if (!image.isFile()) return;
    if (!image.canRead()) return;
    ProcessRunner runner = createOpenImageProcessRunner(image);
    runner.setDaemon(true);
    runner.start();
  }

  protected static ProcessRunner createOpenImageProcessRunner(File image) {
    ProcessRunner result = new ProcessRunner(createOpenImageProcessBuilder(image));
    result.stdoutConsumer(new NullInputStreamConsumer(1024));
    result.stderrConsumer(new NullInputStreamConsumer(1024));
    return result;
  }

  protected static ProcessBuilder createOpenImageProcessBuilder(File image) {
    ProcessBuilder result = new ProcessBuilder(createOpenImageCommand(image));
    result.directory(image.getParentFile().getAbsoluteFile());
    return result;
  }

  protected static List<String> createOpenImageCommand(File image) {
    ArrayList<String> result = new ArrayList<String>(2);
    result.add(Resources.getInstance().getDisplayImageFileCommand());
    result.add(image.getAbsolutePath());
    return result;
  }
}
