package imcacat.imcadr;

import imcacat.imcadr.util.GeometryUtils;

import java.awt.BorderLayout;
import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * I display a {@link Rasterizer2d}.
 */
public class Rasterizer2dPanel extends JPanel {
  private static final long serialVersionUID = 1;

  protected static final double MIN_CELL_WIDTH_IN_DLU = 20.0;
  protected static final double MIN_CELL_HEIGHT_IN_DLU = 14.0;

  protected Rasterizer2dPresentationModel model;

  public Rasterizer2dPanel(Rasterizer2dPresentationModel model) {
    super(new BorderLayout());
    this.model = model;

    model.addPropertyChangeListener(PresentationModel.PROPERTYNAME_BEAN,
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent event) {
            rasterizerChanged((Rasterizer2d)event.getNewValue());
          }
        });
    rasterizerChanged(model.getBean());
  }

  protected void rasterizerChanged(Rasterizer2d newRasterizer) {
    removeAll();
    add(build(newRasterizer), BorderLayout.CENTER);
  }

  protected JPanel build(Rasterizer2d rasterizer) {
    Raster2d raster = rasterizer.getRaster();
    Rectangle cellSizeInEm =
        GeometryUtils.scale(raster.getCellWidth(), raster.getCellHeight(),
            MIN_CELL_WIDTH_IN_DLU, MIN_CELL_HEIGHT_IN_DLU);
    int widthInCells = raster.columnCount();
    int heightInCells = raster.rowCount();
    ColumnSpec[] colSpecs = new ColumnSpec[widthInCells];
    for (int i = 0; i < colSpecs.length; i++)
      colSpecs[i] =
          ColumnSpec.decode(String.format("fill:%.0fdlu:none", cellSizeInEm.getWidth()));
    RowSpec[] rowSpecs = new RowSpec[heightInCells];
    for (int i = 0; i < rowSpecs.length; i++)
      rowSpecs[i] = RowSpec.decode(String.format("fill:%.0fdlu:none", cellSizeInEm.getHeight()));
    FormLayout layout = new FormLayout(colSpecs, rowSpecs);

    int[] colGroup = new int[widthInCells];
    for (int i = 0; i < colGroup.length; i++)
      colGroup[i] = i + 1;
    layout.setColumnGroups(new int[][] {colGroup});
    int[] rowGroup = new int[heightInCells];
    for (int i = 0; i < rowGroup.length; i++)
      rowGroup[i] = i + 1;
    layout.setRowGroups(new int[][] {rowGroup});

    PanelBuilder builder = new PanelBuilder(layout);
    CellConstraints cc = new CellConstraints();

    for (int col : colGroup) {
      for (int row : rowGroup) {
        builder.add(new RasterCellLabel(new RasterCellPresentationModel(rasterizer.getRaster()
            .getCell(col - 1, row - 1), model.getMouseOverStringModel())), cc.xy(col, row));
      }
    }

    JPanel panel = builder.getPanel();
    panel.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createLineBorder(Color.GRAY, 1),
        BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1)));
    return panel;
  }
}
