package imcacat.imcadr;

import java.io.PrintWriter;

/**
 * I print a <code>Rasterizer3d</code> to a file as HTML.
 */
public class Rasterizer3dPrinter extends Rasterizer2dPrinter {
  public Rasterizer3dPrinter(PrintWriter writer) {
    super(writer);
  }

  public void print(Rasterizer3d rasterizer) {
    printHeader();
    printWarningsIfAny(rasterizer);
    printRasterizer(rasterizer);
    printDivider();
    printDiagnostics(rasterizer);
    printFooter();
  }

  protected void printWarningsIfAny(Rasterizer3d rasterizer) {
    if (rasterizer.wasSuccessful() && !rasterizer.getZyRasterizer().getRaster().isClipped() &&
        !rasterizer.getZxRasterizer().getRaster().isClipped()) {
      return;
    }
    printWarningHeader();
    if (!rasterizer.wasSuccessful()) {
      printUnsuccessfulWarning(rasterizer);
    }
    Raster2dPlane[] planes;
    if (rasterizer.isZxFirst()) {
      planes = new Raster2dPlane[] {
          Raster2dPlane.ZX,
          Raster2dPlane.ZY};
    } else {
      planes = new Raster2dPlane[] {
          Raster2dPlane.ZY,
          Raster2dPlane.ZX};
    }
    for (Raster2dPlane each : planes) {
      if (rasterizer.getRasterizer(each).getRaster().isClipped()) {
        printClippedWarning(rasterizer.getRasterizer(each).getRaster());
      }
    }
    printWarningFooter();
  }

  protected void printRasterizer(Rasterizer3d rasterizer) {
    if (rasterizer.isZxFirst()) {
      printRasterizer(rasterizer.getZxRasterizer());
      printRasterizer(rasterizer.getZyRasterizer());
    } else {
      printRasterizer(rasterizer.getZyRasterizer());
      printRasterizer(rasterizer.getZxRasterizer());
    }
    printSolution(rasterizer);
  }

  protected void printSolution(Rasterizer3d rasterizer) {
    writer.println();
    writer.println("<h1>Solution</h1>");
    writer.println();
    writer.println("<table>");
    writer.println("  <tr>");
    writer.println("    <td>Center (X, Y, Z):</td>");
    writer.format("    <td>%s</td>%n", escape(rasterizer.getBestPositionString()));
    writer.println("  </tr>");
    writer.println("  <tr>");
    writer.println("    <td>Time (total, acquire, score):</td>");
    writer.format("    <td>%s</td>%n", escape(rasterizer.getTimeString()));
    writer.println("  </tr>");
    writer.println("</table>");
  }

  protected void printDiagnostics(Rasterizer3d rasterizer) {
    printDiagnosticsHeader();
    printDiagnosticsFor(rasterizer);
    printDiagnosticsFooter();
  }

  protected void printDiagnosticsFor(Rasterizer3d rasterizer) {
    if (rasterizer.isZxFirst()) {
      printDiagnosticsFor(rasterizer.getZxRasterizer());
      printDiagnosticsFor(rasterizer.getZyRasterizer());
    } else {
      printDiagnosticsFor(rasterizer.getZyRasterizer());
      printDiagnosticsFor(rasterizer.getZxRasterizer());
    }
  }
}
