package imcacat.imcadr;

import java.util.Arrays;
import java.util.Locale;

/**
 * I model a crystallography unit cell.
 */
public class UnitCell {
  private final double[] lengths;
  private final double[] angles;

  public UnitCell(double l1, double l2, double l3, double a1, double a2, double a3) {
    /* @formatter:off */
    this(new double[] {l1, l2, l3}, new double[] {a1, a2, a3});
    /* @formatter:on */
  }

  public UnitCell(double[] lengths, double[] angles) {
    super();
    if (lengths.length != 3) {
      throw new IllegalArgumentException("lengths must be of length 3");
    }
    if (angles.length != 3) {
      throw new IllegalArgumentException("angles must be of length 3");
    }
    this.lengths = lengths;
    this.angles = angles;
  }

  public double[] getLengths() {
    return lengths;
  }

  public double[] getAngles() {
    return angles;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) return false;
    if (!(obj instanceof UnitCell)) return false;
    UnitCell objUnitCell = (UnitCell)obj;
    if (!Arrays.equals(lengths, objUnitCell.lengths)) return false;
    return Arrays.equals(angles, objUnitCell.angles);
  }

  @Override
  public int hashCode() {
    return Double.valueOf(lengths[0]).hashCode() ^ Double.valueOf(lengths[1]).hashCode() ^
        Double.valueOf(lengths[2]).hashCode() ^ Double.valueOf(angles[0]).hashCode() ^
        Double.valueOf(angles[1]).hashCode() ^ Double.valueOf(angles[2]).hashCode();
  }

  @Override
  public String toString() {
    return String.format(Locale.US, "UnitCell([%f, %f, %f], [%f, %f, %f])", lengths[0],
        lengths[1], lengths[2], angles[0], angles[1], angles[2]);
  }
}
