package imcacat.imcadr.util;

import java.math.BigDecimal;
import java.math.BigInteger;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;

/**
 * I provide utilities related to durations of time.
 */
public class DurationUtils {
  private DurationUtils() {
    throw new AssertionError();
  }

  public static String createTimeString(long millis) {
    long millisAsSeconds = millis / 1000L;
    if ((millisAsSeconds * 1000L) != millis) millisAsSeconds++;
    long millisRoundedUpToNearestSecond = millisAsSeconds * 1000L;
    Duration duration;
    try {
      duration = DatatypeFactory.newInstance().newDuration(millisRoundedUpToNearestSecond);
    } catch (DatatypeConfigurationException e) {
      throw new RuntimeException(e);
    }
    String result = "";
    boolean foundNonzero = false;
    Number years = duration.getField(DatatypeConstants.YEARS);
    if (years == null) years = BigInteger.ZERO;
    if (!BigInteger.ZERO.equals(years)) foundNonzero = true;
    if (foundNonzero) result += years + "y";
    Number months = duration.getField(DatatypeConstants.MONTHS);
    if (months == null) months = BigInteger.ZERO;
    if (!BigInteger.ZERO.equals(months)) foundNonzero = true;
    if (foundNonzero) result += months + "mo";
    Number days = duration.getField(DatatypeConstants.DAYS);
    if (days == null) days = BigInteger.ZERO;
    if (!BigInteger.ZERO.equals(days)) foundNonzero = true;
    if (foundNonzero) result += days + "d";
    Number hours = duration.getField(DatatypeConstants.HOURS);
    if (hours == null) hours = BigInteger.ZERO;
    if (!BigInteger.ZERO.equals(hours)) foundNonzero = true;
    if (foundNonzero) result += hours + "h";
    Number minutes = duration.getField(DatatypeConstants.MINUTES);
    if (minutes == null) minutes = BigInteger.ZERO;
    if (!BigInteger.ZERO.equals(minutes)) foundNonzero = true;
    if (foundNonzero) result += minutes + "m";
    Number seconds = duration.getField(DatatypeConstants.SECONDS);
    if (seconds == null) seconds = BigDecimal.ZERO;
    result += seconds.intValue() + "s";
    return result;
  }

  public static double millisToSeconds(long ms) {
    return ((double)ms) / 1e3;
  }

  public static long secondsToMillis(double s) {
    return (long)(s * 1e3);
  }
}
