package imcacat.imcadr.util;

import imcacat.imcadr.Rectangle;

/**
 * I provide utility methods related to geometry.
 */
public class GeometryUtils {
  private GeometryUtils() {
    throw new AssertionError();
  }

  /**
   * Scales a rectangle to the size of a target rectangle, but with no side
   * smaller than the corresponding side of the target rectangle, while
   * preserving the aspect ratio.
   */
  public static Rectangle scale(Rectangle unscaled, Rectangle target) {
    return scale(unscaled.getWidth(), unscaled.getHeight(), target.getWidth(),
        target.getHeight());
  }

  /**
   * Scales a rectangle to the size of a target rectangle, but with no side
   * smaller than the corresponding side of the target rectangle, while
   * preserving the aspect ratio.
   */
  public static Rectangle scale(double unscaledWidth, double unscaledHeight, double targetWidth,
      double targetHeight) {
    double heightToWidthRatio = unscaledHeight / unscaledWidth;
    double scaledHeight = targetHeight;
    double scaledWidth = scaledHeight / heightToWidthRatio;
    if (scaledWidth < targetWidth) {
      double scale = targetWidth / scaledWidth;
      scaledWidth = targetWidth;
      scaledHeight = scaledHeight * scale;
    }
    return new Rectangle(scaledWidth, scaledHeight);
  }
}
