package imcacat.imcadr.util;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * I provide utility methods related to computer networks.
 */
public class NetworkUtils {
  public static final String LOCALHOST_NAME = "localhost";

  private static final Set<String> COMMON_LOCALHOST_NAMES_AND_ADDRS = Collections
      .unmodifiableSet(new HashSet<>(Arrays.asList(LOCALHOST_NAME, "127.0.0.1", "::1")));
  private static final Pattern IPV4_STRING_LITERAL = Pattern
      .compile("(\\d{1,3}+)\\.(\\d{1,3}+)\\.(\\d{1,3}+)\\.(\\d{1,3}+)");

  private NetworkUtils() {
    throw new AssertionError();
  }

  /**
   * Answers whether the specified hostname refers to the loopback network
   * interface of the computer on which the Java VM is running. This method
   * does not query DNS.
   *
   * @param hostname the computer hostname; may be a computer name or an IP
   *          address literal
   *
   * @return {@code true} if {@code hostname} refers to this computer's
   *         loopback interface; {@code false} otherwise
   */
  public static boolean isLocalhost(String hostname) {
    if (hostname == null) return false;
    if (hostname.isEmpty()) return false;
    if (COMMON_LOCALHOST_NAMES_AND_ADDRS.contains(hostname)) return true;
    Matcher m = IPV4_STRING_LITERAL.matcher(hostname);
    if (!m.matches()) return false;
    if (!m.group(1).equals("127")) return false;
    try {
      return InetAddress.getByName(hostname).isLoopbackAddress();
    } catch (UnknownHostException e) {
      return false;
    }
  }
}
