package imcacat.imcadr.util;

import java.util.Locale;

/**
 * I provide utilities related to numbers.
 */
public class NumberUtils {
  private NumberUtils() {
    throw new AssertionError();
  }

  public static String toString(Double value, int precision, double scientificNotationFloor,
      int maxCharacterCount) {
    if (value == null) return "";
    if (value < scientificNotationFloor) {
      return toDecimalNotationString(value, precision, maxCharacterCount);
    } else {
      return toScientificNotationString(value, precision, maxCharacterCount);
    }
  }

  public static String toDecimalNotationString(Double value, int precision, int maxCharacterCount) {
    if (value == null) return "";
    String format = "%." + precision + "f";
    String result = format(format, value);
    if (result.length() <= maxCharacterCount) return result;
    if (precision == 0) return result;
    int reducedPrecision = Math.max(0, precision - (result.length() - maxCharacterCount));
    return toDecimalNotationString(value, reducedPrecision, maxCharacterCount);
  }

  public static String toScientificNotationString(Double value, int precision,
      int maxCharacterCount) {
    if (value == null) return "";
    String format = "%." + precision + "e";
    String result =
        format(format, value).replace("e+0", "e").replace("e-0", "e-").replace("+", "");
    if (result.length() <= maxCharacterCount) return result;
    if (precision == 0) return result;
    int reducedPrecision = Math.max(0, precision - (result.length() - maxCharacterCount));
    return toScientificNotationString(value, reducedPrecision, maxCharacterCount);
  }

  private static String format(String format, Object... args) {
    return String.format(Locale.US, format, args);
  }
}
