package imcacat.imcadr.util;

/**
 * I provide utility methods related to physics.
 * <p>
 * The constants I use for converting between eV and Angstroms are from <a
 * href="http://physics.nist.gov/cuu/Constants/">2006 CODATA recommended
 * values</a>.
 */
public class PhysicsUtils {
  /* Planck's constant in eV s */
  protected static final double H = 4.13566733e-15;
  /* speed of light in vacuum in m/s */
  protected static final double C = 2.99792458e8;
  protected static final double ANGSTROMS_IN_METER = 1e10;

  private PhysicsUtils() {
    throw new AssertionError();
  }

  /**
   * Converts wavelength in Angstroms to energy in eV.
   */
  public static double convertToEnergy(double wavelength) {
    return H * C * ANGSTROMS_IN_METER / wavelength;
  }

  /**
   * Converts energy in eV to wavelength in Angstroms.
   */
  public static double convertToWavelength(double energy) {
    return H * C * ANGSTROMS_IN_METER / energy;
  }
}
