package imcacat.imcadr.util;

/**
 * I provide additional {@link String} methods.
 */
public class StringUtils {
  private StringUtils() {
    throw new AssertionError();
  }

  /**
   * Joins specified strings separated by the specified separator.
   */
  public static String join(String sep, String... a) {
    if (a.length == 0) return "";
    StringBuilder b = new StringBuilder();
    for (String each : a) {
      b.append(each);
      b.append(sep);
    }
    b.delete(b.length() - sep.length(), b.length());
    return b.toString();
  }

  /**
   * Converts the first character in the specified string to a lowercase
   * character if no other characters in the first word are uppercase.
   *
   * @param s string to make not capitalized
   *
   * @return an uncapitalized string or the specified string if no conversion
   *         was performed
   */
  public static String uncapitalize(String s) {
    /*
     * NOTE: This is a verbatim copy of JavaUtilities.uncapitalize(String)
     * in Prgm.  Any changes should be made there first since it is the
     * authoritative implementation and includes test cases.
     */
    if (s == null) return s;
    if (s.isEmpty()) return s;
    int firstCodePoint = s.codePointAt(0);
    if (!Character.isUpperCase(firstCodePoint)) return s;
    int secondCodePointIndex = Character.charCount(firstCodePoint);
    for (int eachCharCount, i = secondCodePointIndex; i < s.length(); i += eachCharCount) {
      int each = s.codePointAt(i);
      eachCharCount = Character.charCount(each);
      if (Character.isUpperCase(each)) return s;
      if (Character.isWhitespace(each)) break;
    }
    String old = s.substring(0, secondCodePointIndex);
    StringBuilder result = new StringBuilder(s);
    return result.replace(0, old.length(), old.toLowerCase()).toString();
  }
}
